@@ -14,48 +14,11 @@
 
 "use strict";
 
-requestLongerTimeout(5);
-
-/* import-globals-from ../../../framework/browser-toolbox/test/helpers-browser-toolbox.js */
-Services.scriptloader.loadSubScript(
-  "chrome://mochitests/content/browser/devtools/client/framework/browser-toolbox/test/helpers-browser-toolbox.js",
-  this
-);
-
 const testServer = createVersionizedHttpTestServer(
   "examples/sourcemaps-reload-uncompressed"
 );
 const TEST_URL = testServer.urlFor("index.html");
 
-const INTEGRATION_TEST_PAGE_SOURCES = [
-  "index.html",
-  "iframe.html",
-  "script.js",
-  "onload.js",
-  "test-functions.js",
-  "query.js?x=1",
-  "query.js?x=2",
-  "bundle.js",
-  "original.js",
-  "bundle-with-another-original.js",
-  "original-with-no-update.js",
-  "replaced-bundle.js",
-  "removed-original.js",
-  "named-eval.js",
-  // Webpack generated some extra sources:
-  "bootstrap 3b1a221408fdde86aa49",
-  "bootstrap a1ecee2f86e1d0ea3fb5",
-  "bootstrap 6fda1f7ea9ecbc1a2d5b",
-  // There is 3 occurences, one per target (main thread, worker and iframe).
-  // But there is even more source actors (named evals and duplicated script tags).
-  "same-url.sjs",
-  "same-url.sjs",
-];
-// The iframe one is only available when fission is enabled, or EFT
-if (isFissionEnabled() || isEveryFrameTargetEnabled()) {
-  INTEGRATION_TEST_PAGE_SOURCES.push("same-url.sjs");
-}
-
 /**
  * This test opens the SourceTree manually via click events on the nested source,
  * and then adds a source dynamically and asserts it is visible.
@@ -70,7 +33,11 @@ add_task(async function testSimpleSourcesWithManualClickExpand() {
   );
 
   // Expand nodes and make sure more sources appear.
-  is(getLabel(dbg, 1), "Main Thread", "Main thread is labeled properly");
+  is(
+    getSourceTreeLabel(dbg, 1),
+    "Main Thread",
+    "Main thread is labeled properly"
+  );
   info("Before interacting with the source tree, no source are displayed");
   await waitForSourcesInSourceTree(dbg, [], { noExpand: true });
   await clickElement(dbg, "sourceDirectoryLabel", 3);
@@ -112,7 +79,7 @@ add_task(async function testSimpleSourcesWithManualClickExpand() {
   await assertNodeIsFocused(dbg, 5);
 
   // Make sure new sources appear in the list.
-  await SpecialPowers.spawn(gBrowser.selectedBrowser, [], function() {
+  await SpecialPowers.spawn(gBrowser.selectedBrowser, [], function () {
     const script = content.document.createElement("script");
     script.src = "math.min.js";
     content.document.body.appendChild(script);
@@ -139,6 +106,55 @@ add_task(async function testSimpleSourcesWithManualClickExpand() {
 
   info("Assert that nested-source.js is still the selected source");
   await assertNodeIsFocused(dbg, 5);
+
+  info("Test the copy to clipboard context menu");
+  const mathMinTreeNode = findSourceNodeWithText(dbg, "math.min.js");
+  await triggerSourceTreeContextMenu(
+    dbg,
+    mathMinTreeNode,
+    "#node-menu-copy-source"
+  );
+  const clipboardData = SpecialPowers.getClipboardData("text/plain");
+  is(
+    clipboardData,
+    EXAMPLE_URL + "math.min.js",
+    "The clipboard content is the selected source URL"
+  );
+
+  info("Test the download file context menu");
+  // Before trigerring the menu, mock the file picker
+  const MockFilePicker = SpecialPowers.MockFilePicker;
+  MockFilePicker.init(window);
+  const nsiFile = FileUtils.getFile("TmpD", [
+    `export_source_content_${Date.now()}.log`,
+  ]);
+  MockFilePicker.setFiles([nsiFile]);
+  const path = nsiFile.path;
+
+  await triggerSourceTreeContextMenu(
+    dbg,
+    mathMinTreeNode,
+    "#node-menu-download-file"
+  );
+
+  info("Wait for the downloaded file to be fully saved to disk");
+  await BrowserTestUtils.waitForCondition(() => IOUtils.exists(path));
+  await BrowserTestUtils.waitForCondition(async () => {
+    const { size } = await IOUtils.stat(path);
+    return size > 0;
+  });
+  const buffer = await IOUtils.read(path);
+  const savedFileContent = new TextDecoder().decode(buffer);
+
+  const mathMinRequest = await fetch(EXAMPLE_URL + "math.min.js");
+  const mathMinContent = await mathMinRequest.text();
+
+  is(
+    savedFileContent,
+    mathMinContent,
+    "The downloaded file has the expected content"
+  );
+
   dbg.toolbox.closeToolbox();
 });
 
@@ -273,6 +289,7 @@ add_task(async function testSourceTreeOnTheIntegrationTestPage() {
     "test-functions.js",
     "query.js?x=1",
     "query.js?x=2",
+    "query2.js?y=3",
     "bundle.js",
     "original.js",
     "replaced-bundle.js",
@@ -280,55 +297,85 @@ add_task(async function testSourceTreeOnTheIntegrationTestPage() {
     "named-eval.js"
   );
 
+  info("Verify source tree content");
   await waitForSourcesInSourceTree(dbg, INTEGRATION_TEST_PAGE_SOURCES);
 
+  info("Verify Thread Source Items");
+  const mainThreadItem = findSourceTreeThreadByName(dbg, "Main Thread");
+  ok(mainThreadItem, "Found the thread item for the main thread");
+  ok(
+    mainThreadItem.querySelector("span.img.window"),
+    "The thread has the window icon"
+  );
+
   info(
     "Assert the number of sources and source actors for the same-url.sjs sources"
   );
-  const mainThreadSameUrlSource = findSourceInThread(
-    dbg,
-    "same-url.sjs",
-    "Main Thread"
-  );
-  ok(mainThreadSameUrlSource, "Found same-url.js in the main thread");
+  const sameUrlSource = findSource(dbg, "same-url.sjs");
+  ok(sameUrlSource, "Found same-url.js in the main thread");
+
+  const sourceActors = dbg.selectors.getSourceActorsForSource(sameUrlSource.id);
+
+  const mainThread = dbg.selectors
+    .getAllThreads()
+    .find(thread => thread.name == "Main Thread");
+
   is(
-    dbg.selectors.getSourceActorsForSource(mainThreadSameUrlSource.id).length,
+    sourceActors.filter(actor => actor.thread == mainThread.actor).length,
     // When EFT is disabled the iframe's source is meld into the main target
     isEveryFrameTargetEnabled() ? 3 : 4,
     "same-url.js is loaded 3 times in the main thread"
   );
 
-  const iframeSameUrlSource = findSourceInThread(
-    dbg,
-    "same-url.sjs",
-    testServer.urlFor("iframe.html")
-  );
   if (isEveryFrameTargetEnabled()) {
-    ok(iframeSameUrlSource, "Found same-url.js in the iframe thread");
+    const iframeThread = dbg.selectors
+      .getAllThreads()
+      .find(thread => thread.name == testServer.urlFor("iframe.html"));
+
     is(
-      dbg.selectors.getSourceActorsForSource(iframeSameUrlSource.id).length,
+      sourceActors.filter(actor => actor.thread == iframeThread.actor).length,
       1,
       "same-url.js is loaded one time in the iframe thread"
     );
-  } else {
-    ok(
-      !iframeSameUrlSource,
-      "When EFT is off, the iframe source is into the main thread bucket"
-    );
   }
 
-  const workerSameUrlSource = findSourceInThread(
-    dbg,
-    "same-url.sjs",
-    "same-url.sjs"
-  );
-  ok(workerSameUrlSource, "Found same-url.js in the worker thread");
+  const workerThread = dbg.selectors
+    .getAllThreads()
+    .find(thread => thread.name == testServer.urlFor("same-url.sjs"));
+
   is(
-    dbg.selectors.getSourceActorsForSource(workerSameUrlSource.id).length,
+    sourceActors.filter(actor => actor.thread == workerThread.actor).length,
     1,
     "same-url.js is loaded one time in the worker thread"
   );
 
+  const workerThreadItem = findSourceTreeThreadByName(dbg, "same-url.sjs");
+  ok(workerThreadItem, "Found the thread item for the worker");
+  ok(
+    workerThreadItem.querySelector("span.img.worker"),
+    "The thread has the worker icon"
+  );
+
+  info("Verify source icons");
+  assertSourceIcon(dbg, "index.html", "file");
+  assertSourceIcon(dbg, "script.js", "javascript");
+  assertSourceIcon(dbg, "query.js?x=1", "javascript");
+  assertSourceIcon(dbg, "original.js", "javascript");
+  // Framework icons are only displayed when we parse the source,
+  // which happens when we select the source
+  assertSourceIcon(dbg, "react-component-module.js", "javascript");
+  await selectSource(dbg, "react-component-module.js");
+  assertSourceIcon(dbg, "react-component-module.js", "react");
+
+  info("Verify blackbox source icon");
+  await selectSource(dbg, "script.js");
+  await clickElement(dbg, "blackbox");
+  await waitForDispatch(dbg.store, "BLACKBOX_WHOLE_SOURCES");
+  assertSourceIcon(dbg, "script.js", "blackBox");
+  await clickElement(dbg, "blackbox");
+  await waitForDispatch(dbg.store, "UNBLACKBOX_WHOLE_SOURCES");
+  assertSourceIcon(dbg, "script.js", "javascript");
+
   info("Assert the content of the named eval");
   await selectSource(dbg, "named-eval.js");
   assertTextContentOnLine(dbg, 3, `console.log("named-eval");`);
@@ -355,6 +402,7 @@ add_task(async function testSourceTreeOnTheIntegrationTestPage() {
   clickElement(dbg, "prettyPrintButton");
   await waitForSource(dbg, "query.js?x=1:formatted");
   await waitForSelectedSource(dbg, "query.js?x=1:formatted");
+  assertSourceIcon(dbg, "query.js?x=1", "prettyPrint");
 
   const prettyTab = findElement(dbg, "activeTab");
   is(prettyTab.innerText, "query.js?x=1", "Tab label is query.js?x=1");
@@ -393,24 +441,31 @@ add_task(async function testSourceTreeWithWebExtensionContentScript() {
   let dbg = await initDebugger("doc-content-script-sources.html");
   // Let some time for unexpected source to appear
   await wait(1000);
-  // Bug 1761975 - While the content script doesn't show up,
-  // the internals of WebExtension codebase appear in the debugger...
-  await waitForSourcesInSourceTree(dbg, ["ExtensionContent.jsm"]);
+  await waitForSourcesInSourceTree(dbg, []);
   await dbg.toolbox.closeToolbox();
 
   info("With the chrome preference, the content script shows up");
   await pushPref("devtools.chrome.enabled", true);
   const toolbox = await openToolboxForTab(gBrowser.selectedTab, "jsdebugger");
   dbg = createDebuggerContext(toolbox);
-  await waitForSourcesInSourceTree(dbg, [
-    "content_script.js",
-    "ExtensionContent.jsm",
-  ]);
+  await waitForSourcesInSourceTree(dbg, ["content_script.js"]);
   await selectSource(dbg, "content_script.js");
   ok(
     findElementWithSelector(dbg, ".sources-list .focused"),
     "Source is focused"
   );
+
+  const contentScriptGroupItem = findSourceNodeWithText(
+    dbg,
+    "Test content script extension"
+  );
+  ok(contentScriptGroupItem, "Found the group item for the content script");
+  ok(
+    contentScriptGroupItem.querySelector("span.img.extension"),
+    "The group has the extension icon"
+  );
+  assertSourceIcon(dbg, "content_script.js", "javascript");
+
   for (let i = 1; i < 3; i++) {
     info(
       `Reloading tab (${i} time), the content script should always be reselected`
@@ -427,111 +482,61 @@ add_task(async function testSourceTreeWithWebExtensionContentScript() {
   await extension.unload();
 });
 
-// Test that the Web extension name is shown in source tree rather than
-// the extensions internal UUID. This checks both the web toolbox and the
-// browser toolbox.
-add_task(async function testSourceTreeNamesForWebExtensions() {
-  await pushPref("devtools.chrome.enabled", true);
-  const extension = await installAndStartContentScriptExtension();
+add_task(async function testSourceTreeWithEncodedPaths() {
+  const httpServer = createTestHTTPServer();
+  httpServer.registerContentType("html", "text/html");
+  httpServer.registerContentType("js", "application/javascript");
 
-  const dbg = await initDebugger("doc-content-script-sources.html");
-  await waitForSourcesInSourceTree(dbg, [], {
-    noExpand: true,
+  httpServer.registerPathHandler("/index.html", function (request, response) {
+    response.setStatusLine(request.httpVersion, 200, "OK");
+    response.write(`<!DOCTYPE html>
+    <html>
+      <head>
+      <script src="/my folder/my file.js"></script>
+      <script src="/malformedUri.js?%"></script>
+      </head>
+      <body>
+      <h1>Encoded scripts paths</h1>
+      </body>
+    `);
   });
-
-  is(
-    getLabel(dbg, 2),
-    "Test content script extension",
-    "Test content script extension is labeled properly"
-  );
-  is(getLabel(dbg, 3), "resource://gre", "resource://gre is labeled properly");
-
-  await dbg.toolbox.closeToolbox();
-  await extension.unload();
-
-  // Make sure the toolbox opens with the debugger selected.
-  await pushPref("devtools.browsertoolbox.panel", "jsdebugger");
-
-  const ToolboxTask = await initBrowserToolboxTask();
-  await ToolboxTask.importFunctions({
-    createDebuggerContext,
-    waitUntil,
-    findSourceNodeWithText,
-    findAllElements,
-    getSelector,
-    findAllElementsWithSelector,
-    assertSourceTreeNode,
-  });
-
-  // ToolboxTask.spawn pass input arguments by stringify them via string concatenation.
-  // This mean we have to stringify the input object, but don't have to parse it from the task.
-  await ToolboxTask.spawn(JSON.stringify(selectors), async _selectors => {
-    this.selectors = _selectors;
-  });
-
-  await ToolboxTask.spawn(null, async () => {
-    try {
-      /* global gToolbox */
-      // Wait for the debugger to finish loading.
-      await gToolbox.getPanelWhenReady("jsdebugger");
-      const dbgx = createDebuggerContext(gToolbox);
-      let rootNodeForExtensions = null;
-      await waitUntil(() => {
-        rootNodeForExtensions = findSourceNodeWithText(dbgx, "extension");
-        return !!rootNodeForExtensions;
-      });
-      // Find the root node for extensions and expand it if needed
-      if (
-        !!rootNodeForExtensions &&
-        !rootNodeForExtensions.querySelector(".arrow.expanded")
-      ) {
-        rootNodeForExtensions.querySelector(".arrow").click();
-      }
-
-      // Assert that extensions are displayed in the source tree
-      // with their extension name.
-      await assertSourceTreeNode(dbgx, "Picture-In-Picture");
-      await assertSourceTreeNode(dbgx, "Form Autofill");
-    } catch (e) {
-      console.log("Caught exception in spawn", e);
-      throw e;
+  httpServer.registerPathHandler(
+    encodeURI("/my folder/my file.js"),
+    function (request, response) {
+      response.setStatusLine(request.httpVersion, 200, "OK");
+      response.setHeader("Content-Type", "application/javascript", false);
+      response.write(`const x = 42`);
     }
-  });
-
-  await ToolboxTask.destroy();
-});
-
-/**
- * Return the text content for a given line in the Source Tree.
- *
- * @param {Object} dbg
- * @param {Number} index
- *        Line number in the source tree
- */
-function getLabel(dbg, index) {
-  return (
-    findElement(dbg, "sourceNode", index)
-      .textContent.trim()
-      // There is some special whitespace character which aren't removed by trim()
-      .replace(/^[\s\u200b]*/g, "")
   );
-}
+  httpServer.registerPathHandler(
+    "/malformedUri.js",
+    function (request, response) {
+      response.setStatusLine(request.httpVersion, 200, "OK");
+      response.setHeader("Content-Type", "application/javascript", false);
+      response.write(`const y = "malformed"`);
+    }
+  );
+  const port = httpServer.identity.primaryPort;
 
-/**
- * Find and assert the source tree node with the specified text
- * exists on the source tree.
- *
- * @param {Object} dbg
- * @param {String} text The node text displayed
- */
-async function assertSourceTreeNode(dbg, text) {
-  let node = null;
-  await waitUntil(() => {
-    node = findSourceNodeWithText(dbg, text);
-    return !!node;
-  });
-  ok(!!node, `Source tree node with text "${text}" exists`);
-}
+  const dbg = await initDebuggerWithAbsoluteURL(
+    `http://localhost:${port}/index.html`,
+    "my file.js"
+  );
+
+  await waitForSourcesInSourceTree(dbg, ["my file.js", "malformedUri.js?%"]);
+  ok(
+    true,
+    "source name are decoded in the tree, and malformed uri source are displayed"
+  );
+  is(
+    // We don't have any specific class on the folder item, so let's target the folder
+    // icon next sibling, which is the directory label.
+    findElementWithSelector(dbg, ".sources-panel .node .folder + .label")
+      .innerText,
+    "my folder",
+    "folder name is decoded in the tree"
+  );
+});
 
 /**
  * Assert the location displayed in the breakpoint list, in the right sidebar.
