@@ -464,9 +464,12 @@ bool PasteCommand::IsCommandEnabled(Command aCommand,
 
 nsresult PasteCommand::DoCommand(Command aCommand, EditorBase& aEditorBase,
                                  nsIPrincipal* aPrincipal) const {
-  nsresult rv = aEditorBase.PasteAsAction(nsIClipboard::kGlobalClipboard, true,
+  nsresult rv = aEditorBase.PasteAsAction(nsIClipboard::kGlobalClipboard,
+                                          EditorBase::DispatchPasteEvent::Yes,
                                           aPrincipal);
-  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "EditorBase::PasteAsAction() failed");
+  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
+                       "EditorBase::PasteAsAction(nsIClipboard::"
+                       "kGlobalClipboard, DispatchPasteEvent::Yes) failed");
   return rv;
 }
 
@@ -504,10 +507,11 @@ nsresult PasteTransferableCommand::DoCommandParam(
   if (NS_WARN_IF(!aTransferableParam)) {
     return NS_ERROR_INVALID_ARG;
   }
-  nsresult rv =
-      aEditorBase.PasteTransferableAsAction(aTransferableParam, aPrincipal);
-  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
-                       "EditorBase::PasteTransferableAsAction() failed");
+  nsresult rv = aEditorBase.PasteTransferableAsAction(
+      aTransferableParam, EditorBase::DispatchPasteEvent::Yes, aPrincipal);
+  NS_WARNING_ASSERTION(
+      NS_SUCCEEDED(rv),
+      "EditorBase::PasteTransferableAsAction(DispatchPasteEvent::Yes) failed");
   return rv;
 }
 
@@ -955,9 +959,11 @@ nsresult PasteQuotationCommand::DoCommand(Command aCommand,
                                           EditorBase& aEditorBase,
                                           nsIPrincipal* aPrincipal) const {
   nsresult rv = aEditorBase.PasteAsQuotationAsAction(
-      nsIClipboard::kGlobalClipboard, true, aPrincipal);
+      nsIClipboard::kGlobalClipboard, EditorBase::DispatchPasteEvent::Yes,
+      aPrincipal);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
-                       "EditorBase::PasteAsQuotationAsAction() failed");
+                       "EditorBase::PasteAsQuotationAsAction(nsIClipboard::"
+                       "kGlobalClipboard, DispatchPasteEvent::Yes) failed");
   return rv;
 }
 
