@@ -6,9 +6,11 @@
 #ifndef InsertTextTransaction_h
 #define InsertTextTransaction_h
 
-#include "mozilla/EditTransactionBase.h"  // base class
+#include "EditTransactionBase.h"  // base class
+
+#include "EditorDOMPoint.h"
+#include "EditorForwards.h"
 
-#include "mozilla/EditorDOMPoint.h"        // EditorDOMPointInText
 #include "nsCycleCollectionParticipant.h"  // various macros
 #include "nsID.h"                          // NS_DECLARE_STATIC_IID_ACCESSOR
 #include "nsISupportsImpl.h"               // NS_DECL_ISUPPORTS_INHERITED
@@ -16,9 +18,6 @@
 #include "nscore.h"                        // NS_IMETHOD, nsAString
 
 namespace mozilla {
-
-class EditorBase;
-
 namespace dom {
 class Text;
 }  // namespace dom
@@ -59,6 +58,14 @@ class InsertTextTransaction final : public EditTransactionBase {
    */
   void GetData(nsString& aResult);
 
+  template <typename EditorDOMPointType>
+  EditorDOMPointType SuggestPointToPutCaret() const {
+    if (NS_WARN_IF(!mTextNode)) {
+      return EditorDOMPointType();
+    }
+    return EditorDOMPointType(mTextNode, mOffset + mStringToInsert.Length());
+  }
+
   friend std::ostream& operator<<(std::ostream& aStream,
                                   const InsertTextTransaction& aTransaction);
 
