@@ -3,13 +3,16 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "ErrorList.h"
 #include "TextEditor.h"
 
+#include "AutoRangeArray.h"
+#include "EditAction.h"
+#include "EditorDOMPoint.h"
+#include "EditorUtils.h"
+#include "HTMLEditor.h"
+
 #include "mozilla/Assertions.h"
-#include "mozilla/EditAction.h"
-#include "mozilla/EditorDOMPoint.h"
-#include "mozilla/EditorUtils.h"
-#include "mozilla/HTMLEditor.h"
 #include "mozilla/LookAndFeel.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/StaticPrefs_editor.h"
@@ -19,6 +22,7 @@
 #include "mozilla/dom/NodeFilterBinding.h"
 #include "mozilla/dom/NodeIterator.h"
 #include "mozilla/dom/Selection.h"
+
 #include "nsAString.h"
 #include "nsCOMPtr.h"
 #include "nsCRT.h"
@@ -31,7 +35,7 @@
 #include "nsIContent.h"
 #include "nsIHTMLCollection.h"
 #include "nsINode.h"
-#include "nsISupportsBase.h"
+#include "nsISupports.h"
 #include "nsLiteralString.h"
 #include "nsNameSpaceManager.h"
 #include "nsPrintfCString.h"
@@ -44,7 +48,7 @@ using namespace dom;
 
 #define CANCEL_OPERATION_AND_RETURN_EDIT_ACTION_RESULT_IF_READONLY \
   if (IsReadonly()) {                                              \
-    return EditActionCanceled(NS_OK);                              \
+    return EditActionResult::CanceledResult();                     \
   }
 
 void TextEditor::OnStartToHandleTopLevelEditSubAction(
@@ -81,7 +85,7 @@ void TextEditor::OnStartToHandleTopLevelEditSubAction(
     // For spell checker, previous selected node should be text node if
     // possible. If anchor is root of editor, it may become invalid offset
     // after inserting text.
-    EditorRawDOMPoint point =
+    const EditorRawDOMPoint point =
         FindBetterInsertionPoint(EditorRawDOMPoint(SelectionRef().AnchorRef()));
     NS_WARNING_ASSERTION(
         point.IsSet(),
@@ -141,7 +145,35 @@ nsresult TextEditor::OnEndHandlingTopLevelEditSubAction() {
   return rv;
 }
 
-EditActionResult TextEditor::InsertLineFeedCharacterAtSelection() {
+nsresult TextEditor::InsertLineBreakAsSubAction() {
+  MOZ_ASSERT(IsEditActionDataAvailable());
+
+  if (NS_WARN_IF(!mInitSucceeded)) {
+    return NS_ERROR_NOT_INITIALIZED;
+  }
+
+  IgnoredErrorResult ignoredError;
+  AutoEditSubActionNotifier startToHandleEditSubAction(
+      *this, EditSubAction::eInsertLineBreak, nsIEditor::eNext, ignoredError);
+  if (NS_WARN_IF(ignoredError.ErrorCodeIs(NS_ERROR_EDITOR_DESTROYED))) {
+    return ignoredError.StealNSResult();
+  }
+  NS_WARNING_ASSERTION(
+      !ignoredError.Failed(),
+      "TextEditor::OnStartToHandleTopLevelEditSubAction() failed, but ignored");
+
+  Result<EditActionResult, nsresult> result =
+      InsertLineFeedCharacterAtSelection();
+  if (MOZ_UNLIKELY(result.isErr())) {
+    NS_WARNING(
+        "TextEditor::InsertLineFeedCharacterAtSelection() failed, but ignored");
+    return result.unwrapErr();
+  }
+  return NS_OK;
+}
+
+Result<EditActionResult, nsresult>
+TextEditor::InsertLineFeedCharacterAtSelection() {
   MOZ_ASSERT(IsEditActionDataAvailable());
   MOZ_ASSERT(!IsSingleLineEditor());
 
@@ -151,16 +183,16 @@ EditActionResult TextEditor::InsertLineFeedCharacterAtSelection() {
 
   if (mMaxTextLength >= 0) {
     nsAutoString insertionString(u"\n"_ns);
-    EditActionResult result =
+    Result<EditActionResult, nsresult> result =
         MaybeTruncateInsertionStringForMaxLength(insertionString);
-    if (result.Failed()) {
+    if (MOZ_UNLIKELY(result.isErr())) {
       NS_WARNING(
           "TextEditor::MaybeTruncateInsertionStringForMaxLength() failed");
       return result;
     }
-    if (result.Handled()) {
+    if (result.inspect().Handled()) {
       // Don't return as handled since we stopped inserting the line break.
-      return EditActionCanceled();
+      return EditActionResult::CanceledResult();
     }
   }
 
@@ -171,74 +203,51 @@ EditActionResult TextEditor::InsertLineFeedCharacterAtSelection() {
     if (NS_FAILED(rv)) {
       NS_WARNING(
           "EditorBase::DeleteSelectionAsSubAction(eNone, eNoStrip) failed");
-      return EditActionIgnored(rv);
+      return Err(rv);
     }
   }
 
-  // get the (collapsed) selection location
-  const nsRange* firstRange = SelectionRef().GetRangeAt(0);
-  if (NS_WARN_IF(!firstRange)) {
-    return EditActionIgnored(NS_ERROR_FAILURE);
-  }
-
-  EditorRawDOMPoint pointToInsert(firstRange->StartRef());
+  const auto pointToInsert = GetFirstSelectionStartPoint<EditorDOMPoint>();
   if (NS_WARN_IF(!pointToInsert.IsSet())) {
-    return EditActionIgnored(NS_ERROR_FAILURE);
+    return Err(NS_ERROR_FAILURE);
   }
   MOZ_ASSERT(pointToInsert.IsSetAndValid());
   MOZ_ASSERT(!pointToInsert.IsContainerHTMLElement(nsGkAtoms::br));
 
   RefPtr<Document> document = GetDocument();
   if (NS_WARN_IF(!document)) {
-    return EditActionIgnored(NS_ERROR_NOT_INITIALIZED);
+    return Err(NS_ERROR_NOT_INITIALIZED);
   }
 
-  // Don't change my selection in sub-transactions.
-  AutoTransactionsConserveSelection dontChangeMySelection(*this);
-
   // Insert a linefeed character.
-  EditorRawDOMPoint pointAfterInsertedLineFeed;
-  nsresult rv = InsertTextWithTransaction(*document, u"\n"_ns, pointToInsert,
-                                          &pointAfterInsertedLineFeed);
-  if (!pointAfterInsertedLineFeed.IsSet()) {
-    NS_WARNING(
-        "EditorBase::InsertTextWithTransaction(\\n) didn't return position of "
-        "inserted linefeed");
-    return EditActionIgnored(NS_ERROR_FAILURE);
+  Result<InsertTextResult, nsresult> insertTextResult =
+      InsertTextWithTransaction(*document, u"\n"_ns, pointToInsert);
+  if (MOZ_UNLIKELY(insertTextResult.isErr())) {
+    NS_WARNING("TextEditor::InsertTextWithTransaction(\"\\n\") failed");
+    return insertTextResult.propagateErr();
   }
-  if (NS_FAILED(rv)) {
-    NS_WARNING("TextEditor::InsertTextWithTransaction(\\n) failed");
-    return EditActionIgnored(rv);
+  insertTextResult.inspect().IgnoreCaretPointSuggestion();
+  EditorDOMPoint pointToPutCaret = insertTextResult.inspect().Handled()
+                                       ? insertTextResult.inspect()
+                                             .EndOfInsertedTextRef()
+                                             .To<EditorDOMPoint>()
+                                       : pointToInsert;
+  if (NS_WARN_IF(!pointToPutCaret.IsSetAndValid())) {
+    return Err(NS_ERROR_FAILURE);
   }
-
-  // set the selection to the correct location
-  MOZ_ASSERT(
-      !pointAfterInsertedLineFeed.GetChild(),
-      "After inserting text into a text node, pointAfterInsertedLineFeed."
-      "GetChild() should be nullptr");
-  rv = SelectionRef().CollapseInLimiter(pointAfterInsertedLineFeed);
-  if (NS_FAILED(rv)) {
-    NS_WARNING("Selection::CollapseInLimiter() failed");
-    return EditActionIgnored(rv);
-  }
-
   // XXX I don't think we still need this.  This must have been required when
   //     `<textarea>` was implemented with text nodes and `<br>` elements.
-  // see if we're at the end of the editor range
-  EditorRawDOMPoint endPoint(EditorBase::GetEndPoint(SelectionRef()));
-  if (endPoint == pointAfterInsertedLineFeed) {
-    // SetInterlinePosition(true) means we want the caret to stick to the
-    // content on the "right".  We want the caret to stick to whatever is
-    // past the break.  This is because the break is on the same line we
-    // were on, but the next content will be on the following line.
-    IgnoredErrorResult ignoredError;
-    SelectionRef().SetInterlinePosition(true, ignoredError);
-    NS_WARNING_ASSERTION(
-        !ignoredError.Failed(),
-        "Selection::SetInterlinePosition(true) failed, but ignored");
+  // We want the caret to stick to the content on the "right".  We want the
+  // caret to stick to whatever is past the break.  This is because the break is
+  // on the same line we were on, but the next content will be on the following
+  // line.
+  pointToPutCaret.SetInterlinePosition(InterlinePosition::StartOfNextLine);
+  nsresult rv = CollapseSelectionTo(pointToPutCaret);
+  if (NS_FAILED(rv)) {
+    NS_WARNING("EditorBase::CollapseSelectionTo() failed");
+    return Err(rv);
   }
-
-  return EditActionHandled();
+  return EditActionResult::HandledResult();
 }
 
 nsresult TextEditor::EnsureCaretNotAtEndOfTextNode() {
@@ -252,13 +261,16 @@ nsresult TextEditor::EnsureCaretNotAtEndOfTextNode() {
     return NS_OK;
   }
 
-  DebugOnly<nsresult> rvIgnored = CollapseSelectionToEnd();
-  if (NS_WARN_IF(Destroyed())) {
+  nsresult rv = CollapseSelectionToEndOfTextNode();
+  if (MOZ_UNLIKELY(rv == NS_ERROR_EDITOR_DESTROYED)) {
+    NS_WARNING(
+        "TextEditor::CollapseSelectionToEndOfTextNode() caused destroying the "
+        "editor");
     return NS_ERROR_EDITOR_DESTROYED;
   }
   NS_WARNING_ASSERTION(
-      NS_SUCCEEDED(rvIgnored),
-      "EditorBase::CollapseSelectionToEnd() failed, but ignored");
+      NS_SUCCEEDED(rv),
+      "TextEditor::CollapseSelectionToEndOfTextNode() failed, but ignored");
 
   return NS_OK;
 }
@@ -337,11 +349,14 @@ void TextEditor::HandleNewLinesInStringForSingleLineEditor(
   }
 }
 
-EditActionResult TextEditor::HandleInsertText(
-    EditSubAction aEditSubAction, const nsAString& aInsertionString) {
+Result<EditActionResult, nsresult> TextEditor::HandleInsertText(
+    EditSubAction aEditSubAction, const nsAString& aInsertionString,
+    SelectionHandling aSelectionHandling) {
   MOZ_ASSERT(IsEditActionDataAvailable());
   MOZ_ASSERT(aEditSubAction == EditSubAction::eInsertText ||
              aEditSubAction == EditSubAction::eInsertTextComingFromIME);
+  MOZ_ASSERT_IF(aSelectionHandling == SelectionHandling::Ignore,
+                aEditSubAction == EditSubAction::eInsertTextComingFromIME);
 
   UndefineCaretBidiLevel();
 
@@ -352,23 +367,25 @@ EditActionResult TextEditor::HandleInsertText(
     // because IME transaction depend on them
     // There is more work to do to make the
     // world safe for IME.
-    return EditActionCanceled();
+    return EditActionResult::CanceledResult();
   }
 
   nsAutoString insertionString(aInsertionString);
   if (mMaxTextLength >= 0) {
-    EditActionResult result =
+    Result<EditActionResult, nsresult> result =
         MaybeTruncateInsertionStringForMaxLength(insertionString);
-    if (result.Failed()) {
+    if (MOZ_UNLIKELY(result.isErr())) {
       NS_WARNING(
           "TextEditor::MaybeTruncateInsertionStringForMaxLength() failed");
-      return result.MarkAsHandled();
+      EditActionResult unwrappedResult = result.unwrap();
+      unwrappedResult.MarkAsHandled();
+      return unwrappedResult;
     }
     // If we're exceeding the maxlength when composing IME, we need to clean up
     // the composing text, so we shouldn't return early.
-    if (result.Handled() && insertionString.IsEmpty() &&
+    if (result.inspect().Handled() && insertionString.IsEmpty() &&
         aEditSubAction != EditSubAction::eInsertTextComingFromIME) {
-      return EditActionCanceled();
+      return EditActionResult::CanceledResult();
     }
   }
 
@@ -384,13 +401,14 @@ EditActionResult TextEditor::HandleInsertText(
   }
 
   // if the selection isn't collapsed, delete it.
-  if (!SelectionRef().IsCollapsed()) {
+  if (!SelectionRef().IsCollapsed() &&
+      aSelectionHandling == SelectionHandling::Delete) {
     nsresult rv =
         DeleteSelectionAsSubAction(nsIEditor::eNone, nsIEditor::eNoStrip);
     if (NS_FAILED(rv)) {
       NS_WARNING(
           "EditorBase::DeleteSelectionAsSubAction(eNone, eNoStrip) failed");
-      return EditActionHandled(rv);
+      return Err(rv);
     }
   }
 
@@ -423,80 +441,78 @@ EditActionResult TextEditor::HandleInsertText(
     HandleNewLinesInStringForSingleLineEditor(insertionString);
   }
 
-  // get the (collapsed) selection location
-  const nsRange* firstRange = SelectionRef().GetRangeAt(0);
-  if (NS_WARN_IF(!firstRange)) {
-    return EditActionHandled(NS_ERROR_FAILURE);
-  }
-  EditorRawDOMPoint atStartOfSelection(firstRange->StartRef());
+  const auto atStartOfSelection = GetFirstSelectionStartPoint<EditorDOMPoint>();
   if (NS_WARN_IF(!atStartOfSelection.IsSetAndValid())) {
-    return EditActionHandled(NS_ERROR_FAILURE);
+    return Err(NS_ERROR_FAILURE);
   }
   MOZ_ASSERT(!atStartOfSelection.IsContainerHTMLElement(nsGkAtoms::br));
 
   RefPtr<Document> document = GetDocument();
   if (NS_WARN_IF(!document)) {
-    return EditActionHandled(NS_ERROR_NOT_INITIALIZED);
+    return Err(NS_ERROR_NOT_INITIALIZED);
   }
 
   if (aEditSubAction == EditSubAction::eInsertTextComingFromIME) {
-    EditorRawDOMPoint compositionStartPoint = GetCompositionStartPoint();
+    EditorDOMPoint compositionStartPoint =
+        GetFirstIMESelectionStartPoint<EditorDOMPoint>();
     if (!compositionStartPoint.IsSet()) {
       compositionStartPoint = FindBetterInsertionPoint(atStartOfSelection);
       NS_WARNING_ASSERTION(
           compositionStartPoint.IsSet(),
           "EditorBase::FindBetterInsertionPoint() failed, but ignored");
     }
-    nsresult rv = InsertTextWithTransaction(*document, insertionString,
-                                            compositionStartPoint);
-    if (NS_WARN_IF(Destroyed())) {
-      return EditActionHandled(NS_ERROR_EDITOR_DESTROYED);
-    }
-    if (NS_FAILED(rv)) {
+    Result<InsertTextResult, nsresult> insertTextResult =
+        InsertTextWithTransaction(*document, insertionString,
+                                  compositionStartPoint);
+    if (MOZ_UNLIKELY(insertTextResult.isErr())) {
       NS_WARNING("EditorBase::InsertTextWithTransaction() failed");
-      return EditActionHandled(rv);
+      return insertTextResult.propagateErr();
     }
+    nsresult rv = insertTextResult.unwrap().SuggestCaretPointTo(
+        *this, {SuggestCaret::OnlyIfHasSuggestion,
+                SuggestCaret::OnlyIfTransactionsAllowedToDoIt,
+                SuggestCaret::AndIgnoreTrivialError});
+    if (NS_FAILED(rv)) {
+      NS_WARNING("CaretPoint::SuggestCaretPointTo() failed");
+      return Err(rv);
+    }
+    NS_WARNING_ASSERTION(
+        rv != NS_SUCCESS_EDITOR_BUT_IGNORED_TRIVIAL_ERROR,
+        "CaretPoint::SuggestCaretPointTo() failed, but ignored");
   } else {
     MOZ_ASSERT(aEditSubAction == EditSubAction::eInsertText);
 
-    // don't change my selection in subtransactions
-    AutoTransactionsConserveSelection dontChangeMySelection(*this);
-
-    EditorRawDOMPoint pointAfterStringInserted;
-    nsresult rv = InsertTextWithTransaction(*document, insertionString,
-                                            atStartOfSelection,
-                                            &pointAfterStringInserted);
-    if (NS_WARN_IF(Destroyed())) {
-      return EditActionHandled(NS_ERROR_EDITOR_DESTROYED);
-    }
-    if (NS_FAILED(rv)) {
+    Result<InsertTextResult, nsresult> insertTextResult =
+        InsertTextWithTransaction(*document, insertionString,
+                                  atStartOfSelection);
+    if (MOZ_UNLIKELY(insertTextResult.isErr())) {
       NS_WARNING("EditorBase::InsertTextWithTransaction() failed");
-      return EditActionHandled(rv);
+      return insertTextResult.propagateErr();
     }
-
-    if (pointAfterStringInserted.IsSet()) {
+    // Ignore caret suggestion because there was
+    // AutoTransactionsConserveSelection.
+    insertTextResult.inspect().IgnoreCaretPointSuggestion();
+    if (insertTextResult.inspect().Handled()) {
       // Make the caret attach to the inserted text, unless this text ends with
       // a LF, in which case make the caret attach to the next line.
-      bool endsWithLF =
+      const bool endsWithLF =
           !insertionString.IsEmpty() && insertionString.Last() == nsCRT::LF;
-      IgnoredErrorResult ignoredError;
-      SelectionRef().SetInterlinePosition(endsWithLF, ignoredError);
-      NS_WARNING_ASSERTION(
-          !ignoredError.Failed(),
-          "Selection::SetInterlinePosition() failed, but ignored");
-
-      MOZ_ASSERT(
-          !pointAfterStringInserted.GetChild(),
-          "After inserting text into a text node, pointAfterStringInserted."
-          "GetChild() should be nullptr");
-      ignoredError = IgnoredErrorResult();
-      SelectionRef().CollapseInLimiter(pointAfterStringInserted, ignoredError);
-      if (NS_WARN_IF(Destroyed())) {
-        return EditActionHandled(NS_ERROR_EDITOR_DESTROYED);
+      EditorDOMPoint pointToPutCaret = insertTextResult.inspect()
+                                           .EndOfInsertedTextRef()
+                                           .To<EditorDOMPoint>();
+      pointToPutCaret.SetInterlinePosition(
+          endsWithLF ? InterlinePosition::StartOfNextLine
+                     : InterlinePosition::EndOfLine);
+      MOZ_ASSERT(pointToPutCaret.IsInTextNode(),
+                 "After inserting text into a text node, insertTextResult "
+                 "should return a point in a text node");
+      nsresult rv = CollapseSelectionTo(pointToPutCaret);
+      if (NS_WARN_IF(rv == NS_ERROR_EDITOR_DESTROYED)) {
+        return Err(NS_ERROR_EDITOR_DESTROYED);
       }
       NS_WARNING_ASSERTION(
-          !ignoredError.Failed(),
-          "Selection::CollapseInLimiter() failed, but ignored");
+          NS_SUCCEEDED(rv),
+          "EditorBase::CollapseSelectionTo() failed, but ignored");
     }
   }
 
@@ -504,15 +520,17 @@ EditActionResult TextEditor::HandleInsertText(
   if (IsPasswordEditor() && IsMaskingPassword() && CanEchoPasswordNow()) {
     nsresult rv = SetUnmaskRangeAndNotify(start, insertionString.Length(),
                                           LookAndFeel::GetPasswordMaskDelay());
-    NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
-                         "TextEditor::SetUnmaskRangeAndNotify() failed");
-    return EditActionHandled(rv);
+    if (NS_FAILED(rv)) {
+      NS_WARNING("TextEditor::SetUnmaskRangeAndNotify() failed");
+      return Err(rv);
+    }
+    return EditActionResult::HandledResult();
   }
 
-  return EditActionHandled();
+  return EditActionResult::HandledResult();
 }
 
-EditActionResult TextEditor::SetTextWithoutTransaction(
+Result<EditActionResult, nsresult> TextEditor::SetTextWithoutTransaction(
     const nsAString& aValue) {
   MOZ_ASSERT(IsEditActionDataAvailable());
   MOZ_ASSERT(!IsIMEComposing());
@@ -545,7 +563,7 @@ EditActionResult TextEditor::SetTextWithoutTransaction(
     if (!textNode->GetNextSibling() ||
         !EditorUtils::IsPaddingBRElementForEmptyLastLine(
             *textNode->GetNextSibling())) {
-      return EditActionIgnored();
+      return EditActionResult::IgnoredResult();
     }
   }
 
@@ -559,13 +577,13 @@ EditActionResult TextEditor::SetTextWithoutTransaction(
   nsresult rv = SetTextNodeWithoutTransaction(sanitizedValue, *textNode);
   if (NS_FAILED(rv)) {
     NS_WARNING("EditorBase::SetTextNodeWithoutTransaction() failed");
-    return EditActionResult(rv);
+    return Err(rv);
   }
 
-  return EditActionHandled();
+  return EditActionResult::HandledResult();
 }
 
-EditActionResult TextEditor::HandleDeleteSelection(
+Result<EditActionResult, nsresult> TextEditor::HandleDeleteSelection(
     nsIEditor::EDirection aDirectionAndAmount,
     nsIEditor::EStripWrappers aStripWrappers) {
   MOZ_ASSERT(IsEditActionDataAvailable());
@@ -576,22 +594,22 @@ EditActionResult TextEditor::HandleDeleteSelection(
   CANCEL_OPERATION_AND_RETURN_EDIT_ACTION_RESULT_IF_READONLY
 
   if (IsEmpty()) {
-    return EditActionCanceled();
+    return EditActionResult::CanceledResult();
   }
-  EditActionResult result =
+  Result<EditActionResult, nsresult> result =
       HandleDeleteSelectionInternal(aDirectionAndAmount, nsIEditor::eNoStrip);
   // HandleDeleteSelectionInternal() creates SelectionBatcher.  Therefore,
   // quitting from it might cause having destroyed the editor.
   if (NS_WARN_IF(Destroyed())) {
-    return result.SetResult(NS_ERROR_EDITOR_DESTROYED);
+    return Err(NS_ERROR_EDITOR_DESTROYED);
   }
   NS_WARNING_ASSERTION(
-      result.Succeeded(),
+      result.isOk(),
       "TextEditor::HandleDeleteSelectionInternal(eNoStrip) failed");
   return result;
 }
 
-EditActionResult TextEditor::HandleDeleteSelectionInternal(
+Result<EditActionResult, nsresult> TextEditor::HandleDeleteSelectionInternal(
     nsIEditor::EDirection aDirectionAndAmount,
     nsIEditor::EStripWrappers aStripWrappers) {
   MOZ_ASSERT(IsEditActionDataAvailable());
@@ -603,38 +621,40 @@ EditActionResult TextEditor::HandleDeleteSelectionInternal(
   // want to send a single selectionchange event to the document, so we
   // batch the selectionchange events, such that a single event fires after
   // the AutoHideSelectionChanges destructor has been run.
-  SelectionBatcher selectionBatcher(SelectionRef());
+  SelectionBatcher selectionBatcher(SelectionRef(), __FUNCTION__);
   AutoHideSelectionChanges hideSelection(SelectionRef());
   nsAutoScriptBlocker scriptBlocker;
 
   if (IsPasswordEditor() && IsMaskingPassword()) {
     MaskAllCharacters();
   } else {
-    EditorRawDOMPoint selectionStartPoint(
-        EditorBase::GetStartPoint(SelectionRef()));
+    const auto selectionStartPoint =
+        GetFirstSelectionStartPoint<EditorRawDOMPoint>();
     if (NS_WARN_IF(!selectionStartPoint.IsSet())) {
-      return EditActionResult(NS_ERROR_FAILURE);
+      return Err(NS_ERROR_FAILURE);
     }
 
     if (!SelectionRef().IsCollapsed()) {
       nsresult rv = DeleteSelectionWithTransaction(aDirectionAndAmount,
                                                    nsIEditor::eNoStrip);
-      NS_WARNING_ASSERTION(
-          NS_SUCCEEDED(rv),
-          "EditorBase::DeleteSelectionWithTransaction(eNoStrip) failed");
-      return EditActionHandled(rv);
+      if (NS_FAILED(rv)) {
+        NS_WARNING(
+            "EditorBase::DeleteSelectionWithTransaction(eNoStrip) failed");
+        return Err(rv);
+      }
+      return EditActionResult::HandledResult();
     }
 
     // Test for distance between caret and text that will be deleted
     AutoCaretBidiLevelManager bidiLevelManager(*this, aDirectionAndAmount,
                                                selectionStartPoint);
-    if (bidiLevelManager.Failed()) {
+    if (MOZ_UNLIKELY(bidiLevelManager.Failed())) {
       NS_WARNING("EditorBase::AutoCaretBidiLevelManager() failed");
-      return EditActionResult(NS_ERROR_FAILURE);
+      return Err(NS_ERROR_FAILURE);
     }
     bidiLevelManager.MaybeUpdateCaretBidiLevel(*this);
     if (bidiLevelManager.Canceled()) {
-      return EditActionCanceled();
+      return EditActionResult::CanceledResult();
     }
   }
 
@@ -643,27 +663,43 @@ EditActionResult TextEditor::HandleDeleteSelectionInternal(
       rangesToDelete.ExtendAnchorFocusRangeFor(*this, aDirectionAndAmount);
   if (result.isErr()) {
     NS_WARNING("AutoRangeArray::ExtendAnchorFocusRangeFor() failed");
-    return EditActionResult(result.unwrapErr());
+    return result.propagateErr();
+  }
+  if (const Text* theTextNode = GetTextNode()) {
+    rangesToDelete.EnsureRangesInTextNode(*theTextNode);
   }
 
-  nsresult rv = DeleteRangesWithTransaction(
+  Result<CaretPoint, nsresult> caretPointOrError = DeleteRangesWithTransaction(
       result.unwrap(), nsIEditor::eNoStrip, rangesToDelete);
-  NS_WARNING_ASSERTION(
-      NS_SUCCEEDED(rv),
-      "EditorBase::DeleteRangesWithTransaction(eNoStrip) failed");
-  return EditActionHandled(rv);
+  if (MOZ_UNLIKELY(caretPointOrError.isErr())) {
+    NS_WARNING("EditorBase::DeleteRangesWithTransaction(eNoStrip) failed");
+    return caretPointOrError.propagateErr();
+  }
+
+  nsresult rv = caretPointOrError.inspect().SuggestCaretPointTo(
+      *this, {SuggestCaret::OnlyIfHasSuggestion,
+              SuggestCaret::OnlyIfTransactionsAllowedToDoIt,
+              SuggestCaret::AndIgnoreTrivialError});
+  if (NS_FAILED(rv)) {
+    NS_WARNING("CaretPoint::SuggestCaretPointTo() failed");
+    return Err(rv);
+  }
+  NS_WARNING_ASSERTION(rv != NS_SUCCESS_EDITOR_BUT_IGNORED_TRIVIAL_ERROR,
+                       "CaretPoint::SuggestCaretPointTo() failed, but ignored");
+
+  return EditActionResult::HandledResult();
 }
 
-EditActionResult TextEditor::ComputeValueFromTextNodeAndBRElement(
-    nsAString& aValue) const {
+Result<EditActionResult, nsresult>
+TextEditor::ComputeValueFromTextNodeAndBRElement(nsAString& aValue) const {
   MOZ_ASSERT(IsEditActionDataAvailable());
   MOZ_ASSERT(!IsHTMLEditor());
 
   Element* anonymousDivElement = GetRoot();
-  if (!anonymousDivElement) {
+  if (MOZ_UNLIKELY(!anonymousDivElement)) {
     // Don't warn this case, this is possible, e.g., 997805.html
     aValue.Truncate();
-    return EditActionHandled();
+    return EditActionResult::HandledResult();
   }
 
   Text* textNode = Text::FromNodeOrNull(anonymousDivElement->GetFirstChild());
@@ -671,7 +707,7 @@ EditActionResult TextEditor::ComputeValueFromTextNodeAndBRElement(
 
   if (!textNode->Length()) {
     aValue.Truncate();
-    return EditActionHandled();
+    return EditActionResult::HandledResult();
   }
 
   nsIContent* firstChildExceptText = textNode->GetNextSibling();
@@ -684,21 +720,22 @@ EditActionResult TextEditor::ComputeValueFromTextNodeAndBRElement(
                  !EditorUtils::IsPaddingBRElementForEmptyLastLine(
                      *firstChildExceptText) &&
                  !firstChildExceptText->IsXULElement(nsGkAtoms::scrollbar))) {
-    return EditActionIgnored();
+    return EditActionResult::IgnoredResult();
   }
 
   // Otherwise, the text data is the value.
   textNode->GetData(aValue);
-  return EditActionHandled();
+  return EditActionResult::HandledResult();
 }
 
-EditActionResult TextEditor::MaybeTruncateInsertionStringForMaxLength(
+Result<EditActionResult, nsresult>
+TextEditor::MaybeTruncateInsertionStringForMaxLength(
     nsAString& aInsertionString) {
   MOZ_ASSERT(IsEditActionDataAvailable());
   MOZ_ASSERT(mMaxTextLength >= 0);
 
   if (IsIMEComposing()) {
-    return EditActionIgnored();
+    return EditActionResult::IgnoredResult();
   }
 
   // Ignore user pastes
@@ -713,7 +750,7 @@ EditActionResult TextEditor::MaybeTruncateInsertionStringForMaxLength(
         // By now we are certain that this is a user paste, before we ignore it,
         // lets check if the user explictly enabled truncating user pastes.
         if (!StaticPrefs::editor_truncate_user_pastes()) {
-          return EditActionIgnored();
+          return EditActionResult::IgnoredResult();
         }
       }
       [[fallthrough]];
@@ -725,7 +762,7 @@ EditActionResult TextEditor::MaybeTruncateInsertionStringForMaxLength(
   nsresult rv = GetTextLength(&currentLength);
   if (NS_FAILED(rv)) {
     NS_WARNING("TextEditor::GetTextLength() failed");
-    return EditActionResult(rv);
+    return Err(rv);
   }
 
   uint32_t selectionStart, selectionEnd;
@@ -744,12 +781,12 @@ EditActionResult TextEditor::MaybeTruncateInsertionStringForMaxLength(
       currentLength - kSelectionLength - kOldCompositionStringLength;
   if (kNewLength >= AssertedCast<uint32_t>(mMaxTextLength)) {
     aInsertionString.Truncate();  // Too long, we cannot accept new character.
-    return EditActionHandled();
+    return EditActionResult::HandledResult();
   }
 
   if (aInsertionString.Length() + kNewLength <=
       AssertedCast<uint32_t>(mMaxTextLength)) {
-    return EditActionIgnored();  // Enough short string.
+    return EditActionResult::IgnoredResult();  // Enough short string.
   }
 
   int32_t newInsertionStringLength = mMaxTextLength - kNewLength;
@@ -765,7 +802,7 @@ EditActionResult TextEditor::MaybeTruncateInsertionStringForMaxLength(
   // XXX What should we do if we're removing IVS but its preceding
   //     character won't be removed?
   aInsertionString.Truncate(newInsertionStringLength);
-  return EditActionHandled();
+  return EditActionResult::HandledResult();
 }
 
 bool TextEditor::CanEchoPasswordNow() const {
