@@ -1,17 +1,14 @@
-/* import-globals-from antitracking_head.js */
-
 gFeatures = "noopener";
 
-AntiTracking.runTest(
+AntiTracking.runTestInNormalAndPrivateMode(
   "Blocking in the case of noopener windows",
   async _ => {
-    is(window.localStorage, null, "LocalStorage is null");
     try {
       localStorage.foo = 42;
       ok(false, "LocalStorage cannot be used!");
     } catch (e) {
       ok(true, "LocalStorage cannot be used!");
-      is(e.name, "TypeError", "We want a type error message.");
+      is(e.name, "SecurityError", "We want a security error message.");
     }
   },
   async phase => {
@@ -21,13 +18,12 @@ AntiTracking.runTest(
         ok(true, "LocalStorage is allowed");
         break;
       case 2:
-        is(window.localStorage, null, "LocalStorage is null");
         try {
           localStorage.foo = 42;
           ok(false, "LocalStorage cannot be used!");
         } catch (e) {
           ok(true, "LocalStorage cannot be used!");
-          is(e.name, "TypeError", "We want a type error message.");
+          is(e.name, "SecurityError", "We want a security error message.");
         }
         break;
     }
