@@ -6,14 +6,13 @@
 #include "nsAutoCompleteController.h"
 #include "nsAutoCompleteSimpleResult.h"
 
-#include "nsAutoPtr.h"
 #include "nsNetCID.h"
 #include "nsIIOService.h"
-#include "nsIServiceManager.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIObserverService.h"
+#include "nsServiceManagerUtils.h"
 #include "mozilla/Services.h"
 #include "mozilla/Unused.h"
 #include "mozilla/dom/KeyboardEventBinding.h"
@@ -61,14 +60,10 @@ nsAutoCompleteController::nsAutoCompleteController()
 
 nsAutoCompleteController::~nsAutoCompleteController() { SetInput(nullptr); }
 
-void nsAutoCompleteController::SetValueOfInputTo(const nsString& aValue,
-                                                 uint16_t aReason) {
+void nsAutoCompleteController::SetValueOfInputTo(const nsString& aValue) {
   mSetValue = aValue;
   nsCOMPtr<nsIAutoCompleteInput> input(mInput);
-  nsresult rv = input->SetTextValueWithReason(aValue, aReason);
-  if (NS_FAILED(rv)) {
-    input->SetTextValue(aValue);
-  }
+  input->SetTextValue(aValue);
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -422,21 +417,16 @@ nsAutoCompleteController::HandleKeyNavigation(uint32_t aKey, bool* _retval) {
       aKey == dom::KeyboardEvent_Binding::DOM_VK_DOWN ||
       aKey == dom::KeyboardEvent_Binding::DOM_VK_PAGE_UP ||
       aKey == dom::KeyboardEvent_Binding::DOM_VK_PAGE_DOWN) {
-    // Prevent the input from handling up/down events, as it may move
-    // the cursor to home/end on some systems
-    *_retval = true;
-
     bool isOpen = false;
     input->GetPopupOpen(&isOpen);
     if (isOpen) {
+      // Prevent the input from handling up/down events, as it may move
+      // the cursor to home/end on some systems
+      *_retval = true;
       bool reverse = aKey == dom::KeyboardEvent_Binding::DOM_VK_UP ||
-                             aKey == dom::KeyboardEvent_Binding::DOM_VK_PAGE_UP
-                         ? true
-                         : false;
+                     aKey == dom::KeyboardEvent_Binding::DOM_VK_PAGE_UP;
       bool page = aKey == dom::KeyboardEvent_Binding::DOM_VK_PAGE_UP ||
-                          aKey == dom::KeyboardEvent_Binding::DOM_VK_PAGE_DOWN
-                      ? true
-                      : false;
+                  aKey == dom::KeyboardEvent_Binding::DOM_VK_PAGE_DOWN;
 
       // Fill in the value of the textbox with whatever is selected in the popup
       // if the completeSelectedIndex attribute is set.  We check this before
@@ -464,17 +454,13 @@ nsAutoCompleteController::HandleKeyNavigation(uint32_t aKey, bool* _retval) {
             // autofilled.  Else, fill the result and move the caret to the end.
             int32_t start;
             if (value.Equals(mPlaceholderCompletionString,
-                             nsCaseInsensitiveStringComparator())) {
+                             nsCaseInsensitiveStringComparator)) {
               start = mSearchString.Length();
               value = mPlaceholderCompletionString;
-              SetValueOfInputTo(
-                  value,
-                  nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETEDEFAULT);
+              SetValueOfInputTo(value);
             } else {
               start = value.Length();
-              SetValueOfInputTo(
-                  value,
-                  nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETESELECTED);
+              SetValueOfInputTo(value);
             }
 
             input->SelectTextRange(start, value.Length());
@@ -482,68 +468,86 @@ nsAutoCompleteController::HandleKeyNavigation(uint32_t aKey, bool* _retval) {
           mCompletedSelectionIndex = selectedIndex;
         } else {
           // Nothing is selected, so fill in the last typed value
-          SetValueOfInputTo(mSearchString,
-                            nsIAutoCompleteInput::TEXTVALUE_REASON_REVERT);
+          SetValueOfInputTo(mSearchString);
           input->SelectTextRange(mSearchString.Length(),
                                  mSearchString.Length());
           mCompletedSelectionIndex = -1;
         }
       }
     } else {
-#ifdef XP_MACOSX
-      // on Mac, only show the popup if the caret is at the start or end of
-      // the input and there is no selection, so that the default defined key
-      // shortcuts for up and down move to the beginning and end of the field
-      // otherwise.
-      int32_t start, end;
-      if (aKey == dom::KeyboardEvent_Binding::DOM_VK_UP) {
+      // Only show the popup if the caret is at the start or end of the input
+      // and there is no selection, so that the default defined key shortcuts
+      // for up and down move to the beginning and end of the field otherwise.
+      if (aKey == dom::KeyboardEvent_Binding::DOM_VK_UP ||
+          aKey == dom::KeyboardEvent_Binding::DOM_VK_DOWN) {
+        const bool isUp = aKey == dom::KeyboardEvent_Binding::DOM_VK_UP;
+
+        int32_t start, end;
         input->GetSelectionStart(&start);
         input->GetSelectionEnd(&end);
-        if (start > 0 || start != end) *_retval = false;
-      } else if (aKey == dom::KeyboardEvent_Binding::DOM_VK_DOWN) {
-        nsAutoString text;
-        input->GetTextValue(text);
-        input->GetSelectionStart(&start);
-        input->GetSelectionEnd(&end);
-        if (start != end || end < (int32_t)text.Length()) *_retval = false;
-      }
-#endif
-      if (*_retval) {
-        nsAutoString oldSearchString;
-        uint16_t oldResult = 0;
 
-        // Open the popup if there has been a previous non-errored search, or
-        // else kick off a new search
-        if (!mResults.IsEmpty() &&
-            NS_SUCCEEDED(mResults[0]->GetSearchResult(&oldResult)) &&
-            oldResult != nsIAutoCompleteResult::RESULT_FAILURE &&
-            NS_SUCCEEDED(mResults[0]->GetSearchString(oldSearchString)) &&
-            oldSearchString.Equals(mSearchString,
-                                   nsCaseInsensitiveStringComparator())) {
-          if (mMatchCount) {
-            OpenPopup();
+        if (isUp) {
+          if (start > 0 || start != end) {
+            return NS_OK;
           }
         } else {
-          // Stop all searches in case they are async.
-          StopSearch();
-
-          if (!mInput) {
-            // StopSearch() can call PostSearchCleanup() which might result
-            // in a blur event, which could null out mInput, so we need to check
-            // it again.  See bug #395344 for more details
+          nsAutoString text;
+          input->GetTextValue(text);
+          if (start != end || end < (int32_t)text.Length()) {
             return NS_OK;
           }
-
-          // Some script may have changed the value of the text field since our
-          // last keypress or after our focus handler and we don't want to
-          // search for a stale string.
-          nsAutoString value;
-          input->GetTextValue(value);
-          SetSearchStringInternal(value);
-
-          StartSearches();
         }
       }
+
+      // Open the popup if there has been a previous non-errored search, or
+      // else kick off a new search
+      bool hadPreviousSearch = false;
+      for (uint32_t i = 0; i < mResults.Length(); ++i) {
+        nsAutoString oldSearchString;
+        uint16_t oldResult = 0;
+        nsIAutoCompleteResult* oldResultObject = mResults[i];
+        if (oldResultObject &&
+            NS_SUCCEEDED(oldResultObject->GetSearchResult(&oldResult)) &&
+            oldResult != nsIAutoCompleteResult::RESULT_FAILURE &&
+            NS_SUCCEEDED(oldResultObject->GetSearchString(oldSearchString)) &&
+            oldSearchString.Equals(mSearchString,
+                                   nsCaseInsensitiveStringComparator)) {
+          hadPreviousSearch = true;
+          break;
+        }
+      }
+      if (hadPreviousSearch) {
+        if (mMatchCount) {
+          OpenPopup();
+        }
+      } else {
+        // Stop all searches in case they are async.
+        StopSearch();
+
+        if (!mInput) {
+          // StopSearch() can call PostSearchCleanup() which might result
+          // in a blur event, which could null out mInput, so we need to check
+          // it again.  See bug #395344 for more details
+          return NS_OK;
+        }
+
+        // Some script may have changed the value of the text field since our
+        // last keypress or after our focus handler and we don't want to
+        // search for a stale string.
+        nsAutoString value;
+        input->GetTextValue(value);
+        SetSearchStringInternal(value);
+
+        StartSearches();
+      }
+
+      bool isOpen = false;
+      input->GetPopupOpen(&isOpen);
+      if (isOpen) {
+        // Prevent the default action if we opened the popup in any of the code
+        // paths above.
+        *_retval = true;
+      }
     }
   } else if (aKey == dom::KeyboardEvent_Binding::DOM_VK_LEFT ||
              aKey == dom::KeyboardEvent_Binding::DOM_VK_RIGHT
@@ -600,17 +604,15 @@ nsAutoCompleteController::HandleKeyNavigation(uint32_t aKey, bool* _retval) {
         input->GetTextValue(inputValue);
         if (NS_SUCCEEDED(GetDefaultCompleteValue(-1, false, value))) {
           nsAutoString suggestedValue;
-          int32_t pos = inputValue.Find(" >> ");
+          int32_t pos = inputValue.Find(u" >> ");
           if (pos > 0) {
             inputValue.Right(suggestedValue, inputValue.Length() - pos - 4);
           } else {
             suggestedValue = inputValue;
           }
 
-          if (value.Equals(suggestedValue,
-                           nsCaseInsensitiveStringComparator())) {
-            SetValueOfInputTo(
-                value, nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETEDEFAULT);
+          if (value.Equals(suggestedValue, nsCaseInsensitiveStringComparator)) {
+            SetValueOfInputTo(value);
             input->SelectTextRange(value.Length(), value.Length());
           }
         }
@@ -619,8 +621,7 @@ nsAutoCompleteController::HandleKeyNavigation(uint32_t aKey, bool* _retval) {
         // The pop-up is open and has a selection, take its value
         nsAutoString value;
         if (NS_SUCCEEDED(GetResultValueAt(selectedIndex, false, value))) {
-          SetValueOfInputTo(
-              value, nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETESELECTED);
+          SetValueOfInputTo(value);
           input->SelectTextRange(value.Length(), value.Length());
         }
       }
@@ -673,11 +674,19 @@ nsAutoCompleteController::HandleDelete(bool* _retval) {
   nsIAutoCompleteResult* result = mResults.SafeObjectAt(searchIndex);
   NS_ENSURE_TRUE(result, NS_ERROR_FAILURE);
 
+  bool removable;
+  nsresult rv = result->IsRemovableAt(matchIndex, &removable);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (!removable) {
+    return NS_OK;
+  }
+
   nsAutoString search;
   input->GetSearchParam(search);
 
   // Clear the match in our result and in the DB.
-  result->RemoveValueAt(matchIndex, true);
+  result->RemoveValueAt(matchIndex);
   --mMatchCount;
 
   // We removed it, so make sure we cancel the event that triggered this call.
@@ -822,13 +831,12 @@ nsAutoCompleteController::OnSearchResult(nsIAutoCompleteSearch* aSearch,
   for (uint32_t i = 0; i < mSearches.Length(); ++i) {
     if (mSearches[i] == aSearch) {
       ProcessResult(i, aResult);
+      break;
     }
   }
 
-  if (mSearchesOngoing == 0) {
-    // If this is the last search to return, cleanup.
-    PostSearchCleanup();
-  }
+  // If a match is found in ProcessResult, PostSearchCleanup will open the popup
+  PostSearchCleanup();
 
   return NS_OK;
 }
@@ -899,11 +907,16 @@ nsresult nsAutoCompleteController::BeforeSearches() {
   mSearchStatus = nsIAutoCompleteController::STATUS_SEARCHING;
   mDefaultIndexCompleted = false;
 
-  // ClearResults will clear the mResults array, but we should pass the previous
-  // result to each search to allow reusing it.  So we temporarily cache the
-  // current results until AfterSearches().
-  if (!mResultCache.AppendObjects(mResults)) {
-    return NS_ERROR_OUT_OF_MEMORY;
+  bool invalidatePreviousResult = false;
+  mInput->GetInvalidatePreviousResult(&invalidatePreviousResult);
+
+  if (!invalidatePreviousResult) {
+    // ClearResults will clear the mResults array, but we should pass the
+    // previous result to each search to allow reusing it.  So we temporarily
+    // cache the current results until AfterSearches().
+    if (!mResultCache.AppendObjects(mResults)) {
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
   }
   ClearResults(true);
   mSearchesOngoing = mSearches.Length();
@@ -966,7 +979,8 @@ nsresult nsAutoCompleteController::StartSearch(uint16_t aSearchType) {
     }
 
     rv = search->StartSearch(mSearchString, searchParam, result,
-                             static_cast<nsIAutoCompleteObserver*>(this));
+                             static_cast<nsIAutoCompleteObserver*>(this),
+                             nullptr);
     if (NS_FAILED(rv)) {
       ++mSearchesFailed;
       MOZ_ASSERT(mSearchesOngoing > 0);
@@ -988,7 +1002,10 @@ nsresult nsAutoCompleteController::StartSearch(uint16_t aSearchType) {
 
 void nsAutoCompleteController::AfterSearches() {
   mResultCache.Clear();
-  if (mSearchesFailed == mSearches.Length()) PostSearchCleanup();
+  // if the below evaluates to true, that means mSearchesOngoing must be 0
+  if (mSearchesFailed == mSearches.Length()) {
+    PostSearchCleanup();
+  }
 }
 
 NS_IMETHODIMP
@@ -1039,7 +1056,7 @@ void nsAutoCompleteController::MaybeCompletePlaceholder() {
       selectionEnd == selectionStart &&
       selectionEnd == (int32_t)mSearchString.Length() &&
       StringBeginsWith(mPlaceholderCompletionString, mSearchString,
-                       nsCaseInsensitiveStringComparator());
+                       nsCaseInsensitiveStringComparator);
 
   if (usePlaceholderCompletion) {
     CompleteValue(mPlaceholderCompletionString);
@@ -1161,6 +1178,8 @@ nsresult nsAutoCompleteController::EnterMatch(bool aIsPopupSelection,
 
   // Ask the popup if it wants to enter a special value into the textbox
   nsAutoString value;
+  nsAutoString comment;
+
   popup->GetOverrideValue(value);
   if (value.IsEmpty()) {
     bool shouldComplete;
@@ -1171,6 +1190,7 @@ nsresult nsAutoCompleteController::EnterMatch(bool aIsPopupSelection,
     if (selectedIndex >= 0) {
       nsAutoString inputValue;
       input->GetTextValue(inputValue);
+      GetCommentAt(selectedIndex, comment);
       if (aIsPopupSelection || !completeSelection) {
         // We need to fill-in the value if:
         //  * completeselectedindex is false
@@ -1178,7 +1198,7 @@ nsresult nsAutoCompleteController::EnterMatch(bool aIsPopupSelection,
         GetResultValueAt(selectedIndex, true, value);
       } else if (mDefaultIndexCompleted &&
                  inputValue.Equals(mPlaceholderCompletionString,
-                                   nsCaseInsensitiveStringComparator())) {
+                                   nsCaseInsensitiveStringComparator)) {
         // We also need to fill-in the value if the default index completion was
         // confirmed, though we cannot use the selectedIndex cause the selection
         // may have been changed by the mouse in the meanwhile.
@@ -1218,7 +1238,7 @@ nsresult nsAutoCompleteController::EnterMatch(bool aIsPopupSelection,
       nsAutoString inputValue;
       input->GetTextValue(inputValue);
       nsAutoString suggestedValue;
-      int32_t pos = inputValue.Find(" >> ");
+      int32_t pos = inputValue.Find(u" >> ");
       if (pos > 0) {
         inputValue.Right(suggestedValue, inputValue.Length() - pos - 4);
       } else {
@@ -1234,7 +1254,7 @@ nsresult nsAutoCompleteController::EnterMatch(bool aIsPopupSelection,
             nsAutoString matchValue;
             result->GetValueAt(j, matchValue);
             if (suggestedValue.Equals(matchValue,
-                                      nsCaseInsensitiveStringComparator())) {
+                                      nsCaseInsensitiveStringComparator)) {
               nsAutoString finalMatchValue;
               result->GetFinalCompleteValueAt(j, finalMatchValue);
               value = finalMatchValue;
@@ -1262,23 +1282,28 @@ nsresult nsAutoCompleteController::EnterMatch(bool aIsPopupSelection,
     }
   }
 
+  if (comment.IsEmpty()) {
+    comment.Assign(u"{}");
+  }
+
   nsCOMPtr<nsIObserverService> obsSvc = services::GetObserverService();
   NS_ENSURE_STATE(obsSvc);
-  obsSvc->NotifyObservers(input, "autocomplete-will-enter-text", nullptr);
+  obsSvc->NotifyObservers(input, "autocomplete-will-enter-text", comment.get());
 
   if (!value.IsEmpty()) {
-    SetValueOfInputTo(value, nsIAutoCompleteInput::TEXTVALUE_REASON_ENTERMATCH);
+    SetValueOfInputTo(value);
     input->SelectTextRange(value.Length(), value.Length());
     SetSearchStringInternal(value);
   }
 
   obsSvc->NotifyObservers(input, "autocomplete-did-enter-text", nullptr);
-  ClosePopup();
 
   bool cancel;
   bool itemWasSelected = selectedIndex >= 0 && !value.IsEmpty();
   input->OnTextEntered(aEvent, itemWasSelected, &cancel);
 
+  ClosePopup();
+
   return NS_OK;
 }
 
@@ -1312,8 +1337,7 @@ nsresult nsAutoCompleteController::RevertTextValue() {
     // events. NOTE: how can |RevertTextValue| be called with inputValue !=
     // oldValue?
     if (mSearchString != currentValue) {
-      SetValueOfInputTo(mSearchString,
-                        nsIAutoCompleteInput::TEXTVALUE_REASON_REVERT);
+      SetValueOfInputTo(mSearchString);
     }
 
     obsSvc->NotifyObservers(input, "autocomplete-did-revert-text", nullptr);
@@ -1327,7 +1351,6 @@ nsresult nsAutoCompleteController::ProcessResult(
   NS_ENSURE_STATE(mInput);
   MOZ_ASSERT(aResult, "ProcessResult should always receive a result");
   NS_ENSURE_ARG(aResult);
-  nsCOMPtr<nsIAutoCompleteInput> input(mInput);
 
   uint16_t searchResult = 0;
   aResult->GetSearchResult(&searchResult);
@@ -1396,18 +1419,6 @@ nsresult nsAutoCompleteController::ProcessResult(
   NS_ENSURE_TRUE(popup != nullptr, NS_ERROR_FAILURE);
   popup->Invalidate(nsIAutoCompletePopup::INVALIDATE_REASON_NEW_RESULT);
 
-  uint32_t minResults;
-  input->GetMinResultsForPopup(&minResults);
-
-  // Make sure the popup is open, if necessary, since we now have at least one
-  // search result ready to display. Don't force the popup closed if we might
-  // get results in the future to avoid unnecessarily canceling searches.
-  if (mMatchCount || !minResults) {
-    OpenPopup();
-  } else if (mSearchesOngoing == 0) {
-    ClosePopup();
-  }
-
   return NS_OK;
 }
 
@@ -1420,17 +1431,17 @@ nsresult nsAutoCompleteController::PostSearchCleanup() {
 
   if (mMatchCount || minResults == 0) {
     OpenPopup();
-    if (mMatchCount)
-      mSearchStatus = nsIAutoCompleteController::STATUS_COMPLETE_MATCH;
-    else
-      mSearchStatus = nsIAutoCompleteController::STATUS_COMPLETE_NO_MATCH;
-  } else {
-    mSearchStatus = nsIAutoCompleteController::STATUS_COMPLETE_NO_MATCH;
+  } else if (mSearchesOngoing == 0) {
     ClosePopup();
   }
 
-  // notify the input that the search is complete
-  input->OnSearchComplete();
+  if (mSearchesOngoing == 0) {
+    mSearchStatus = mMatchCount
+                        ? nsIAutoCompleteController::STATUS_COMPLETE_MATCH
+                        : nsIAutoCompleteController::STATUS_COMPLETE_NO_MATCH;
+    // notify the input that the search is complete
+    input->OnSearchComplete();
+  }
 
   return NS_OK;
 }
@@ -1466,7 +1477,7 @@ nsresult nsAutoCompleteController::CompleteDefaultIndex(int32_t aResultIndex) {
       selectionEnd == (int32_t)mPlaceholderCompletionString.Length() &&
       selectionStart == (int32_t)mSearchString.Length() &&
       StringBeginsWith(mPlaceholderCompletionString, mSearchString,
-                       nsCaseInsensitiveStringComparator());
+                       nsCaseInsensitiveStringComparator);
 
   // Don't try to automatically complete to the first result if there's already
   // a selection or the cursor isn't at the end of the input. In case the
@@ -1493,8 +1504,7 @@ nsresult nsAutoCompleteController::CompleteDefaultIndex(int32_t aResultIndex) {
     nsAutoString inputValue;
     input->GetTextValue(inputValue);
     if (!inputValue.Equals(mSearchString)) {
-      SetValueOfInputTo(mSearchString,
-                        nsIAutoCompleteInput::TEXTVALUE_REASON_REVERT);
+      SetValueOfInputTo(mSearchString);
       input->SelectTextRange(mSearchString.Length(), mSearchString.Length());
     }
     mPlaceholderCompletionString.Truncate();
@@ -1559,9 +1569,8 @@ nsresult nsAutoCompleteController::GetDefaultCompleteValue(int32_t aResultIndex,
 
   nsAutoString resultValue;
   result->GetValueAt(defaultIndex, resultValue);
-  if (aPreserveCasing &&
-      StringBeginsWith(resultValue, mSearchString,
-                       nsCaseInsensitiveStringComparator())) {
+  if (aPreserveCasing && StringBeginsWith(resultValue, mSearchString,
+                                          nsCaseInsensitiveStringComparator)) {
     // We try to preserve user casing, otherwise we would end up changing
     // the case of what he typed, if we have a result with a different casing.
     // For example if we have result "Test", and user starts writing "tuna",
@@ -1593,7 +1602,7 @@ nsresult nsAutoCompleteController::GetFinalDefaultCompleteValue(
   result->GetValueAt(defaultIndex, _retval);
   nsAutoString inputValue;
   input->GetTextValue(inputValue);
-  if (!_retval.Equals(inputValue, nsCaseInsensitiveStringComparator())) {
+  if (!_retval.Equals(inputValue, nsCaseInsensitiveStringComparator)) {
     return NS_ERROR_FAILURE;
   }
 
@@ -1617,15 +1626,13 @@ nsresult nsAutoCompleteController::CompleteValue(nsString& aValue)
   const int32_t mSearchStringLength = mSearchString.Length();
   int32_t endSelect = aValue.Length();  // By default, select all of aValue.
 
-  if (aValue.IsEmpty() ||
-      StringBeginsWith(aValue, mSearchString,
-                       nsCaseInsensitiveStringComparator())) {
+  if (aValue.IsEmpty() || StringBeginsWith(aValue, mSearchString,
+                                           nsCaseInsensitiveStringComparator)) {
     // aValue is empty (we were asked to clear mInput), or mSearchString
     // matches the beginning of aValue.  In either case we can simply
     // autocomplete to aValue.
     mPlaceholderCompletionString = aValue;
-    SetValueOfInputTo(aValue,
-                      nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETEDEFAULT);
+    SetValueOfInputTo(aValue);
   } else {
     nsresult rv;
     nsCOMPtr<nsIIOService> ios = do_GetService(NS_IOSERVICE_CONTRACTID, &rv);
@@ -1642,23 +1649,21 @@ nsresult nsAutoCompleteController::CompleteValue(nsString& aValue)
       if ((endSelect < findIndex + mSearchStringLength) ||
           !scheme.EqualsLiteral("http") ||
           !Substring(aValue, findIndex, mSearchStringLength)
-               .Equals(mSearchString, nsCaseInsensitiveStringComparator())) {
+               .Equals(mSearchString, nsCaseInsensitiveStringComparator)) {
         return NS_OK;
       }
 
       mPlaceholderCompletionString =
           mSearchString +
           Substring(aValue, mSearchStringLength + findIndex, endSelect);
-      SetValueOfInputTo(mPlaceholderCompletionString,
-                        nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETEDEFAULT);
+      SetValueOfInputTo(mPlaceholderCompletionString);
 
       endSelect -= findIndex;  // We're skipping this many characters of aValue.
     } else {
       // Autocompleting something other than a URI from the middle.
       // Use the format "searchstring >> full string" to indicate to the user
       // what we are going to replace their search string with.
-      SetValueOfInputTo(mSearchString + NS_LITERAL_STRING(" >> ") + aValue,
-                        nsIAutoCompleteInput::TEXTVALUE_REASON_COMPLETEDEFAULT);
+      SetValueOfInputTo(mSearchString + u" >> "_ns + aValue);
 
       endSelect = mSearchString.Length() + 4 + aValue.Length();
 
