@@ -2,10 +2,8 @@
 /* vim: set sts=2 sw=2 et tw=80: */
 "use strict";
 
-PromiseTestUtils.whitelistRejectionsGlobally(/Message manager disconnected/);
-
-const { Downloads } = ChromeUtils.import(
-  "resource://gre/modules/Downloads.jsm"
+const { Downloads } = ChromeUtils.importESModule(
+  "resource://gre/modules/Downloads.sys.mjs"
 );
 
 const server = createHttpServer();
@@ -182,7 +180,7 @@ add_task(async function test_search() {
     equal(item.status, "success", "search() succeeded");
     equal(item.downloads.length, 1, "search() found exactly 1 download");
 
-    Object.keys(expect).forEach(function(field) {
+    Object.keys(expect).forEach(function (field) {
       equal(
         item.downloads[0][field],
         expect[field],
@@ -260,9 +258,7 @@ add_task(async function test_search() {
         equal(
           id,
           downloadIds[expected[idx]],
-          `search() for ${description} returned ${
-            expected[idx]
-          } in position ${idx}`
+          `search() for ${description} returned ${expected[idx]} in position ${idx}`
         );
       });
     } else {
@@ -635,7 +631,9 @@ add_task(async function test_inprogress() {
     resumePromise = new Promise(resolve => {
       resume = resolve;
     });
-  server.registerPathHandler("/slow", async (request, response) => {
+  let hit = false;
+  server.registerPathHandler("/data/slow", async (request, response) => {
+    hit = true;
     response.processAsync();
     await resumePromise;
     response.setHeader("Content-type", "text/plain");
@@ -664,7 +662,7 @@ add_task(async function test_inprogress() {
         );
 
         browser.downloads.onChanged.addListener(info => {
-          if (info.id == id && info.state.current == "complete") {
+          if (info.id == id && info.state && info.state.current == "complete") {
             browser.test.notifyPass("done");
           }
         });
@@ -680,4 +678,5 @@ add_task(async function test_inprogress() {
   resume();
   await extension.awaitFinish("done");
   await extension.unload();
+  Assert.ok(hit, "slow path was actually hit");
 });
