@@ -7,7 +7,7 @@ async function focusWindow(win) {
   let promise = new Promise(resolve => {
     win.addEventListener(
       "focus",
-      function() {
+      function () {
         resolve();
       },
       { capture: true, once: true }
@@ -19,10 +19,10 @@ async function focusWindow(win) {
 
 function getDialogDoc() {
   // Trudge through all the open windows, until we find the one
-  // that has either commonDialog.xul or selectDialog.xul loaded.
+  // that has either commonDialog.xhtml or selectDialog.xhtml loaded.
   // var enumerator = Services.wm.getEnumerator("navigator:browser");
   for (let { docShell } of Services.wm.getEnumerator(null)) {
-    var containedDocShells = docShell.getDocShellEnumerator(
+    var containedDocShells = docShell.getAllDocShellsInSubtree(
       docShell.typeChrome,
       docShell.ENUMERATE_FORWARDS
     );
@@ -34,8 +34,9 @@ function getDialogDoc() {
       }
       var childDoc = childDocShell.contentViewer.DOMDocument;
       if (
-        childDoc.location.href != "chrome://global/content/commonDialog.xul" &&
-        childDoc.location.href != "chrome://global/content/selectDialog.xul"
+        childDoc.location.href !=
+          "chrome://global/content/commonDialog.xhtml" &&
+        childDoc.location.href != "chrome://global/content/selectDialog.xhtml"
       ) {
         continue;
       }
@@ -53,51 +54,6 @@ function getDialogDoc() {
   return null;
 }
 
-async function submitForm(browser, formAction, selectorValues) {
-  function contentSubmitForm([contentFormAction, contentSelectorValues]) {
-    let doc = content.document;
-    let form = doc.getElementById("form");
-    form.action = contentFormAction;
-    for (let [sel, value] of Object.entries(contentSelectorValues)) {
-      try {
-        doc.querySelector(sel).value = value;
-      } catch (ex) {
-        throw new Error(`submitForm: Couldn't set value of field at: ${sel}`);
-      }
-    }
-    form.submit();
-  }
-  await ContentTask.spawn(
-    browser,
-    [formAction, selectorValues],
-    contentSubmitForm
-  );
-  let result = await getResponseResult(browser, formAction);
-  return result;
-}
-
-async function getResponseResult(browser, resultUrl) {
-  let fieldValues = await ContentTask.spawn(
-    browser,
-    [resultUrl],
-    async function(contentResultUrl) {
-      await ContentTaskUtils.waitForCondition(() => {
-        return (
-          content.location.pathname.endsWith(contentResultUrl) &&
-          content.document.readyState == "complete"
-        );
-      }, `Wait for form submission load (${contentResultUrl})`);
-      let username = content.document.getElementById("user").textContent;
-      let password = content.document.getElementById("pass").textContent;
-      return {
-        username,
-        password,
-      };
-    }
-  );
-  return fieldValues;
-}
-
 async function waitForAuthPrompt() {
   let promptDoc = await TestUtils.waitForCondition(() => {
     return getAuthPrompt();
@@ -116,16 +72,15 @@ function getAuthPrompt() {
 
 async function loadAccessRestrictedURL(browser, url, username, password) {
   let browserLoaded = BrowserTestUtils.browserLoaded(browser);
-  BrowserTestUtils.loadURI(browser, url);
+  BrowserTestUtils.loadURIString(browser, url);
 
-  let promptDoc = await waitForAuthPrompt();
-  let dialogUI = promptDoc.defaultView.Dialog.ui;
-  ok(dialogUI, "Got expected HTTP auth dialog Dialog.ui");
+  // Wait for the auth prompt, enter the login details and close the prompt
+  await PromptTestUtils.handleNextPrompt(
+    browser,
+    { modalType: authPromptModalType, promptType: "promptUserAndPass" },
+    { buttonNumClick: 0, loginInput: username, passwordInput: password }
+  );
 
-  // fill and submit the dialog form
-  dialogUI.loginTextbox.value = username;
-  dialogUI.password1Textbox.value = password;
-  promptDoc.getElementById("commonDialog").acceptDialog();
   await SimpleTest.promiseFocus(browser.ownerGlobal);
   await browserLoaded;
 }
@@ -146,19 +101,21 @@ let login = new nsLoginInfo(
   "pass"
 );
 const form1Url = `https://example.com/${DIRECTORY_PATH}subtst_privbrowsing_1.html`;
-const form2Url = `https://example.com/${DIRECTORY_PATH}subtst_privbrowsing_2.html`;
+const form2Url = `https://example.com/${DIRECTORY_PATH}form_password_change.html`;
 const authUrl = `https://example.com/${DIRECTORY_PATH}authenticate.sjs`;
 
 let normalWin;
 let privateWin;
+let authPromptModalType;
 
 // XXX: Note that tasks are currently run in sequence. Some tests may assume the state
 // resulting from successful or unsuccessful logins in previous tasks
 
 add_task(async function test_setup() {
+  authPromptModalType = Services.prefs.getIntPref("prompts.modalType.httpAuth");
   normalWin = await BrowserTestUtils.openNewBrowserWindow({ private: false });
   privateWin = await BrowserTestUtils.openNewBrowserWindow({ private: true });
-  Services.logins.removeAllLogins();
+  Services.logins.removeAllUserFacingLogins();
 });
 
 add_task(async function test_normal_popup_notification_1() {
@@ -169,26 +126,38 @@ add_task(async function test_normal_popup_notification_1() {
       gBrowser: normalWin.gBrowser,
       url: form1Url,
     },
-    async function(browser) {
-      let fieldValues = await submitForm(browser, "formsubmit.sjs", {
-        "#user": "notifyu1",
-        "#pass": "notifyp1",
-      });
-      is(fieldValues.username, "notifyu1", "Checking submitted username");
-      is(fieldValues.password, "notifyp1", "Checking submitted password");
+    async function (browser) {
+      let fieldValues = await submitFormAndGetResults(
+        browser,
+        "formsubmit.sjs",
+        {
+          "#user": "notifyu1",
+          "#pass": "notifyp1",
+        }
+      );
+      Assert.equal(
+        fieldValues.username,
+        "notifyu1",
+        "Checking submitted username"
+      );
+      Assert.equal(
+        fieldValues.password,
+        "notifyp1",
+        "Checking submitted password"
+      );
 
       let notif = getCaptureDoorhanger(
         "password-save",
         PopupNotifications,
         browser
       );
-      ok(notif, "got notification popup");
+      Assert.ok(notif, "got notification popup");
       if (notif) {
         await TestUtils.waitForCondition(
           () => !notif.dismissed,
           "notification should not be dismissed"
         );
-        notif.remove();
+        await cleanupDoorhanger(notif);
       }
     }
   );
@@ -202,43 +171,90 @@ add_task(async function test_private_popup_notification_2() {
   const capturePrefValue = Services.prefs.getBoolPref(
     PRIVATE_BROWSING_CAPTURE_PREF
   );
-  ok(
+  Assert.ok(
     capturePrefValue,
     `Expect ${PRIVATE_BROWSING_CAPTURE_PREF} to default to true`
   );
 
   // clear existing logins for parity with the previous test
-  Services.logins.removeAllLogins();
+  Services.logins.removeAllUserFacingLogins();
   await focusWindow(privateWin);
   await BrowserTestUtils.withNewTab(
     {
       gBrowser: privateWin.gBrowser,
       url: form1Url,
     },
-    async function(browser) {
-      let fieldValues = await submitForm(browser, "formsubmit.sjs", {
-        "#user": "notifyu1",
-        "#pass": "notifyp1",
-      });
-      is(fieldValues.username, "notifyu1", "Checking submitted username");
-      is(fieldValues.password, "notifyp1", "Checking submitted password");
+    async function (browser) {
+      let fieldValues = await submitFormAndGetResults(
+        browser,
+        "formsubmit.sjs",
+        {
+          "#user": "notifyu1",
+          "#pass": "notifyp1",
+        }
+      );
+      Assert.equal(
+        fieldValues.username,
+        "notifyu1",
+        "Checking submitted username"
+      );
+      Assert.equal(
+        fieldValues.password,
+        "notifyp1",
+        "Checking submitted password"
+      );
 
       let notif = getCaptureDoorhanger(
         "password-save",
         PopupNotifications,
         browser
       );
-      ok(notif, "Expected notification popup");
+      Assert.ok(notif, "Expected notification popup");
       if (notif) {
         await TestUtils.waitForCondition(
           () => notif.dismissed,
           "notification should be dismissed"
         );
-        notif.remove();
+
+        let { panel } = privateWin.PopupNotifications;
+        let promiseShown = BrowserTestUtils.waitForEvent(panel, "popupshown");
+        notif.anchorElement.click();
+        await promiseShown;
+
+        let notificationElement = panel.childNodes[0];
+        let toggleCheckbox = notificationElement.querySelector(
+          "#password-notification-visibilityToggle"
+        );
+
+        Assert.ok(
+          !toggleCheckbox.hidden,
+          "Toggle should be visible upon 1st opening"
+        );
+
+        info("Hiding popup.");
+        let promiseHidden = BrowserTestUtils.waitForEvent(panel, "popuphidden");
+        panel.hidePopup();
+        await promiseHidden;
+
+        info("Clicking on anchor to reshow popup.");
+        promiseShown = BrowserTestUtils.waitForEvent(panel, "popupshown");
+        notif.anchorElement.click();
+        await promiseShown;
+
+        Assert.ok(
+          toggleCheckbox.hidden,
+          "Toggle should be hidden upon 2nd opening"
+        );
+
+        await cleanupDoorhanger(notif);
       }
     }
   );
-  is(Services.logins.getAllLogins().length, 0, "No logins were saved");
+  Assert.equal(
+    Services.logins.getAllLogins().length,
+    0,
+    "No logins were saved"
+  );
 });
 
 add_task(async function test_private_popup_notification_no_capture_pref_2b() {
@@ -253,7 +269,7 @@ add_task(async function test_private_popup_notification_no_capture_pref_2b() {
   Services.prefs.setBoolPref(PRIVATE_BROWSING_CAPTURE_PREF, false);
 
   // clear existing logins for parity with the previous test
-  Services.logins.removeAllLogins();
+  Services.logins.removeAllUserFacingLogins();
 
   await focusWindow(privateWin);
   await BrowserTestUtils.withNewTab(
@@ -261,13 +277,25 @@ add_task(async function test_private_popup_notification_no_capture_pref_2b() {
       gBrowser: privateWin.gBrowser,
       url: form1Url,
     },
-    async function(browser) {
-      let fieldValues = await submitForm(browser, "formsubmit.sjs", {
-        "#user": "notifyu1",
-        "#pass": "notifyp1",
-      });
-      is(fieldValues.username, "notifyu1", "Checking submitted username");
-      is(fieldValues.password, "notifyp1", "Checking submitted password");
+    async function (browser) {
+      let fieldValues = await submitFormAndGetResults(
+        browser,
+        "formsubmit.sjs",
+        {
+          "#user": "notifyu1",
+          "#pass": "notifyp1",
+        }
+      );
+      Assert.equal(
+        fieldValues.username,
+        "notifyu1",
+        "Checking submitted username"
+      );
+      Assert.equal(
+        fieldValues.password,
+        "notifyp1",
+        "Checking submitted password"
+      );
 
       let notif = getCaptureDoorhanger(
         "password-save",
@@ -280,13 +308,17 @@ add_task(async function test_private_popup_notification_no_capture_pref_2b() {
         capturePrefValue
       );
 
-      ok(!notif, "Expected no notification popup");
+      Assert.ok(!notif, "Expected no notification popup");
       if (notif) {
-        notif.remove();
+        await cleanupDoorhanger(notif);
       }
     }
   );
-  is(Services.logins.getAllLogins().length, 0, "No logins were saved");
+  Assert.equal(
+    Services.logins.getAllLogins().length,
+    0,
+    "No logins were saved"
+  );
 });
 
 add_task(async function test_normal_popup_notification_3() {
@@ -295,11 +327,11 @@ add_task(async function test_normal_popup_notification_3() {
       "match existing username/password: no popup notification should appear"
   );
 
-  Services.logins.removeAllLogins();
-  Services.logins.addLogin(login);
+  Services.logins.removeAllUserFacingLogins();
+  await Services.logins.addLoginAsync(login);
   let allLogins = Services.logins.getAllLogins();
   // Sanity check the HTTP login exists.
-  is(allLogins.length, 1, "Should have the HTTP login");
+  Assert.equal(allLogins.length, 1, "Should have the HTTP login");
   let timeLastUsed = allLogins[0].timeLastUsed;
   let loginGuid = allLogins[0].guid;
 
@@ -309,32 +341,40 @@ add_task(async function test_normal_popup_notification_3() {
       gBrowser: normalWin.gBrowser,
       url: form1Url,
     },
-    async function(browser) {
-      let fieldValues = await submitForm(browser, "formsubmit.sjs", {
-        "#user": "notifyu1",
-        "#pass": "notifyp1",
-      });
-      is(fieldValues.username, "notifyu1", "Checking submitted username");
-      is(fieldValues.password, "notifyp1", "Checking submitted password");
-
-      let notif = getCaptureDoorhanger(
-        "password-save",
-        PopupNotifications,
-        browser
+    async function (browser) {
+      let fieldValues = await submitFormAndGetResults(
+        browser,
+        "formsubmit.sjs",
+        {
+          "#user": "notifyu1",
+          "#pass": "notifyp1",
+        }
       );
-      ok(!notif, "got no notification popup");
+      Assert.equal(
+        fieldValues.username,
+        "notifyu1",
+        "Checking submitted username"
+      );
+      Assert.equal(
+        fieldValues.password,
+        "notifyp1",
+        "Checking submitted password"
+      );
+
+      let notif = getCaptureDoorhanger("any", PopupNotifications, browser);
+      Assert.ok(!notif, "got no notification popup");
       if (notif) {
-        notif.remove();
+        await cleanupDoorhanger(notif);
       }
     }
   );
   allLogins = Services.logins.getAllLogins();
-  is(
+  Assert.equal(
     allLogins[0].guid,
     loginGuid,
     "Sanity-check we are comparing the same login record"
   );
-  ok(
+  Assert.ok(
     allLogins[0].timeLastUsed > timeLastUsed,
     "The timeLastUsed timestamp has been updated"
   );
@@ -346,11 +386,11 @@ add_task(async function test_private_popup_notification_3b() {
       " match existing username/password: no popup notification should appear"
   );
 
-  Services.logins.removeAllLogins();
-  Services.logins.addLogin(login);
+  Services.logins.removeAllUserFacingLogins();
+  await Services.logins.addLoginAsync(login);
   let allLogins = Services.logins.getAllLogins();
   // Sanity check the HTTP login exists.
-  is(allLogins.length, 1, "Should have the HTTP login");
+  Assert.equal(allLogins.length, 1, "Should have the HTTP login");
   let timeLastUsed = allLogins[0].timeLastUsed;
   let loginGuid = allLogins[0].guid;
 
@@ -360,32 +400,41 @@ add_task(async function test_private_popup_notification_3b() {
       gBrowser: privateWin.gBrowser,
       url: form1Url,
     },
-    async function(browser) {
-      let fieldValues = await submitForm(browser, "formsubmit.sjs", {
-        "#user": "notifyu1",
-        "#pass": "notifyp1",
-      });
-      is(fieldValues.username, "notifyu1", "Checking submitted username");
-      is(fieldValues.password, "notifyp1", "Checking submitted password");
-
-      let notif = getCaptureDoorhanger(
-        "password-save",
-        PopupNotifications,
-        browser
+    async function (browser) {
+      let fieldValues = await submitFormAndGetResults(
+        browser,
+        "formsubmit.sjs",
+        {
+          "#user": "notifyu1",
+          "#pass": "notifyp1",
+        }
       );
-      ok(!notif, "got no notification popup");
+      Assert.equal(
+        fieldValues.username,
+        "notifyu1",
+        "Checking submitted username"
+      );
+      Assert.equal(
+        fieldValues.password,
+        "notifyp1",
+        "Checking submitted password"
+      );
+
+      let notif = getCaptureDoorhanger("any", PopupNotifications, browser);
+
+      Assert.ok(!notif, "got no notification popup");
       if (notif) {
-        notif.remove();
+        await cleanupDoorhanger(notif);
       }
     }
   );
   allLogins = Services.logins.getAllLogins();
-  is(
+  Assert.equal(
     allLogins[0].guid,
     loginGuid,
     "Sanity-check we are comparing the same login record"
   );
-  is(
+  Assert.equal(
     allLogins[0].timeLastUsed,
     timeLastUsed,
     "The timeLastUsed timestamp has not been updated"
@@ -397,11 +446,11 @@ add_task(async function test_normal_new_password_4() {
     "test 4: run with a login, outside of private mode," +
       " add a new password: popup notification should appear"
   );
-  Services.logins.removeAllLogins();
-  Services.logins.addLogin(login);
+  Services.logins.removeAllUserFacingLogins();
+  await Services.logins.addLoginAsync(login);
   let allLogins = Services.logins.getAllLogins();
   // Sanity check the HTTP login exists.
-  is(allLogins.length, 1, "Should have the HTTP login");
+  Assert.equal(allLogins.length, 1, "Should have the HTTP login");
   let timeLastUsed = allLogins[0].timeLastUsed;
   let loginGuid = allLogins[0].guid;
 
@@ -411,36 +460,44 @@ add_task(async function test_normal_new_password_4() {
       gBrowser: normalWin.gBrowser,
       url: form2Url,
     },
-    async function(browser) {
-      let fieldValues = await submitForm(browser, "formsubmit.sjs", {
-        "#pass": "notifyp1",
-        "#newpass": "notifyp2",
-      });
-      is(fieldValues.password, "notifyp1", "Checking submitted password");
+    async function (browser) {
+      let fieldValues = await submitFormAndGetResults(
+        browser,
+        "formsubmit.sjs",
+        {
+          "#pass": "notifyp1",
+          "#newpass": "notifyp2",
+        }
+      );
+      Assert.equal(
+        fieldValues.password,
+        "notifyp1",
+        "Checking submitted password"
+      );
       let notif = getCaptureDoorhanger(
         "password-change",
         PopupNotifications,
         browser
       );
-      ok(notif, "got notification popup");
+      Assert.ok(notif, "got notification popup");
       if (notif) {
         await TestUtils.waitForCondition(
           () => !notif.dismissed,
           "notification should not be dismissed"
         );
-        notif.remove();
+        await cleanupDoorhanger(notif);
       }
     }
   );
   // We put up a doorhanger, but didn't interact with it, so we expect the login timestamps
   // to be unchanged
   allLogins = Services.logins.getAllLogins();
-  is(
+  Assert.equal(
     allLogins[0].guid,
     loginGuid,
     "Sanity-check we are comparing the same login record"
   );
-  is(
+  Assert.equal(
     allLogins[0].timeLastUsed,
     timeLastUsed,
     "The timeLastUsed timestamp was not updated"
@@ -456,14 +513,14 @@ add_task(async function test_private_new_password_5() {
   const capturePrefValue = Services.prefs.getBoolPref(
     PRIVATE_BROWSING_CAPTURE_PREF
   );
-  ok(
+  Assert.ok(
     capturePrefValue,
     `Expect ${PRIVATE_BROWSING_CAPTURE_PREF} to default to true`
   );
 
   let allLogins = Services.logins.getAllLogins();
   // Sanity check the HTTP login exists.
-  is(allLogins.length, 1, "Should have the HTTP login");
+  Assert.equal(allLogins.length, 1, "Should have the HTTP login");
   let timeLastUsed = allLogins[0].timeLastUsed;
   let loginGuid = allLogins[0].guid;
 
@@ -473,36 +530,44 @@ add_task(async function test_private_new_password_5() {
       gBrowser: privateWin.gBrowser,
       url: form2Url,
     },
-    async function(browser) {
-      let fieldValues = await submitForm(browser, "formsubmit.sjs", {
-        "#pass": "notifyp1",
-        "#newpass": "notifyp2",
-      });
-      is(fieldValues.password, "notifyp1", "Checking submitted password");
+    async function (browser) {
+      let fieldValues = await submitFormAndGetResults(
+        browser,
+        "formsubmit.sjs",
+        {
+          "#pass": "notifyp1",
+          "#newpass": "notifyp2",
+        }
+      );
+      Assert.equal(
+        fieldValues.password,
+        "notifyp1",
+        "Checking submitted password"
+      );
       let notif = getCaptureDoorhanger(
         "password-change",
         PopupNotifications,
         browser
       );
-      ok(notif, "Expected notification popup");
+      Assert.ok(notif, "Expected notification popup");
       if (notif) {
         await TestUtils.waitForCondition(
           () => !notif.dismissed,
           "notification should not be dismissed"
         );
-        notif.remove();
+        await cleanupDoorhanger(notif);
       }
     }
   );
   // We put up a doorhanger, but didn't interact with it, so we expect the login timestamps
   // to be unchanged
   allLogins = Services.logins.getAllLogins();
-  is(
+  Assert.equal(
     allLogins[0].guid,
     loginGuid,
     "Sanity-check we are comparing the same login record"
   );
-  is(
+  Assert.equal(
     allLogins[0].timeLastUsed,
     timeLastUsed,
     "The timeLastUsed timestamp has not been updated"
@@ -521,24 +586,32 @@ add_task(async function test_normal_with_login_6() {
       gBrowser: normalWin.gBrowser,
       url: form2Url,
     },
-    async function(browser) {
-      let fieldValues = await submitForm(browser, "formsubmit.sjs", {
-        "#pass": "notifyp1",
-        "#newpass": "notifyp2",
-      });
-      is(fieldValues.password, "notifyp1", "Checking submitted password");
+    async function (browser) {
+      let fieldValues = await submitFormAndGetResults(
+        browser,
+        "formsubmit.sjs",
+        {
+          "#pass": "notifyp1",
+          "#newpass": "notifyp2",
+        }
+      );
+      Assert.equal(
+        fieldValues.password,
+        "notifyp1",
+        "Checking submitted password"
+      );
       let notif = getCaptureDoorhanger(
         "password-change",
         PopupNotifications,
         browser
       );
-      ok(notif, "got notification popup");
+      Assert.ok(notif, "got notification popup");
       if (notif) {
         await TestUtils.waitForCondition(
           () => !notif.dismissed,
           "notification should not be dismissed"
         );
-        notif.remove();
+        await cleanupDoorhanger(notif);
       }
       Services.logins.removeLogin(login);
     }
@@ -547,10 +620,14 @@ add_task(async function test_normal_with_login_6() {
 
 add_task(async function test_normal_autofilled_7() {
   info("test 7: verify that the user/pass pair was autofilled");
-  Services.logins.addLogin(login);
+  await Services.logins.addLoginAsync(login);
 
   // Sanity check the HTTP login exists.
-  is(Services.logins.getAllLogins().length, 1, "Should have the HTTP login");
+  Assert.equal(
+    Services.logins.getAllLogins().length,
+    1,
+    "Should have the HTTP login"
+  );
 
   await focusWindow(normalWin);
   await BrowserTestUtils.withNewTab(
@@ -558,23 +635,29 @@ add_task(async function test_normal_autofilled_7() {
       gBrowser: normalWin.gBrowser,
       url: "about:blank",
     },
-    async function(browser) {
+    async function (browser) {
       // Add the observer before loading the form page
-      let formFilled = ContentTask.spawn(browser, null, async function() {
-        const { TestUtils } = ChromeUtils.import(
-          "resource://testing-common/TestUtils.jsm"
-        );
-        await TestUtils.topicObserved("passwordmgr-processed-form");
-        await Promise.resolve();
-      });
+      let formFilled = listenForTestNotification("FormProcessed");
       await SimpleTest.promiseFocus(browser.ownerGlobal);
-      await BrowserTestUtils.loadURI(browser, form1Url);
+      BrowserTestUtils.loadURIString(browser, form1Url);
       await formFilled;
 
       // the form should have been autofilled, so submit without updating field values
-      let fieldValues = await submitForm(browser, "formsubmit.sjs", {});
-      is(fieldValues.username, "notifyu1", "Checking submitted username");
-      is(fieldValues.password, "notifyp1", "Checking submitted password");
+      let fieldValues = await submitFormAndGetResults(
+        browser,
+        "formsubmit.sjs",
+        {}
+      );
+      Assert.equal(
+        fieldValues.username,
+        "notifyu1",
+        "Checking submitted username"
+      );
+      Assert.equal(
+        fieldValues.password,
+        "notifyp1",
+        "Checking submitted password"
+      );
     }
   );
 });
@@ -582,7 +665,13 @@ add_task(async function test_normal_autofilled_7() {
 add_task(async function test_private_not_autofilled_8() {
   info("test 8: verify that the user/pass pair was not autofilled");
   // Sanity check the HTTP login exists.
-  is(Services.logins.getAllLogins().length, 1, "Should have the HTTP login");
+  Assert.equal(
+    Services.logins.getAllLogins().length,
+    1,
+    "Should have the HTTP login"
+  );
+
+  let formFilled = listenForTestNotification("FormProcessed");
 
   await focusWindow(privateWin);
   await BrowserTestUtils.withNewTab(
@@ -590,10 +679,15 @@ add_task(async function test_private_not_autofilled_8() {
       gBrowser: privateWin.gBrowser,
       url: form1Url,
     },
-    async function(browser) {
-      let fieldValues = await submitForm(browser, "formsubmit.sjs", {});
-      ok(!fieldValues.username, "Checking submitted username");
-      ok(!fieldValues.password, "Checking submitted password");
+    async function (browser) {
+      await formFilled;
+      let fieldValues = await submitFormAndGetResults(
+        browser,
+        "formsubmit.sjs",
+        {}
+      );
+      Assert.ok(!fieldValues.username, "Checking submitted username");
+      Assert.ok(!fieldValues.password, "Checking submitted password");
     }
   );
 });
@@ -602,7 +696,7 @@ add_task(async function test_private_not_autofilled_8() {
 // add_task(async function test_private_autocomplete_9() {
 //   info("test 9: verify that the user/pass pair was available for autocomplete");
 //   // Sanity check the HTTP login exists.
-//   is(Services.logins.getAllLogins().length, 1, "Should have the HTTP login");
+//   Assert.equal(Services.logins.getAllLogins().length, 1, "Should have the HTTP login");
 
 //   await focusWindow(privateWin);
 //   await BrowserTestUtils.withNewTab({
@@ -610,7 +704,7 @@ add_task(async function test_private_not_autofilled_8() {
 //     url: form1Url,
 //   }, async function(browser) {
 //     let popup = document.getElementById("PopupAutoComplete");
-//     ok(popup, "Got popup");
+//     Assert.ok(popup, "Got popup");
 
 //     let promiseShown = BrowserTestUtils.waitForEvent(popup, "popupshown");
 
@@ -619,7 +713,7 @@ add_task(async function test_private_not_autofilled_8() {
 //       content.document.getElementById("user").focus();
 //     });
 //     await promiseShown;
-//     ok(promiseShown, "autocomplete shown");
+//     Assert.ok(promiseShown, "autocomplete shown");
 
 //     let promiseFormInput = ContentTask.spawn(browser, null, async function() {
 //       let doc = content.document;
@@ -634,9 +728,9 @@ add_task(async function test_private_not_autofilled_8() {
 //     await BrowserTestUtils.synthesizeKey("VK_RETURN", {}, browser);
 //     await promiseFormInput;
 
-//     let fieldValues = await submitForm(browser, "formsubmit.sjs", {});
-//     is(fieldValues.username, "notifyu1", "Checking submitted username");
-//     is(fieldValues.password, "notifyp1", "Checking submitted password");
+//     let fieldValues = await submitFormAndGetResults(browser, "formsubmit.sjs", {});
+//     Assert.equal(fieldValues.username, "notifyu1", "Checking submitted username");
+//     Assert.equal(fieldValues.password, "notifyp1", "Checking submitted password");
 //   });
 // });
 
@@ -645,7 +739,13 @@ add_task(async function test_normal_autofilled_10() {
     "test 10: verify that the user/pass pair does get autofilled in non-private window"
   );
   // Sanity check the HTTP login exists.
-  is(Services.logins.getAllLogins().length, 1, "Should have the HTTP login");
+  Assert.equal(
+    Services.logins.getAllLogins().length,
+    1,
+    "Should have the HTTP login"
+  );
+
+  let formFilled = listenForTestNotification("FormProcessed");
 
   await focusWindow(normalWin);
   await BrowserTestUtils.withNewTab(
@@ -653,10 +753,23 @@ add_task(async function test_normal_autofilled_10() {
       gBrowser: normalWin.gBrowser,
       url: form1Url,
     },
-    async function(browser) {
-      let fieldValues = await submitForm(browser, "formsubmit.sjs", {});
-      is(fieldValues.username, "notifyu1", "Checking submitted username");
-      is(fieldValues.password, "notifyp1", "Checking submitted password");
+    async function (browser) {
+      await formFilled;
+      let fieldValues = await submitFormAndGetResults(
+        browser,
+        "formsubmit.sjs",
+        {}
+      );
+      Assert.equal(
+        fieldValues.username,
+        "notifyu1",
+        "Checking submitted username"
+      );
+      Assert.equal(
+        fieldValues.password,
+        "notifyp1",
+        "Checking submitted password"
+      );
     }
   );
 });
@@ -665,7 +778,7 @@ add_task(async function test_normal_http_basic_auth() {
   info(
     "test normal/basic-auth: verify that we get a doorhanger after basic-auth login"
   );
-  Services.logins.removeAllLogins();
+  Services.logins.removeAllUserFacingLogins();
   clearHttpAuths();
 
   await focusWindow(normalWin);
@@ -674,37 +787,49 @@ add_task(async function test_normal_http_basic_auth() {
       gBrowser: normalWin.gBrowser,
       url: "https://example.com",
     },
-    async function(browser) {
+    async function (browser) {
       await loadAccessRestrictedURL(browser, authUrl, "test", "testpass");
-      ok(true, "Auth-required page loaded");
+      Assert.ok(true, "Auth-required page loaded");
 
       // verify result in the response document
-      let fieldValues = await ContentTask.spawn(browser, [], async function() {
-        let username = content.document.getElementById("user").textContent;
-        let password = content.document.getElementById("pass").textContent;
-        let ok = content.document.getElementById("ok").textContent;
-        return {
-          username,
-          password,
-          ok,
-        };
-      });
-      is(fieldValues.ok, "PASS", "Checking authorization passed");
-      is(fieldValues.username, "test", "Checking authorized username");
-      is(fieldValues.password, "testpass", "Checking authorized password");
+      let fieldValues = await SpecialPowers.spawn(
+        browser,
+        [[]],
+        async function () {
+          let username = content.document.getElementById("user").textContent;
+          let password = content.document.getElementById("pass").textContent;
+          let ok = content.document.getElementById("ok").textContent;
+          return {
+            username,
+            password,
+            ok,
+          };
+        }
+      );
+      Assert.equal(fieldValues.ok, "PASS", "Checking authorization passed");
+      Assert.equal(
+        fieldValues.username,
+        "test",
+        "Checking authorized username"
+      );
+      Assert.equal(
+        fieldValues.password,
+        "testpass",
+        "Checking authorized password"
+      );
 
       let notif = getCaptureDoorhanger(
         "password-save",
         PopupNotifications,
         browser
       );
-      ok(notif, "got notification popup");
+      Assert.ok(notif, "got notification popup");
       if (notif) {
         await TestUtils.waitForCondition(
           () => !notif.dismissed,
           "notification should not be dismissed"
         );
-        notif.remove();
+        await cleanupDoorhanger(notif);
       }
     }
   );
@@ -714,13 +839,13 @@ add_task(async function test_private_http_basic_auth() {
   info(
     "test private/basic-auth: verify that we don't get a doorhanger after basic-auth login"
   );
-  Services.logins.removeAllLogins();
+  Services.logins.removeAllUserFacingLogins();
   clearHttpAuths();
 
   const capturePrefValue = Services.prefs.getBoolPref(
     PRIVATE_BROWSING_CAPTURE_PREF
   );
-  ok(
+  Assert.ok(
     capturePrefValue,
     `Expect ${PRIVATE_BROWSING_CAPTURE_PREF} to default to true`
   );
@@ -731,25 +856,36 @@ add_task(async function test_private_http_basic_auth() {
       gBrowser: privateWin.gBrowser,
       url: "https://example.com",
     },
-    async function(browser) {
+    async function (browser) {
       await loadAccessRestrictedURL(browser, authUrl, "test", "testpass");
 
-      let fieldValues = await getResponseResult(browser, "authenticate.sjs");
-      is(fieldValues.username, "test", "Checking authorized username");
-      is(fieldValues.password, "testpass", "Checking authorized password");
+      let fieldValues = await getFormSubmitResponseResult(
+        browser,
+        "authenticate.sjs"
+      );
+      Assert.equal(
+        fieldValues.username,
+        "test",
+        "Checking authorized username"
+      );
+      Assert.equal(
+        fieldValues.password,
+        "testpass",
+        "Checking authorized password"
+      );
 
       let notif = getCaptureDoorhanger(
         "password-save",
         PopupNotifications,
         browser
       );
-      ok(notif, "got notification popup");
+      Assert.ok(notif, "got notification popup");
       if (notif) {
         await TestUtils.waitForCondition(
           () => notif.dismissed,
           "notification should be dismissed"
         );
-        notif.remove();
+        await cleanupDoorhanger(notif);
       }
     }
   );
@@ -766,7 +902,7 @@ add_task(async function test_private_http_basic_auth_no_capture_pref() {
   );
   Services.prefs.setBoolPref(PRIVATE_BROWSING_CAPTURE_PREF, false);
 
-  Services.logins.removeAllLogins();
+  Services.logins.removeAllUserFacingLogins();
   clearHttpAuths();
 
   await focusWindow(privateWin);
@@ -775,12 +911,23 @@ add_task(async function test_private_http_basic_auth_no_capture_pref() {
       gBrowser: privateWin.gBrowser,
       url: "https://example.com",
     },
-    async function(browser) {
+    async function (browser) {
       await loadAccessRestrictedURL(browser, authUrl, "test", "testpass");
 
-      let fieldValues = await getResponseResult(browser, "authenticate.sjs");
-      is(fieldValues.username, "test", "Checking authorized username");
-      is(fieldValues.password, "testpass", "Checking authorized password");
+      let fieldValues = await getFormSubmitResponseResult(
+        browser,
+        "authenticate.sjs"
+      );
+      Assert.equal(
+        fieldValues.username,
+        "test",
+        "Checking authorized username"
+      );
+      Assert.equal(
+        fieldValues.password,
+        "testpass",
+        "Checking authorized password"
+      );
 
       let notif = getCaptureDoorhanger(
         "password-save",
@@ -793,9 +940,9 @@ add_task(async function test_private_http_basic_auth_no_capture_pref() {
         capturePrefValue
       );
 
-      ok(!notif, "got no notification popup");
+      Assert.ok(!notif, "got no notification popup");
       if (notif) {
-        notif.remove();
+        await cleanupDoorhanger(notif);
       }
     }
   );
