@@ -6,7 +6,7 @@
 const INITIAL_URL =
   "http://example.com/tests/toolkit/components/places/tests/browser/begin.html";
 const FINAL_URL =
-  "http://example.com/tests/toolkit/components/places/tests/browser/final.html";
+  "http://test1.example.com/tests/toolkit/components/places/tests/browser/final.html";
 
 /**
  * One-time observer callback.
@@ -20,7 +20,7 @@ function promiseObserve(name) {
   });
 }
 
-add_task(async function() {
+add_task(async function () {
   await SpecialPowers.pushPrefEnv({ set: [["places.history.enabled", false]] });
 
   let visitUriPromise = promiseObserve("uri-visit-saved");
@@ -32,7 +32,7 @@ add_task(async function() {
   let browserLoadedPromise = BrowserTestUtils.browserLoaded(
     gBrowser.selectedBrowser
   );
-  BrowserTestUtils.loadURI(gBrowser, FINAL_URL);
+  BrowserTestUtils.loadURIString(gBrowser, FINAL_URL);
   await browserLoadedPromise;
 
   let subject = await visitUriPromise;
