@@ -3,13 +3,13 @@
 
 // Tests that history initialization correctly handles a corrupt places schema.
 
-add_task(async function() {
-  let path = await setupPlacesDatabase("../migration/places_v38.sqlite");
+add_task(async function () {
+  let path = await setupPlacesDatabase(["migration", "places_v43.sqlite"]);
 
   // Ensure the database will go through a migration that depends on moz_places
   // and break the schema by dropping that table.
   let db = await Sqlite.openConnection({ path });
-  await db.setSchemaVersion(38);
+  await db.setSchemaVersion(43);
   await db.execute("DROP TABLE moz_places");
   await db.close();
 
