@@ -2,12 +2,11 @@
  * http://creativecommons.org/publicdomain/zero/1.0/ */
 
 add_task(async function test_duping_local_newer() {
-  let mergeTelemetryEvents = [];
+  let mergeTelemetryCounts;
   let buf = await openMirror("duping_local_newer", {
-    recordTelemetryEvent(object, method, value, extra) {
-      equal(object, "mirror", "Wrong object for telemetry event");
-      if (method == "merge") {
-        mergeTelemetryEvents.push({ value, extra });
+    recordStepTelemetry(name, took, counts) {
+      if (name == "merge") {
+        mergeTelemetryCounts = counts.filter(({ count }) => count > 0);
       }
     },
   });
@@ -22,6 +21,7 @@ add_task(async function test_duping_local_newer() {
         parentid: "places",
         type: "folder",
         children: ["bookmarkAAA5"],
+        dateAdded: localModified.getTime(),
       },
       {
         id: "bookmarkAAA5",
@@ -29,6 +29,7 @@ add_task(async function test_duping_local_newer() {
         type: "bookmark",
         bmkUri: "http://example.com/a",
         title: "A",
+        dateAdded: localModified.getTime(),
       },
     ],
     { needsMerge: false }
@@ -96,19 +97,16 @@ add_task(async function test_duping_local_newer() {
   let changesToUpload = await buf.apply({
     remoteTimeSeconds: localModified / 1000,
   });
-  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
   deepEqual(
-    mergeTelemetryEvents,
+    await buf.fetchUnmergedGuids(),
+    ["bookmarkAAA4", "bookmarkAAAA", PlacesUtils.bookmarks.menuGuid],
+    "Should leave A4, A, menu with new remote structure unmerged"
+  );
+  deepEqual(
+    mergeTelemetryCounts,
     [
-      {
-        value: "structure",
-        extra: {
-          remoteRevives: 0,
-          localDeletes: 0,
-          localRevives: 0,
-          remoteDeletes: 0,
-        },
-      },
+      { name: "items", count: 9 },
+      { name: "dupes", count: 2 },
     ],
     "Should record telemetry with dupe counts"
   );
@@ -188,6 +186,21 @@ add_task(async function test_duping_local_newer() {
           bmkUri: "http://example.com/a",
         },
       },
+      bookmarkAAA5: {
+        tombstone: false,
+        counter: 1,
+        synced: false,
+        cleartext: {
+          id: "bookmarkAAA5",
+          type: "bookmark",
+          parentid: "menu",
+          hasDupe: true,
+          parentName: menuInfo.title,
+          dateAdded: localModified.getTime(),
+          title: "A",
+          bmkUri: "http://example.com/a",
+        },
+      },
     },
     "Should uploaded newer deduped local items"
   );
@@ -445,7 +458,11 @@ add_task(async function test_duping_remote_newer() {
   let changesToUpload = await buf.apply({
     remoteTimeSeconds: localModified / 1000,
   });
-  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+  deepEqual(
+    await buf.fetchUnmergedGuids(),
+    [PlacesUtils.bookmarks.menuGuid],
+    "Should leave menu with new remote structure unmerged"
+  );
 
   let idsToUpload = inspectChangeRecords(changesToUpload);
   deepEqual(
@@ -587,20 +604,23 @@ add_task(async function test_duping_remote_newer() {
   );
 
   ok(
-    (await PlacesTestUtils.fetchBookmarkSyncFields(
-      "menu________",
-      "folderB11111",
-      "bookmarkC222",
-      "separatorF11",
-      "folderA11111",
-      "bookmarkG111",
-      "separatorE11",
-      "queryD111111"
-    )).every(
-      info => info.syncStatus == PlacesUtils.bookmarks.SYNC_STATUS.NORMAL
-    )
+    (
+      await PlacesTestUtils.fetchBookmarkSyncFields(
+        "menu________",
+        "folderB11111",
+        "bookmarkC222",
+        "separatorF11",
+        "folderA11111",
+        "bookmarkG111",
+        "separatorE11",
+        "queryD111111"
+      )
+    ).every(info => info.syncStatus == PlacesUtils.bookmarks.SYNC_STATUS.NORMAL)
   );
 
+  await storeChangesInMirror(buf, changesToUpload);
+  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+
   await buf.finalize();
   await PlacesUtils.bookmarks.eraseEverything();
   await PlacesSyncUtils.bookmarks.reset();
@@ -1040,7 +1060,11 @@ add_task(async function test_applying_two_empty_folders_matches_only_one() {
 
   info("Apply remote");
   let changesToUpload = await buf.apply();
-  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+  deepEqual(
+    await buf.fetchUnmergedGuids(),
+    [PlacesUtils.bookmarks.mobileGuid],
+    "Should leave mobile with new remote structure unmerged"
+  );
 
   let idsToUpload = inspectChangeRecords(changesToUpload);
   deepEqual(
@@ -1083,6 +1107,9 @@ add_task(async function test_applying_two_empty_folders_matches_only_one() {
     "Should apply 1 and dedupe L0 to 3"
   );
 
+  await storeChangesInMirror(buf, changesToUpload);
+  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+
   await buf.finalize();
   await PlacesUtils.bookmarks.eraseEverything();
   await PlacesSyncUtils.bookmarks.reset();
@@ -1132,7 +1159,11 @@ add_task(async function test_duping_mobile_bookmarks() {
 
   info("Apply remote");
   let changesToUpload = await buf.apply();
-  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+  deepEqual(
+    await buf.fetchUnmergedGuids(),
+    [PlacesUtils.bookmarks.mobileGuid],
+    "Should leave mobile with new remote structure unmerged"
+  );
 
   let idsToUpload = inspectChangeRecords(changesToUpload);
   deepEqual(
@@ -1164,6 +1195,9 @@ add_task(async function test_duping_mobile_bookmarks() {
     "Should dedupe A1 to A with different parent title"
   );
 
+  await storeChangesInMirror(buf, changesToUpload);
+  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+
   await buf.finalize();
   await PlacesUtils.bookmarks.eraseEverything();
   // Restore the original mobile root title.
@@ -1173,3 +1207,84 @@ add_task(async function test_duping_mobile_bookmarks() {
   });
   await PlacesSyncUtils.bookmarks.reset();
 });
+
+add_task(async function test_duping_invalid() {
+  // To check if invalid items are prevented from deduping
+
+  info("Set up empty mirror");
+  await PlacesTestUtils.markBookmarksAsSynced();
+
+  await PlacesUtils.bookmarks.insertTree({
+    guid: PlacesUtils.bookmarks.menuGuid,
+    children: [
+      {
+        guid: "bookmarkAAA1",
+        title: "A",
+        url: "http://example.com/a",
+      },
+    ],
+  });
+
+  let buf = await openMirror("duping_invalid");
+  await storeRecords(buf, [
+    {
+      id: "menu",
+      parentid: "places",
+      type: "folder",
+      children: ["bookmarkAAA2"],
+    },
+    {
+      id: "bookmarkAAA2",
+      parentid: "menu",
+      type: "bookmark",
+      title: "A",
+      bmkUri: "http://example.com/a",
+    },
+  ]);
+
+  // Invalidate bookmarkAAA2 so that it does not dedupe to bookmarkAAA1
+  await buf.db.execute(
+    `UPDATE items SET
+       validity = :validity
+     WHERE guid = :guid`,
+    {
+      validity: Ci.mozISyncedBookmarksMerger.VALIDITY_REPLACE,
+      guid: "bookmarkAAA2",
+    }
+  );
+
+  let changesToUpload = await buf.apply();
+  deepEqual(
+    changesToUpload.menu.cleartext.children,
+    ["bookmarkAAA1"],
+    "Should upload A1 in menu"
+  );
+  ok(
+    !changesToUpload.bookmarkAAA1.tombstone,
+    "Should not upload tombstone for A1"
+  );
+  ok(changesToUpload.bookmarkAAA2.tombstone, "Should upload tombstone for A2");
+  await assertLocalTree(
+    PlacesUtils.bookmarks.menuGuid,
+    {
+      guid: PlacesUtils.bookmarks.menuGuid,
+      type: PlacesUtils.bookmarks.TYPE_FOLDER,
+      index: 0,
+      title: BookmarksMenuTitle,
+      children: [
+        {
+          guid: "bookmarkAAA1",
+          type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+          index: 0,
+          title: "A",
+          url: "http://example.com/a",
+        },
+      ],
+    },
+    "No deduping of invalid items"
+  );
+
+  await buf.finalize();
+  await PlacesUtils.bookmarks.eraseEverything();
+  await PlacesSyncUtils.bookmarks.reset();
+});
