@@ -123,8 +123,13 @@ add_task(async function test_value_structure_conflict() {
   let observer = expectBookmarkChangeNotifications();
   let changesToUpload = await buf.apply({
     remoteTimeSeconds: Date.now() / 1000,
+    notifyInStableOrder: true,
   });
-  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+  deepEqual(
+    await buf.fetchUnmergedGuids(),
+    ["folderDDDDDD"],
+    "Should leave D with new remote structure unmerged"
+  );
 
   let idsToUpload = inspectChangeRecords(changesToUpload);
   deepEqual(
@@ -144,12 +149,10 @@ add_task(async function test_value_structure_conflict() {
   ]);
   observer.check([
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("bookmarkEEEE"),
-        oldParentId: localItemIds.get("folderDDDDDD"),
         oldIndex: 1,
-        newParentId: localItemIds.get("folderDDDDDD"),
         newIndex: 0,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         guid: "bookmarkEEEE",
@@ -157,15 +160,14 @@ add_task(async function test_value_structure_conflict() {
         newParentGuid: "folderDDDDDD",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
         urlHref: "http://example.com/e",
+        isTagging: false,
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("bookmarkBBBB"),
-        oldParentId: localItemIds.get("folderDDDDDD"),
         oldIndex: 0,
-        newParentId: localItemIds.get("folderDDDDDD"),
         newIndex: 1,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         guid: "bookmarkBBBB",
@@ -173,21 +175,16 @@ add_task(async function test_value_structure_conflict() {
         newParentGuid: "folderDDDDDD",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
         urlHref: "http://example.com/b",
+        isTagging: false,
       },
     },
     {
-      name: "onItemChanged",
+      name: "bookmark-title-changed",
       params: {
         itemId: localItemIds.get("folderDDDDDD"),
-        property: "title",
-        isAnnoProperty: false,
-        newValue: "D (remote)",
-        type: PlacesUtils.bookmarks.TYPE_FOLDER,
-        parentId: PlacesUtils.bookmarksMenuFolderId,
+        title: "D (remote)",
         guid: "folderDDDDDD",
         parentGuid: PlacesUtils.bookmarks.menuGuid,
-        oldValue: "D",
-        source: PlacesUtils.bookmarks.SOURCES.SYNC,
       },
     },
   ]);
@@ -242,6 +239,9 @@ add_task(async function test_value_structure_conflict() {
     "Should reconcile structure and value changes"
   );
 
+  await storeChangesInMirror(buf, changesToUpload);
+  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+
   await buf.finalize();
   await PlacesUtils.bookmarks.eraseEverything();
   await PlacesSyncUtils.bookmarks.reset();
@@ -368,7 +368,9 @@ add_task(async function test_move() {
 
   info("Apply remote");
   let observer = expectBookmarkChangeNotifications();
-  let changesToUpload = await buf.apply();
+  let changesToUpload = await buf.apply({
+    notifyInStableOrder: true,
+  });
   deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
 
   let idsToUpload = inspectChangeRecords(changesToUpload);
@@ -392,46 +394,40 @@ add_task(async function test_move() {
   ]);
   observer.check([
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("devFolder___"),
-        oldParentId: PlacesUtils.bookmarksMenuFolderId,
         oldIndex: 0,
-        newParentId: PlacesUtils.toolbarFolderId,
         newIndex: 0,
         type: PlacesUtils.bookmarks.TYPE_FOLDER,
         guid: "devFolder___",
         oldParentGuid: PlacesUtils.bookmarks.menuGuid,
         newParentGuid: PlacesUtils.bookmarks.toolbarGuid,
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
-        urlHref: null,
+        urlHref: "",
+        isTagging: false,
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("mozFolder___"),
-        oldParentId: localItemIds.get("devFolder___"),
         oldIndex: 1,
-        newParentId: await PlacesUtils.promiseItemId(
-          PlacesUtils.bookmarks.unfiledGuid
-        ),
         newIndex: 0,
         type: PlacesUtils.bookmarks.TYPE_FOLDER,
         guid: "mozFolder___",
         oldParentGuid: "devFolder___",
         newParentGuid: PlacesUtils.bookmarks.unfiledGuid,
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
-        urlHref: null,
+        urlHref: "",
+        isTagging: false,
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("bzBmk_______"),
-        oldParentId: PlacesUtils.bookmarksMenuFolderId,
         oldIndex: 1,
-        newParentId: localItemIds.get("devFolder___"),
         newIndex: 0,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         guid: "bzBmk_______",
@@ -439,15 +435,14 @@ add_task(async function test_move() {
         newParentGuid: "devFolder___",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
         urlHref: "https://bugzilla.mozilla.org/",
+        isTagging: false,
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("wmBmk_______"),
-        oldParentId: localItemIds.get("devFolder___"),
         oldIndex: 2,
-        newParentId: localItemIds.get("devFolder___"),
         newIndex: 1,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         guid: "wmBmk_______",
@@ -455,15 +450,14 @@ add_task(async function test_move() {
         newParentGuid: "devFolder___",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
         urlHref: "https://webmaker.org/",
+        isTagging: false,
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("nightlyBmk__"),
-        oldParentId: localItemIds.get("mozFolder___"),
         oldIndex: 1,
-        newParentId: localItemIds.get("mozFolder___"),
         newIndex: 0,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         guid: "nightlyBmk__",
@@ -471,15 +465,14 @@ add_task(async function test_move() {
         newParentGuid: "mozFolder___",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
         urlHref: "https://nightly.mozilla.org/",
+        isTagging: false,
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("mdnBmk______"),
-        oldParentId: localItemIds.get("devFolder___"),
         oldIndex: 0,
-        newParentId: localItemIds.get("mozFolder___"),
         newIndex: 1,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         guid: "mdnBmk______",
@@ -487,15 +480,14 @@ add_task(async function test_move() {
         newParentGuid: "mozFolder___",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
         urlHref: "https://developer.mozilla.org/",
+        isTagging: false,
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("fxBmk_______"),
-        oldParentId: localItemIds.get("mozFolder___"),
         oldIndex: 0,
-        newParentId: localItemIds.get("mozFolder___"),
         newIndex: 2,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         guid: "fxBmk_______",
@@ -503,6 +495,7 @@ add_task(async function test_move() {
         newParentGuid: "mozFolder___",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
         urlHref: "http://getfirefox.com/",
+        isTagging: false,
       },
     },
   ]);
@@ -688,7 +681,9 @@ add_task(async function test_move_into_parent_sibling() {
 
   info("Apply remote");
   let observer = expectBookmarkChangeNotifications();
-  let changesToUpload = await buf.apply();
+  let changesToUpload = await buf.apply({
+    notifyInStableOrder: true,
+  });
   deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
 
   let idsToUpload = inspectChangeRecords(changesToUpload);
@@ -705,13 +700,14 @@ add_task(async function test_move_into_parent_sibling() {
     "folderCCCCCC",
     "bookmarkBBBB",
     "folderAAAAAA",
+    PlacesUtils.bookmarks.menuGuid,
   ]);
   observer.check([
     {
       name: "bookmark-added",
       params: {
         itemId: localItemIds.get("folderCCCCCC"),
-        parentId: PlacesUtils.bookmarksMenuFolderId,
+        parentId: localItemIds.get(PlacesUtils.bookmarks.menuGuid),
         index: 1,
         type: PlacesUtils.bookmarks.TYPE_FOLDER,
         urlHref: "",
@@ -722,12 +718,10 @@ add_task(async function test_move_into_parent_sibling() {
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("bookmarkBBBB"),
-        oldParentId: localItemIds.get("folderAAAAAA"),
         oldIndex: 0,
-        newParentId: localItemIds.get("folderCCCCCC"),
         newIndex: 0,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         guid: "bookmarkBBBB",
@@ -735,6 +729,7 @@ add_task(async function test_move_into_parent_sibling() {
         newParentGuid: "folderCCCCCC",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
         urlHref: "http://example.com/b",
+        isTagging: false,
       },
     },
   ]);
@@ -779,12 +774,11 @@ add_task(async function test_move_into_parent_sibling() {
 });
 
 add_task(async function test_complex_move_with_additions() {
-  let mergeTelemetryEvents = [];
+  let mergeTelemetryCounts;
   let buf = await openMirror("complex_move_with_additions", {
-    recordTelemetryEvent(object, method, value, extra) {
-      equal(object, "mirror", "Wrong object for telemetry event");
-      if (method == "merge") {
-        mergeTelemetryEvents.push({ value, extra });
+    recordStepTelemetry(name, took, counts) {
+      if (name == "merge") {
+        mergeTelemetryCounts = counts;
       }
     },
   });
@@ -897,21 +891,17 @@ add_task(async function test_complex_move_with_additions() {
 
   info("Apply remote");
   let observer = expectBookmarkChangeNotifications();
-  let changesToUpload = await buf.apply();
-  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+  let changesToUpload = await buf.apply({
+    notifyInStableOrder: true,
+  });
   deepEqual(
-    mergeTelemetryEvents,
-    [
-      {
-        value: "structure",
-        extra: {
-          remoteRevives: 0,
-          localDeletes: 0,
-          localRevives: 0,
-          remoteDeletes: 0,
-        },
-      },
-    ],
+    await buf.fetchUnmergedGuids(),
+    ["folderAAAAAA"],
+    "Should leave A with new remote structure unmerged"
+  );
+  deepEqual(
+    mergeTelemetryCounts,
+    [{ name: "items", count: 10 }],
     "Should record telemetry with structure change counts"
   );
 
@@ -946,12 +936,10 @@ add_task(async function test_complex_move_with_additions() {
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("bookmarkCCCC"),
-        oldParentId: localItemIds.get("folderAAAAAA"),
         oldIndex: 1,
-        newParentId: PlacesUtils.bookmarksMenuFolderId,
         newIndex: 0,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         guid: "bookmarkCCCC",
@@ -959,22 +947,22 @@ add_task(async function test_complex_move_with_additions() {
         newParentGuid: PlacesUtils.bookmarks.menuGuid,
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
         urlHref: "http://example.com/c",
+        isTagging: false,
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("folderAAAAAA"),
-        oldParentId: PlacesUtils.bookmarksMenuFolderId,
         oldIndex: 0,
-        newParentId: PlacesUtils.toolbarFolderId,
         newIndex: 0,
         type: PlacesUtils.bookmarks.TYPE_FOLDER,
         guid: "folderAAAAAA",
         oldParentGuid: PlacesUtils.bookmarks.menuGuid,
         newParentGuid: PlacesUtils.bookmarks.toolbarGuid,
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
-        urlHref: null,
+        urlHref: "",
+        isTagging: false,
       },
     },
   ]);
@@ -1060,6 +1048,9 @@ add_task(async function test_complex_move_with_additions() {
     "Should take remote order and preserve local children"
   );
 
+  await storeChangesInMirror(buf, changesToUpload);
+  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+
   await buf.finalize();
   await PlacesUtils.bookmarks.eraseEverything();
   await PlacesSyncUtils.bookmarks.reset();
@@ -1249,7 +1240,11 @@ add_task(async function test_reorder_and_insert() {
     remoteTimeSeconds: now / 1000,
     localTimeSeconds: now / 1000,
   });
-  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+  deepEqual(
+    await buf.fetchUnmergedGuids(),
+    [PlacesUtils.bookmarks.menuGuid, PlacesUtils.bookmarks.toolbarGuid],
+    "Should leave roots with new remote structure unmerged"
+  );
 
   let idsToUpload = inspectChangeRecords(changesToUpload);
   deepEqual(
@@ -1372,6 +1367,9 @@ add_task(async function test_reorder_and_insert() {
     "Should use timestamps to decide base folder order"
   );
 
+  await storeChangesInMirror(buf, changesToUpload);
+  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+
   await buf.finalize();
   await PlacesUtils.bookmarks.eraseEverything();
   await PlacesSyncUtils.bookmarks.reset();
@@ -1707,7 +1705,16 @@ add_task(async function test_newer_remote_moves() {
     localTimeSeconds: now / 1000,
     remoteTimeSeconds: now / 1000,
   });
-  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+  deepEqual(
+    await buf.fetchUnmergedGuids(),
+    [
+      PlacesUtils.bookmarks.menuGuid,
+      PlacesUtils.bookmarks.mobileGuid,
+      PlacesUtils.bookmarks.toolbarGuid,
+      PlacesUtils.bookmarks.unfiledGuid,
+    ],
+    "Should leave roots with new remote structure unmerged"
+  );
   let datesAdded = await promiseManyDatesAdded([
     PlacesUtils.bookmarks.menuGuid,
     PlacesUtils.bookmarks.mobileGuid,
@@ -1723,7 +1730,7 @@ add_task(async function test_newer_remote_moves() {
       // in Places (see the comment for F below), we'll reupload them.
       menu: {
         tombstone: false,
-        counter: 1,
+        counter: 2,
         synced: false,
         cleartext: {
           id: "menu",
@@ -1886,6 +1893,9 @@ add_task(async function test_newer_remote_moves() {
     "Should use newer remote parents and order"
   );
 
+  await storeChangesInMirror(buf, changesToUpload);
+  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+
   await buf.finalize();
   await PlacesUtils.bookmarks.eraseEverything();
   await PlacesSyncUtils.bookmarks.reset();
@@ -2204,7 +2214,23 @@ add_task(async function test_newer_local_moves() {
     localTimeSeconds: now / 1000,
     remoteTimeSeconds: now / 1000,
   });
-  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+  deepEqual(
+    await buf.fetchUnmergedGuids(),
+    [
+      "bookmarkAAAA",
+      "bookmarkCCCC",
+      "bookmarkGGGG",
+      "folderBBBBBB",
+      "folderDDDDDD",
+      "folderFFFFFF",
+      "folderHHHHHH",
+      PlacesUtils.bookmarks.menuGuid,
+      PlacesUtils.bookmarks.mobileGuid,
+      PlacesUtils.bookmarks.toolbarGuid,
+      PlacesUtils.bookmarks.unfiledGuid,
+    ],
+    "Should leave items with new remote structure unmerged"
+  );
   let datesAdded = await promiseManyDatesAdded([
     PlacesUtils.bookmarks.menuGuid,
     PlacesUtils.bookmarks.mobileGuid,
@@ -2217,7 +2243,7 @@ add_task(async function test_newer_local_moves() {
       // Reupload roots with new children.
       menu: {
         tombstone: false,
-        counter: 1,
+        counter: 2,
         synced: false,
         cleartext: {
           id: "menu",
@@ -2373,7 +2399,7 @@ add_task(async function test_newer_local_moves() {
       },
       folderBBBBBB: {
         tombstone: false,
-        counter: 1,
+        counter: 2,
         synced: false,
         cleartext: {
           id: "folderBBBBBB",
@@ -2491,6 +2517,9 @@ add_task(async function test_newer_local_moves() {
     "Should use newer local parents and order"
   );
 
+  await storeChangesInMirror(buf, changesToUpload);
+  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+
   await buf.finalize();
   await PlacesUtils.bookmarks.eraseEverything();
   await PlacesSyncUtils.bookmarks.reset();
@@ -2673,7 +2702,11 @@ add_task(async function test_unchanged_newer_changed_older() {
     localTimeSeconds: modified.getTime() / 1000 + 10,
     remoteTimeSeconds: modified.getTime() / 1000 + 10,
   });
-  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+  deepEqual(
+    await buf.fetchUnmergedGuids(),
+    ["bookmarkFFFF", "folderCCCCCC", PlacesUtils.bookmarks.menuGuid],
+    "Should leave deleted C; F and menu with new remote structure unmerged"
+  );
 
   deepEqual(
     changesToUpload,
@@ -2821,6 +2854,16 @@ add_task(async function test_unchanged_newer_changed_older() {
     "Should merge children of changed side first, even if they're older"
   );
 
+  let tombstones = await PlacesTestUtils.fetchSyncTombstones();
+  deepEqual(
+    tombstones.map(({ guid }) => guid),
+    ["folderCCCCCC"],
+    "Should store local tombstone for C"
+  );
+
+  await storeChangesInMirror(buf, changesToUpload);
+  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+
   await buf.finalize();
   await PlacesUtils.bookmarks.eraseEverything();
   await PlacesSyncUtils.bookmarks.reset();
