@@ -10,19 +10,17 @@ const TEST_PREFS = [["reader.parse-on-load.enabled", true]];
 
 const TEST_PATH = getRootDirectory(gTestPath).replace(
   "chrome://mochitests/content",
-  "http://example.com"
+  "https://example.com"
 );
 
 var readerButton = document.getElementById("reader-mode-button");
 
-ChromeUtils.defineModuleGetter(
-  this,
-  "PlacesTestUtils",
-  "resource://testing-common/PlacesTestUtils.jsm"
-);
+ChromeUtils.defineESModuleGetters(this, {
+  PlacesTestUtils: "resource://testing-common/PlacesTestUtils.sys.mjs",
+});
 
 add_task(async function test_reader_button() {
-  registerCleanupFunction(function() {
+  registerCleanupFunction(function () {
     // Reset test prefs.
     TEST_PREFS.forEach(([name, value]) => {
       Services.prefs.clearUserPref(name);
@@ -81,29 +79,21 @@ add_task(async function test_reader_button() {
     readerButton,
     "Reader mode button is present on about:reader"
   );
-  let iconEl = document.getAnonymousElementByAttribute(
-    tab,
-    "anonid",
-    "tab-icon-image"
-  );
+  let iconEl = tab.iconImage;
   await TestUtils.waitForCondition(
     () => iconEl.getBoundingClientRect().width != 0
   );
   is_element_visible(iconEl, "Favicon should be visible");
   is(iconEl.src, favicon, "Correct favicon should be loaded");
 
-  is(gURLBar.value, readerUrl, "gURLBar value is about:reader URL");
-  is(
-    gURLBar.textValue,
-    url.substring("http://".length),
-    "gURLBar is displaying original article URL"
-  );
+  is(gURLBar.untrimmedValue, url, "gURLBar value is about:reader URL");
+  is(gURLBar.value, url, "gURLBar is displaying original article URL");
 
   // Check selected value for URL bar
   await new Promise((resolve, reject) => {
     waitForClipboard(
       url,
-      function() {
+      function () {
         gURLBar.focus();
         gURLBar.select();
         goDoCommand("cmd_copy");
@@ -118,7 +108,9 @@ add_task(async function test_reader_button() {
   // Switch page back out of reader mode.
   let promisePageShow = BrowserTestUtils.waitForContentEvent(
     tab.linkedBrowser,
-    "pageshow"
+    "pageshow",
+    false,
+    e => e.target.location.href != "about:blank"
   );
   readerButton.click();
   await promisePageShow;
@@ -168,7 +160,9 @@ add_task(async function test_reader_button() {
   // Switch page back out of reader mode.
   promisePageShow = BrowserTestUtils.waitForContentEvent(
     newTab.linkedBrowser,
-    "pageshow"
+    "pageshow",
+    false,
+    e => e.target.location.href != "about:blank"
   );
   readerButton.click();
   await promisePageShow;
@@ -185,10 +179,10 @@ add_task(async function test_reader_button() {
 });
 
 add_task(async function test_getOriginalUrl() {
-  let { ReaderMode } = ChromeUtils.import(
-    "resource://gre/modules/ReaderMode.jsm"
+  let { ReaderMode } = ChromeUtils.importESModule(
+    "resource://gre/modules/ReaderMode.sys.mjs"
   );
-  let url = "http://foo.com/article.html";
+  let url = "https://foo.com/article.html";
 
   is(
     ReaderMode.getOriginalUrl("about:reader?url=" + encodeURIComponent(url)),
@@ -206,7 +200,7 @@ add_task(async function test_getOriginalUrl() {
     "Did not find original URL from non-reader URL"
   );
 
-  let badUrl = "http://foo.com/?;$%^^";
+  let badUrl = "https://foo.com/?;$%^^";
   is(
     ReaderMode.getOriginalUrl("about:reader?url=" + encodeURIComponent(badUrl)),
     badUrl,
@@ -220,28 +214,30 @@ add_task(async function test_getOriginalUrl() {
 });
 
 add_task(async function test_reader_view_element_attribute_transform() {
-  registerCleanupFunction(function() {
+  registerCleanupFunction(function () {
     while (gBrowser.tabs.length > 1) {
       gBrowser.removeCurrentTab();
     }
   });
 
-  function observeAttribute(element, attribute, triggerFn, checkFn) {
+  function observeAttribute(element, attributes, triggerFn, checkFn) {
     return new Promise(resolve => {
       let observer = new MutationObserver(mutations => {
-        mutations.forEach(mu => {
-          if (element.getAttribute(attribute) !== mu.oldValue) {
-            checkFn();
-            resolve();
-            observer.disconnect();
+        for (let mu of mutations) {
+          if (element.getAttribute(mu.attributeName) !== mu.oldValue) {
+            if (checkFn()) {
+              resolve();
+              observer.disconnect();
+              return;
+            }
           }
-        });
+        }
       });
 
       observer.observe(element, {
         attributes: true,
         attributeOldValue: true,
-        attributeFilter: [attribute],
+        attributeFilter: Array.isArray(attributes) ? attributes : [attributes],
       });
 
       triggerFn();
@@ -257,132 +253,142 @@ add_task(async function test_reader_view_element_attribute_transform() {
   );
 
   info("Navigate a reader-able page");
-  let waitForPageshow = BrowserTestUtils.waitForContentEvent(
-    tab.linkedBrowser,
-    "pageshow"
-  );
+  function waitForNonBlankPage() {
+    return BrowserTestUtils.waitForContentEvent(
+      tab.linkedBrowser,
+      "pageshow",
+      false,
+      e => e.target.location.href != "about:blank"
+    );
+  }
+  let waitForPageshow = waitForNonBlankPage();
   await observeAttribute(
     menuitem,
     "hidden",
     () => {
       let url = TEST_PATH + "readerModeArticle.html";
-      BrowserTestUtils.loadURI(tab.linkedBrowser, url);
+      BrowserTestUtils.loadURIString(tab.linkedBrowser, url);
     },
-    () => {
-      is(
-        menuitem.hidden,
-        false,
-        "menuitem's hidden attribute should be false on a reader-able page"
-      );
-    }
+    () => !menuitem.hidden
+  );
+  is(
+    menuitem.hidden,
+    false,
+    "menuitem's hidden attribute should be false on a reader-able page"
   );
   await waitForPageshow;
 
   info("Navigate a non-reader-able page");
-  waitForPageshow = BrowserTestUtils.waitForContentEvent(
-    tab.linkedBrowser,
-    "pageshow"
-  );
+  waitForPageshow = waitForNonBlankPage();
   await observeAttribute(
     menuitem,
     "hidden",
     () => {
       let url = TEST_PATH + "readerModeArticleHiddenNodes.html";
-      BrowserTestUtils.loadURI(tab.linkedBrowser, url);
+      BrowserTestUtils.loadURIString(tab.linkedBrowser, url);
     },
-    () => {
-      is(
-        menuitem.hidden,
-        true,
-        "menuitem's hidden attribute should be true on a non-reader-able page"
-      );
-    }
+    () => menuitem.hidden
+  );
+  is(
+    menuitem.hidden,
+    true,
+    "menuitem's hidden attribute should be true on a non-reader-able page"
   );
   await waitForPageshow;
 
   info("Navigate a reader-able page");
-  waitForPageshow = BrowserTestUtils.waitForContentEvent(
-    tab.linkedBrowser,
-    "pageshow"
-  );
+  waitForPageshow = waitForNonBlankPage();
   await observeAttribute(
     menuitem,
     "hidden",
     () => {
       let url = TEST_PATH + "readerModeArticle.html";
-      BrowserTestUtils.loadURI(tab.linkedBrowser, url);
+      BrowserTestUtils.loadURIString(tab.linkedBrowser, url);
     },
-    () => {
-      is(
-        menuitem.hidden,
-        false,
-        "menuitem's hidden attribute should be false on a reader-able page"
-      );
-    }
+    () => !menuitem.hidden
+  );
+  is(
+    menuitem.hidden,
+    false,
+    "menuitem's hidden attribute should be false on a reader-able page"
   );
   await waitForPageshow;
 
   info("Enter Reader Mode");
-  waitForPageshow = BrowserTestUtils.waitForContentEvent(
-    tab.linkedBrowser,
-    "pageshow"
-  );
+  waitForPageshow = waitForNonBlankPage();
   await observeAttribute(
     readerButton,
     "readeractive",
     () => {
       readerButton.click();
     },
-    () => {
-      is(
-        readerButton.getAttribute("readeractive"),
-        "true",
-        "readerButton's readeractive attribute should be true when entering reader mode"
-      );
-    }
+    () => readerButton.getAttribute("readeractive") == "true"
+  );
+  is(
+    readerButton.getAttribute("readeractive"),
+    "true",
+    "readerButton's readeractive attribute should be true when entering reader mode"
   );
   await waitForPageshow;
 
   info("Exit Reader Mode");
-  waitForPageshow = BrowserTestUtils.waitForContentEvent(
-    tab.linkedBrowser,
-    "pageshow"
-  );
+  waitForPageshow = waitForNonBlankPage();
   await observeAttribute(
     readerButton,
-    "readeractive",
+    ["readeractive", "hidden"],
     () => {
       readerButton.click();
     },
     () => {
-      is(
-        readerButton.getAttribute("readeractive"),
-        "",
-        "readerButton's readeractive attribute should be empty when reader mode is exited"
+      info(
+        `active: ${readerButton.getAttribute("readeractive")}; hidden: ${
+          menuitem.hidden
+        }`
       );
+      return !readerButton.getAttribute("readeractive") && !menuitem.hidden;
     }
   );
+  ok(
+    !readerButton.getAttribute("readeractive"),
+    "readerButton's readeractive attribute should be empty when reader mode is exited"
+  );
+  ok(!menuitem.hidden, "menuitem should not be hidden.");
   await waitForPageshow;
 
   info("Navigate a non-reader-able page");
-  waitForPageshow = BrowserTestUtils.waitForContentEvent(
-    tab.linkedBrowser,
-    "pageshow"
-  );
+  waitForPageshow = waitForNonBlankPage();
   await observeAttribute(
     menuitem,
     "hidden",
     () => {
       let url = TEST_PATH + "readerModeArticleHiddenNodes.html";
-      BrowserTestUtils.loadURI(tab.linkedBrowser, url);
+      BrowserTestUtils.loadURIString(tab.linkedBrowser, url);
     },
-    () => {
-      is(
-        menuitem.hidden,
-        true,
-        "menuitem's hidden attribute should be true on a non-reader-able page"
-      );
-    }
+    () => menuitem.hidden
+  );
+  is(
+    menuitem.hidden,
+    true,
+    "menuitem's hidden attribute should be true on a non-reader-able page"
   );
   await waitForPageshow;
 });
+
+add_task(async function test_reader_mode_lang() {
+  let url = TEST_PATH + "readerModeArticle.html";
+  let tab = await BrowserTestUtils.openNewForegroundTab(gBrowser);
+  BrowserTestUtils.loadURIString(tab.linkedBrowser, url);
+
+  await promiseTabLoadEvent(tab, url);
+  await TestUtils.waitForCondition(() => !readerButton.hidden);
+
+  // Switch page into reader mode.
+  let promiseTabLoad = promiseTabLoadEvent(tab);
+  readerButton.click();
+  await promiseTabLoad;
+
+  await SpecialPowers.spawn(gBrowser.selectedBrowser, [], () => {
+    let container = content.document.querySelector(".container");
+    is(container.lang, "en");
+  });
+});
