@@ -7,7 +7,7 @@
 
 #ifdef _MSC_VER
 // Disable exception handler warnings.
-# pragma warning( disable : 4530 )
+#  pragma warning(disable : 4530)
 #endif
 
 #include <fstream>
@@ -18,46 +18,49 @@
 #include <cstdlib>
 #include <cstring>
 #include <string>
+#include <utility>
 
 #ifdef XP_LINUX
-#include <dlfcn.h>
+#  include <dlfcn.h>
 #endif
 
+#include "json/json.h"
 #include "nss.h"
 #include "sechash.h"
 
-using std::string;
-using std::istream;
 using std::ifstream;
+using std::ios;
+using std::istream;
 using std::istringstream;
-using std::ostringstream;
-using std::ostream;
 using std::ofstream;
+using std::ostream;
+using std::ostringstream;
+using std::string;
+using std::unique_ptr;
 using std::vector;
-using std::auto_ptr;
 
 namespace CrashReporter {
 
-StringTable  gStrings;
-string       gSettingsPath;
-string       gEventsPath;
-string       gPingPath;
-int          gArgc;
-char**       gArgv;
-bool         gAutoSubmit;
+StringTable gStrings;
+Json::Value gData;
+string gSettingsPath;
+string gEventsPath;
+string gPingPath;
+int gArgc;
+char** gArgv;
+bool gAutoSubmit;
 
-enum SubmissionResult {Succeeded, Failed};
+enum SubmissionResult { Succeeded, Failed };
 
-static auto_ptr<ofstream> gLogStream(nullptr);
-static string             gReporterDumpFile;
-static string             gExtraFile;
-static string             gMemoryFile;
+static unique_ptr<ofstream> gLogStream(nullptr);
+static string gReporterDumpFile;
+static string gExtraFile;
+static string gMemoryFile;
 
 static const char kExtraDataExtension[] = ".extra";
 static const char kMemoryReportExtension[] = ".memory.json.gz";
 
-void UIError(const string& message)
-{
+void UIError(const string& message) {
   if (gAutoSubmit) {
     return;
   }
@@ -65,9 +68,8 @@ void UIError(const string& message)
   string errorMessage;
   if (!gStrings[ST_CRASHREPORTERERROR].empty()) {
     char buf[2048];
-    UI_SNPRINTF(buf, 2048,
-                gStrings[ST_CRASHREPORTERERROR].c_str(),
-                message.c_str());
+    snprintf(buf, 2048, gStrings[ST_CRASHREPORTERERROR].c_str(),
+             message.c_str());
     errorMessage = buf;
   } else {
     errorMessage = message;
@@ -76,15 +78,12 @@ void UIError(const string& message)
   UIError_impl(errorMessage);
 }
 
-static string Unescape(const string& str)
-{
+static string Unescape(const string& str) {
   string ret;
-  for (string::const_iterator iter = str.begin();
-       iter != str.end();
-       iter++) {
+  for (string::const_iterator iter = str.begin(); iter != str.end(); iter++) {
     if (*iter == '\\') {
       iter++;
-      if (*iter == '\\'){
+      if (*iter == '\\') {
         ret.push_back('\\');
       } else if (*iter == 'n') {
         ret.push_back('\n');
@@ -99,28 +98,7 @@ static string Unescape(const string& str)
   return ret;
 }
 
-static string Escape(const string& str)
-{
-  string ret;
-  for (string::const_iterator iter = str.begin();
-       iter != str.end();
-       iter++) {
-    if (*iter == '\\') {
-      ret += "\\\\";
-    } else if (*iter == '\n') {
-      ret += "\\n";
-    } else if (*iter == '\t') {
-      ret += "\\t";
-    } else {
-      ret.push_back(*iter);
-    }
-  }
-
-  return ret;
-}
-
-bool ReadStrings(istream& in, StringTable& strings, bool unescape)
-{
+bool ReadStrings(istream& in, StringTable& strings, bool unescape) {
   string currentSection;
   while (!in.eof()) {
     string line;
@@ -130,8 +108,7 @@ bool ReadStrings(istream& in, StringTable& strings, bool unescape)
       string key, value;
       key = line.substr(0, sep);
       value = line.substr(sep + 1);
-      if (unescape)
-        value = Unescape(value);
+      if (unescape) value = Unescape(value);
       strings[key] = value;
     }
   }
@@ -139,11 +116,9 @@ bool ReadStrings(istream& in, StringTable& strings, bool unescape)
   return true;
 }
 
-bool ReadStringsFromFile(const string& path,
-                         StringTable& strings,
-                         bool unescape)
-{
-  ifstream* f = UIOpenRead(path);
+bool ReadStringsFromFile(const string& path, StringTable& strings,
+                         bool unescape) {
+  ifstream* f = UIOpenRead(path, ios::in);
   bool success = false;
   if (f->is_open()) {
     success = ReadStrings(*f, strings, unescape);
@@ -154,95 +129,111 @@ bool ReadStringsFromFile(const string& path,
   return success;
 }
 
-bool WriteStrings(ostream& out,
-                  const string& header,
-                  StringTable& strings,
-                  bool escape)
-{
-  out << "[" << header << "]" << std::endl;
-  for (StringTable::iterator iter = strings.begin();
-       iter != strings.end();
-       iter++) {
-    out << iter->first << "=";
-    if (escape)
-      out << Escape(iter->second);
-    else
-      out << iter->second;
-
-    out << std::endl;
-  }
-
-  return true;
-}
-
-bool WriteStringsToFile(const string& path,
-                        const string& header,
-                        StringTable& strings,
-                        bool escape)
-{
-  ofstream* f = UIOpenWrite(path.c_str());
+static bool ReadExtraFile(const string& aExtraDataPath, Json::Value& aExtra) {
   bool success = false;
+  ifstream* f = UIOpenRead(aExtraDataPath, ios::in);
   if (f->is_open()) {
-    success = WriteStrings(*f, header, strings, escape);
-    f->close();
+    Json::CharReaderBuilder builder;
+    success = parseFromStream(builder, *f, &aExtra, nullptr);
   }
 
   delete f;
   return success;
 }
 
-static string Basename(const string& file)
-{
+static string Basename(const string& file) {
   string::size_type slashIndex = file.rfind(UI_DIR_SEPARATOR);
-  if (slashIndex != string::npos)
+  if (slashIndex != string::npos) {
     return file.substr(slashIndex + 1);
-  else
-    return file;
-}
-
-static string GetDumpLocalID()
-{
-  string localId = Basename(gReporterDumpFile);
-  string::size_type dot = localId.rfind('.');
-
-  if (dot == string::npos)
-    return "";
-
-  return localId.substr(0, dot);
-}
-
-// This appends the aKey/aValue entry to the main crash event so that it can
-// be picked up by Firefox once it restarts
-static void AppendToEventFile(const string& aKey, const string& aValue)
-{
-  if (gEventsPath.empty()) {
-    // If there is no path for finding the crash event, skip this step.
-    return;
   }
+  return file;
+}
 
-  string localId = GetDumpLocalID();
-  string path = gEventsPath + UI_DIR_SEPARATOR + localId;
-  ofstream* f = UIOpenWrite(path.c_str(), true);
+static bool ReadEventFile(const string& aPath, string& aEventVersion,
+                          string& aTime, string& aUuid, Json::Value& aData) {
+  bool res = false;
+  ifstream* f = UIOpenRead(aPath, ios::binary);
 
   if (f->is_open()) {
-    *f << aKey << "=" << aValue << std::endl;
-    f->close();
+    std::getline(*f, aEventVersion, '\n');
+    res = f->good();
+    std::getline(*f, aTime, '\n');
+    res &= f->good();
+    std::getline(*f, aUuid, '\n');
+    res &= f->good();
+
+    if (res) {
+      Json::CharReaderBuilder builder;
+      res = parseFromStream(builder, *f, &aData, nullptr);
+    }
+  }
+
+  delete f;
+  return res;
+}
+
+static void OverwriteEventFile(const string& aPath, const string& aEventVersion,
+                               const string& aTime, const string& aUuid,
+                               const Json::Value& aData) {
+  ofstream* f = UIOpenWrite(aPath, ios::binary | ios::trunc);
+  if (f->is_open()) {
+    f->write(aEventVersion.c_str(), aEventVersion.length()) << '\n';
+    f->write(aTime.c_str(), aTime.length()) << '\n';
+    f->write(aUuid.c_str(), aUuid.length()) << '\n';
+
+    Json::StreamWriterBuilder builder;
+    builder["indentation"] = "";
+    std::unique_ptr<Json::StreamWriter> writer(builder.newStreamWriter());
+    writer->write(aData, f);
+    *f << "\n";
   }
 
   delete f;
 }
 
+static void UpdateEventFile(const Json::Value& aExtraData, const string& aHash,
+                            const string& aPingUuid) {
+  if (gEventsPath.empty()) {
+    // If there is no path for finding the crash event, skip this step.
+    return;
+  }
+
+  string localId = CrashReporter::GetDumpLocalID();
+  string path = gEventsPath + UI_DIR_SEPARATOR + localId;
+  string eventVersion;
+  string crashTime;
+  string crashUuid;
+  Json::Value eventData;
+
+  if (!ReadEventFile(path, eventVersion, crashTime, crashUuid, eventData)) {
+    return;
+  }
+
+  if (!aHash.empty()) {
+    eventData["MinidumpSha256Hash"] = aHash;
+  }
+
+  if (!aPingUuid.empty()) {
+    eventData["CrashPingUUID"] = aPingUuid;
+  }
+
+  if (aExtraData.isMember("StackTraces")) {
+    eventData["StackTraces"] = aExtraData["StackTraces"];
+  }
+
+  OverwriteEventFile(path, eventVersion, crashTime, crashUuid, eventData);
+}
+
 static void WriteSubmissionEvent(SubmissionResult result,
-                                 const string& remoteId)
-{
+                                 const string& remoteId) {
   if (gEventsPath.empty()) {
     // If there is no path for writing the submission event, skip it.
     return;
   }
 
-  string localId = GetDumpLocalID();
+  string localId = CrashReporter::GetDumpLocalID();
   string fpath = gEventsPath + UI_DIR_SEPARATOR + localId + "-submission";
-  ofstream* f = UIOpenWrite(fpath.c_str(), false, true);
+  ofstream* f = UIOpenWrite(fpath, ios::binary);
   time_t tm;
   time(&tm);
 
@@ -259,33 +250,29 @@ static void WriteSubmissionEvent(SubmissionResult result,
   delete f;
 }
 
-void LogMessage(const std::string& message)
-{
+void LogMessage(const std::string& message) {
   if (gLogStream.get()) {
     char date[64];
     time_t tm;
     time(&tm);
     if (strftime(date, sizeof(date) - 1, "%c", localtime(&tm)) == 0)
-        date[0] = '\0';
+      date[0] = '\0';
     (*gLogStream) << "[" << date << "] " << message << std::endl;
   }
 }
 
-static void OpenLogFile()
-{
+static void OpenLogFile() {
   string logPath = gSettingsPath + UI_DIR_SEPARATOR + "submit.log";
-  gLogStream.reset(UIOpenWrite(logPath.c_str(), true));
+  gLogStream.reset(UIOpenWrite(logPath, ios::app));
 }
 
-static bool ReadConfig()
-{
+static bool ReadConfig() {
   string iniPath;
   if (!UIGetIniPath(iniPath)) {
     return false;
   }
 
-  if (!ReadStringsFromFile(iniPath, gStrings, true))
-    return false;
+  if (!ReadStringsFromFile(iniPath, gStrings, true)) return false;
 
   // See if we have a string override file, if so process it
   char* overrideEnv = getenv("MOZ_CRASHREPORTER_STRINGS_OVERRIDE");
@@ -295,23 +282,18 @@ static bool ReadConfig()
   return true;
 }
 
-static string
-GetAdditionalFilename(const string& dumpfile, const char* extension)
-{
+static string GetAdditionalFilename(const string& dumpfile,
+                                    const char* extension) {
   string filename(dumpfile);
   int dot = filename.rfind('.');
-  if (dot < 0)
-    return "";
+  if (dot < 0) return "";
 
   filename.replace(dot, filename.length() - dot, extension);
   return filename;
 }
 
-static bool MoveCrashData(const string& toDir,
-                          string& dumpfile,
-                          string& extrafile,
-                          string& memoryfile)
-{
+static bool MoveCrashData(const string& toDir, string& dumpfile,
+                          string& extrafile, string& memoryfile) {
   if (!UIEnsurePathExists(toDir)) {
     UIError(gStrings[ST_ERROR_CREATEDUMPDIR]);
     return false;
@@ -346,19 +328,17 @@ static bool MoveCrashData(const string& toDir,
   return true;
 }
 
-static bool AddSubmittedReport(const string& serverResponse)
-{
+static bool AddSubmittedReport(const string& serverResponse) {
   StringTable responseItems;
   istringstream in(serverResponse);
   ReadStrings(in, responseItems, false);
 
   if (responseItems.find("StopSendingReportsFor") != responseItems.end()) {
     // server wants to tell us to stop sending reports for a certain version
-    string reportPath =
-      gSettingsPath + UI_DIR_SEPARATOR + "EndOfLife" +
-      responseItems["StopSendingReportsFor"];
+    string reportPath = gSettingsPath + UI_DIR_SEPARATOR + "EndOfLife" +
+                        responseItems["StopSendingReportsFor"];
 
-    ofstream* reportFile = UIOpenWrite(reportPath);
+    ofstream* reportFile = UIOpenWrite(reportPath, ios::trunc);
     if (reportFile->is_open()) {
       // don't really care about the contents
       *reportFile << 1 << "\n";
@@ -373,34 +353,30 @@ static bool AddSubmittedReport(const string& serverResponse)
     return false;
   }
 
-  if (responseItems.find("CrashID") == responseItems.end())
-    return false;
+  if (responseItems.find("CrashID") == responseItems.end()) return false;
 
-  string submittedDir =
-    gSettingsPath + UI_DIR_SEPARATOR + "submitted";
+  string submittedDir = gSettingsPath + UI_DIR_SEPARATOR + "submitted";
   if (!UIEnsurePathExists(submittedDir)) {
     return false;
   }
 
-  string path = submittedDir + UI_DIR_SEPARATOR +
-    responseItems["CrashID"] + ".txt";
+  string path =
+      submittedDir + UI_DIR_SEPARATOR + responseItems["CrashID"] + ".txt";
 
-  ofstream* file = UIOpenWrite(path);
+  ofstream* file = UIOpenWrite(path, ios::trunc);
   if (!file->is_open()) {
     delete file;
     return false;
   }
 
   char buf[1024];
-  UI_SNPRINTF(buf, 1024,
-              gStrings["CrashID"].c_str(),
-              responseItems["CrashID"].c_str());
+  snprintf(buf, 1024, gStrings["CrashID"].c_str(),
+           responseItems["CrashID"].c_str());
   *file << buf << "\n";
 
   if (responseItems.find("ViewURL") != responseItems.end()) {
-    UI_SNPRINTF(buf, 1024,
-                gStrings["CrashDetailsURL"].c_str(),
-                responseItems["ViewURL"].c_str());
+    snprintf(buf, 1024, gStrings["CrashDetailsURL"].c_str(),
+             responseItems["ViewURL"].c_str());
     *file << buf << "\n";
   }
 
@@ -411,30 +387,23 @@ static bool AddSubmittedReport(const string& serverResponse)
   return true;
 }
 
-void DeleteDump()
-{
+void DeleteDump() {
   const char* noDelete = getenv("MOZ_CRASHREPORTER_NO_DELETE_DUMP");
   if (!noDelete || *noDelete == '\0') {
-    if (!gReporterDumpFile.empty())
-      UIDeleteFile(gReporterDumpFile);
-    if (!gExtraFile.empty())
-      UIDeleteFile(gExtraFile);
-    if (!gMemoryFile.empty())
-      UIDeleteFile(gMemoryFile);
+    if (!gReporterDumpFile.empty()) UIDeleteFile(gReporterDumpFile);
+    if (!gExtraFile.empty()) UIDeleteFile(gExtraFile);
+    if (!gMemoryFile.empty()) UIDeleteFile(gMemoryFile);
   }
 }
 
-void SendCompleted(bool success, const string& serverResponse)
-{
+void SendCompleted(bool success, const string& serverResponse) {
   if (success) {
     if (AddSubmittedReport(serverResponse)) {
       DeleteDump();
-    }
-    else {
+    } else {
       string directory = gReporterDumpFile;
       int slashpos = directory.find_last_of("/\\");
-      if (slashpos < 2)
-        return;
+      if (slashpos < 2) return;
       directory.resize(slashpos);
       UIPruneSavedDumps(directory);
       WriteSubmissionEvent(Failed, "");
@@ -449,11 +418,11 @@ static string ComputeDumpHash() {
   // On Linux we rely on the system-provided libcurl which uses nss so we have
   // to also use the system-provided nss instead of the ones we have bundled.
   const char* libnssNames[] = {
-    "libnss3.so",
-#ifndef HAVE_64BIT_BUILD
-    // 32-bit versions on 64-bit hosts
-    "/usr/lib32/libnss3.so",
-#endif
+      "libnss3.so",
+#  ifndef HAVE_64BIT_BUILD
+      // 32-bit versions on 64-bit hosts
+      "/usr/lib32/libnss3.so",
+#  endif
   };
   void* lib = nullptr;
 
@@ -477,12 +446,12 @@ static string ComputeDumpHash() {
   void (*HASH_Update)(HASHContext*, const unsigned char*, unsigned int);
   void (*HASH_End)(HASHContext*, unsigned char*, unsigned int*, unsigned int);
 
-  *(void**) (&NSS_Initialize) = dlsym(lib, "NSS_Initialize");
-  *(void**) (&HASH_Create) = dlsym(lib, "HASH_Create");
-  *(void**) (&HASH_Destroy) = dlsym(lib, "HASH_Destroy");
-  *(void**) (&HASH_Begin) = dlsym(lib, "HASH_Begin");
-  *(void**) (&HASH_Update) = dlsym(lib, "HASH_Update");
-  *(void**) (&HASH_End) = dlsym(lib, "HASH_End");
+  *(void**)(&NSS_Initialize) = dlsym(lib, "NSS_Initialize");
+  *(void**)(&HASH_Create) = dlsym(lib, "HASH_Create");
+  *(void**)(&HASH_Destroy) = dlsym(lib, "HASH_Destroy");
+  *(void**)(&HASH_Begin) = dlsym(lib, "HASH_Begin");
+  *(void**)(&HASH_Update) = dlsym(lib, "HASH_Update");
+  *(void**)(&HASH_End) = dlsym(lib, "HASH_End");
 
   if (!HASH_Create || !HASH_Destroy || !HASH_Begin || !HASH_Update ||
       !HASH_End) {
@@ -504,7 +473,7 @@ static string ComputeDumpHash() {
 
   HASH_Begin(hashContext);
 
-  ifstream* f = UIOpenRead(gReporterDumpFile, /* binary */ true);
+  ifstream* f = UIOpenRead(gReporterDumpFile, ios::binary);
   bool error = false;
 
   // Read the minidump contents
@@ -512,7 +481,7 @@ static string ComputeDumpHash() {
     uint8_t buff[4096];
 
     do {
-      f->read((char*) buff, sizeof(buff));
+      f->read((char*)buff, sizeof(buff));
 
       if (f->bad()) {
         error = true;
@@ -550,103 +519,100 @@ static string ComputeDumpHash() {
     }
 
     return hash.str();
-  } else {
-    return ""; // If we encountered an error, return an empty hash
   }
+  return "";  // If we encountered an error, return an empty hash
 }
 
-} // namespace CrashReporter
+string GetDumpLocalID() {
+  string localId = Basename(gReporterDumpFile);
+  string::size_type dot = localId.rfind('.');
+
+  if (dot == string::npos) return "";
+
+  return localId.substr(0, dot);
+}
+
+string GetProgramPath(const string& exename) {
+  string path = gArgv[0];
+  size_t pos = path.rfind(UI_CRASH_REPORTER_FILENAME BIN_SUFFIX);
+  path.erase(pos);
+#ifdef XP_MACOSX
+  // On macOS the crash reporter client is shipped as an application bundle
+  // contained within Firefox' main application bundle. So when it's invoked
+  // its current working directory looks like:
+  // Firefox.app/Contents/MacOS/crashreporter.app/Contents/MacOS/
+  // The other applications we ship with Firefox are stored in the main bundle
+  // (Firefox.app/Contents/MacOS/) so we we need to go back three directories
+  // to reach them.
+  path.append("../../../");
+#endif  // XP_MACOSX
+  path.append(exename + BIN_SUFFIX);
+
+  return path;
+}
+
+}  // namespace CrashReporter
 
 using namespace CrashReporter;
 
-void RewriteStrings(StringTable& queryParameters)
-{
+Json::Value kEmptyJsonString("");
+
+void RewriteStrings(Json::Value& aExtraData) {
   // rewrite some UI strings with the values from the query parameters
-  string product = queryParameters["ProductName"];
-  string vendor = queryParameters["Vendor"];
-  if (vendor.empty()) {
-    // Assume Mozilla if no vendor is specified
-    vendor = "Mozilla";
-  }
+  string product = aExtraData.get("ProductName", kEmptyJsonString).asString();
+  Json::Value mozilla("Mozilla");
+  string vendor = aExtraData.get("Vendor", mozilla).asString();
 
   char buf[4096];
-  UI_SNPRINTF(buf, sizeof(buf),
-              gStrings[ST_CRASHREPORTERVENDORTITLE].c_str(),
-              vendor.c_str());
+  snprintf(buf, sizeof(buf), gStrings[ST_CRASHREPORTERVENDORTITLE].c_str(),
+           vendor.c_str());
   gStrings[ST_CRASHREPORTERTITLE] = buf;
 
-
   string str = gStrings[ST_CRASHREPORTERPRODUCTERROR];
   // Only do the replacement here if the string has two
   // format specifiers to start.  Otherwise
   // we assume it has the product name hardcoded.
   string::size_type pos = str.find("%s");
-  if (pos != string::npos)
-    pos = str.find("%s", pos+2);
+  if (pos != string::npos) pos = str.find("%s", pos + 2);
   if (pos != string::npos) {
     // Leave a format specifier for UIError to fill in
-    UI_SNPRINTF(buf, sizeof(buf),
-                gStrings[ST_CRASHREPORTERPRODUCTERROR].c_str(),
-                product.c_str(),
-                "%s");
+    snprintf(buf, sizeof(buf), gStrings[ST_CRASHREPORTERPRODUCTERROR].c_str(),
+             product.c_str(), "%s");
     gStrings[ST_CRASHREPORTERERROR] = buf;
-  }
-  else {
+  } else {
     // product name is hardcoded
     gStrings[ST_CRASHREPORTERERROR] = str;
   }
 
-  UI_SNPRINTF(buf, sizeof(buf),
-              gStrings[ST_CRASHREPORTERDESCRIPTION].c_str(),
-              product.c_str());
+  snprintf(buf, sizeof(buf), gStrings[ST_CRASHREPORTERDESCRIPTION].c_str(),
+           product.c_str());
   gStrings[ST_CRASHREPORTERDESCRIPTION] = buf;
 
-  UI_SNPRINTF(buf, sizeof(buf),
-              gStrings[ST_CHECKSUBMIT].c_str(),
-              vendor.c_str());
+  snprintf(buf, sizeof(buf), gStrings[ST_CHECKSUBMIT].c_str(), vendor.c_str());
   gStrings[ST_CHECKSUBMIT] = buf;
 
-  UI_SNPRINTF(buf, sizeof(buf),
-              gStrings[ST_CHECKEMAIL].c_str(),
-              vendor.c_str());
-  gStrings[ST_CHECKEMAIL] = buf;
-
-  UI_SNPRINTF(buf, sizeof(buf),
-              gStrings[ST_RESTART].c_str(),
-              product.c_str());
+  snprintf(buf, sizeof(buf), gStrings[ST_RESTART].c_str(), product.c_str());
   gStrings[ST_RESTART] = buf;
 
-  UI_SNPRINTF(buf, sizeof(buf),
-              gStrings[ST_QUIT].c_str(),
-              product.c_str());
+  snprintf(buf, sizeof(buf), gStrings[ST_QUIT].c_str(), product.c_str());
   gStrings[ST_QUIT] = buf;
 
-  UI_SNPRINTF(buf, sizeof(buf),
-              gStrings[ST_ERROR_ENDOFLIFE].c_str(),
-              product.c_str());
+  snprintf(buf, sizeof(buf), gStrings[ST_ERROR_ENDOFLIFE].c_str(),
+           product.c_str());
   gStrings[ST_ERROR_ENDOFLIFE] = buf;
 }
 
-bool CheckEndOfLifed(string version)
-{
+bool CheckEndOfLifed(const Json::Value& aVersion) {
+  if (!aVersion.isString()) {
+    return false;
+  }
+
   string reportPath =
-    gSettingsPath + UI_DIR_SEPARATOR + "EndOfLife" + version;
+      gSettingsPath + UI_DIR_SEPARATOR + "EndOfLife" + aVersion.asString();
   return UIFileExists(reportPath);
 }
 
-static string
-GetProgramPath(const string& exename)
-{
-  string path = gArgv[0];
-  size_t pos = path.rfind(UI_CRASH_REPORTER_FILENAME BIN_SUFFIX);
-  path.erase(pos);
-  path.append(exename + BIN_SUFFIX);
-
-  return path;
-}
-
-int main(int argc, char** argv)
-{
+int main(int argc, char** argv) {
   gArgc = argc;
   gArgv = argv;
 
@@ -674,13 +640,14 @@ int main(int argc, char** argv)
   } else {
     // Start by running minidump analyzer to gather stack traces.
     string reporterDumpFile = gReporterDumpFile;
-    vector<string> args = { reporterDumpFile };
+    vector<string> args = {reporterDumpFile};
     string dumpAllThreadsEnv = UIGetEnv("MOZ_CRASHREPORTER_DUMP_ALL_THREADS");
     if (!dumpAllThreadsEnv.empty()) {
       args.insert(args.begin(), "--full");
     }
-    UIRunProgram(GetProgramPath(UI_MINIDUMP_ANALYZER_FILENAME),
-                 args, /* wait */ true);
+    UIRunProgram(CrashReporter::GetProgramPath(UI_MINIDUMP_ANALYZER_FILENAME),
+                 args,
+                 /* wait */ true);
 
     // go ahead with the crash reporter
     gExtraFile = GetAdditionalFilename(gReporterDumpFile, kExtraDataExtension);
@@ -694,28 +661,28 @@ int main(int argc, char** argv)
       return 0;
     }
 
-    gMemoryFile = GetAdditionalFilename(gReporterDumpFile,
-                                        kMemoryReportExtension);
+    gMemoryFile =
+        GetAdditionalFilename(gReporterDumpFile, kMemoryReportExtension);
     if (!UIFileExists(gMemoryFile)) {
       gMemoryFile.erase();
     }
 
-    StringTable queryParameters;
-    if (!ReadStringsFromFile(gExtraFile, queryParameters, true)) {
+    Json::Value extraData;
+    if (!ReadExtraFile(gExtraFile, extraData)) {
       UIError(gStrings[ST_ERROR_EXTRAFILEREAD]);
       return 0;
     }
 
-    if (queryParameters.find("ProductName") == queryParameters.end()) {
+    if (!extraData.isMember("ProductName")) {
       UIError(gStrings[ST_ERROR_NOPRODUCTNAME]);
       return 0;
     }
 
     // There is enough information in the extra file to rewrite strings
     // to be product specific
-    RewriteStrings(queryParameters);
+    RewriteStrings(extraData);
 
-    if (queryParameters.find("ServerURL") == queryParameters.end()) {
+    if (!extraData.isMember("ServerURL")) {
       UIError(gStrings[ST_ERROR_NOSERVERURL]);
       return 0;
     }
@@ -724,8 +691,10 @@ int main(int argc, char** argv)
     // asking the platform-specific code to guess.
     gSettingsPath = UIGetEnv("MOZ_CRASHREPORTER_DATA_DIRECTORY");
     if (gSettingsPath.empty()) {
-      string product = queryParameters["ProductName"];
-      string vendor = queryParameters["Vendor"];
+      string product =
+          extraData.get("ProductName", kEmptyJsonString).asString();
+      string vendor = extraData.get("Vendor", kEmptyJsonString).asString();
+
       if (!UIGetSettingsPath(vendor, product, gSettingsPath)) {
         gSettingsPath.clear();
       }
@@ -742,23 +711,11 @@ int main(int argc, char** argv)
     gPingPath = UIGetEnv("MOZ_CRASHREPORTER_PING_DIRECTORY");
 
     // Assemble and send the crash ping
-    string hash;
+    string hash = ComputeDumpHash();
+
     string pingUuid;
-
-    hash = ComputeDumpHash();
-    if (!hash.empty()) {
-      AppendToEventFile("MinidumpSha256Hash", hash);
-    }
-
-    if (SendCrashPing(queryParameters, hash, pingUuid, gPingPath)) {
-      AppendToEventFile("CrashPingUUID", pingUuid);
-    }
-
-    // Update the crash event with stacks if they are present
-    auto stackTracesItr = queryParameters.find("StackTraces");
-    if (stackTracesItr != queryParameters.end()) {
-      AppendToEventFile(stackTracesItr->first, stackTracesItr->second);
-    }
+    SendCrashPing(extraData, hash, pingUuid, gPingPath);
+    UpdateEventFile(extraData, hash, pingUuid);
 
     if (!UIFileExists(gReporterDumpFile)) {
       UIError(gStrings[ST_ERROR_DUMPFILEEXISTS]);
@@ -771,18 +728,19 @@ int main(int argc, char** argv)
       return 0;
     }
 
-    string sendURL = queryParameters["ServerURL"];
+    string sendURL = extraData.get("ServerURL", kEmptyJsonString).asString();
     // we don't need to actually send these
-    queryParameters.erase("ServerURL");
-    queryParameters.erase("StackTraces");
+    extraData.removeMember("ServerURL");
+    extraData.removeMember("StackTraces");
 
-    queryParameters["Throttleable"] = "1";
+    extraData["SubmittedFrom"] = "Client";
+    extraData["Throttleable"] = "1";
 
     // re-set XUL_APP_FILE for xulrunner wrapped apps
-    const char *appfile = getenv("MOZ_CRASHREPORTER_RESTART_XUL_APP_FILE");
+    const char* appfile = getenv("MOZ_CRASHREPORTER_RESTART_XUL_APP_FILE");
     if (appfile && *appfile) {
       const char prefix[] = "XUL_APP_FILE=";
-      char *env = (char*) malloc(strlen(appfile) + strlen(prefix) + 1);
+      char* env = (char*)malloc(strlen(appfile) + strlen(prefix) + 1);
       if (!env) {
         UIError("Out of memory");
         return 0;
@@ -795,20 +753,35 @@ int main(int argc, char** argv)
 
     vector<string> restartArgs;
 
-    ostringstream paramName;
-    int i = 0;
-    paramName << "MOZ_CRASHREPORTER_RESTART_ARG_" << i++;
-    const char *param = getenv(paramName.str().c_str());
-    while (param && *param) {
-      restartArgs.push_back(param);
+    if (!extraData.isMember("WindowsErrorReporting")) {
+      // We relaunch the application associated with the client, but only when
+      // we encountered a crash caught by the exception handler. Crashes handled
+      // by WER are prevented from directly restarting the application.
+      string programPath = GetProgramPath(MOZ_APP_NAME);
+#ifndef XP_WIN
+      const char* moz_app_launcher = getenv("MOZ_APP_LAUNCHER");
+      if (moz_app_launcher) {
+        programPath = moz_app_launcher;
+      }
+#endif  // XP_WIN
 
-      paramName.str("");
+      restartArgs.push_back(programPath);
+
+      ostringstream paramName;
+      int i = 1;
       paramName << "MOZ_CRASHREPORTER_RESTART_ARG_" << i++;
-      param = getenv(paramName.str().c_str());
+      const char* param = getenv(paramName.str().c_str());
+      while (param && *param) {
+        restartArgs.push_back(param);
+
+        paramName.str("");
+        paramName << "MOZ_CRASHREPORTER_RESTART_ARG_" << i++;
+        param = getenv(paramName.str().c_str());
+      }
     }
 
     // allow override of the server url via environment variable
-    //XXX: remove this in the far future when our robot
+    // XXX: remove this in the far future when our robot
     // masters force everyone to use XULRunner
     char* urlEnv = getenv("MOZ_CRASHREPORTER_URL");
     if (urlEnv && *urlEnv) {
@@ -816,8 +789,9 @@ int main(int argc, char** argv)
     }
 
     // see if this version has been end-of-lifed
-    if (queryParameters.find("Version") != queryParameters.end() &&
-        CheckEndOfLifed(queryParameters["Version"])) {
+
+    if (extraData.isMember("Version") &&
+        CheckEndOfLifed(extraData["Version"])) {
       UIError(gStrings[ST_ERROR_ENDOFLIFE]);
       DeleteDump();
       return 0;
@@ -829,8 +803,9 @@ int main(int argc, char** argv)
       files["memory_report"] = gMemoryFile;
     }
 
-    if (!UIShowCrashUI(files, queryParameters, sendURL, restartArgs))
+    if (!UIShowCrashUI(files, extraData, sendURL, restartArgs)) {
       DeleteDump();
+    }
   }
 
   UIShutdown();
@@ -839,12 +814,11 @@ int main(int argc, char** argv)
 }
 
 #if defined(XP_WIN) && !defined(__GNUC__)
-#include <windows.h>
+#  include <windows.h>
 
 // We need WinMain in order to not be a console app.  This function is unused
 // if we are a console application.
-int WINAPI wWinMain( HINSTANCE, HINSTANCE, LPWSTR args, int )
-{
+int WINAPI wWinMain(HINSTANCE, HINSTANCE, LPWSTR args, int) {
   // Remove everything except close window from the context menu
   {
     HKEY hkApp;
@@ -854,8 +828,8 @@ int WINAPI wWinMain( HINSTANCE, HINSTANCE, LPWSTR args, int )
     RegCloseKey(hkApp);
     if (RegCreateKeyExW(HKEY_CURRENT_USER,
                         L"Software\\Classes\\Applications\\crashreporter.exe",
-                        0, nullptr, REG_OPTION_VOLATILE, KEY_SET_VALUE,
-                        nullptr, &hkApp, nullptr) == ERROR_SUCCESS) {
+                        0, nullptr, REG_OPTION_VOLATILE, KEY_SET_VALUE, nullptr,
+                        &hkApp, nullptr) == ERROR_SUCCESS) {
       RegSetValueExW(hkApp, L"IsHostApp", 0, REG_NONE, 0, 0);
       RegSetValueExW(hkApp, L"NoOpenWith", 0, REG_NONE, 0, 0);
       RegSetValueExW(hkApp, L"NoStartPage", 0, REG_NONE, 0, 0);
