@@ -25,11 +25,7 @@ using namespace CrashReporter;
 static GtkWidget* gViewReportButton = 0;
 static GtkWidget* gCommentTextLabel = 0;
 static GtkWidget* gCommentText = 0;
-static GtkWidget* gEmailMeCheck = 0;
-static GtkWidget* gEmailEntryLabel = 0;
-static GtkWidget* gEmailEntry = 0;
 
-static bool gEmailFieldHint = true;
 static bool gCommentFieldHint = true;
 
 // handle from dlopen'ing libgnome
@@ -43,27 +39,24 @@ static void LoadSettings() {
    *       code in in nsExceptionHandler.cpp.
    */
 
+  bool includeURL = true;
+  bool submitReport = true;
   StringTable settings;
   if (ReadStringsFromFile(gSettingsPath + "/" + kIniFile, settings, true)) {
-    if (settings.find("Email") != settings.end()) {
-      gtk_entry_set_text(GTK_ENTRY(gEmailEntry), settings["Email"].c_str());
-      gEmailFieldHint = false;
+    if (settings.find("IncludeURL") != settings.end()) {
+      includeURL = settings["IncludeURL"][0] != '0';
     }
-    if (settings.find("EmailMe") != settings.end()) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gEmailMeCheck),
-                                   settings["EmailMe"][0] != '0');
+    if (settings.find("SubmitReport") != settings.end()) {
+      submitReport = settings["SubmitReport"][0] != '0';
     }
-    if (settings.find("IncludeURL") != settings.end() &&
-        gIncludeURLCheck != 0) {
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gIncludeURLCheck),
-                                   settings["IncludeURL"][0] != '0');
-    }
-    bool enabled = true;
-    if (settings.find("SubmitReport") != settings.end())
-      enabled = settings["SubmitReport"][0] != '0';
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gSubmitReportCheck),
-                                 enabled);
   }
+
+  if (gIncludeURLCheck) {
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gIncludeURLCheck),
+                                 includeURL);
+  }
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gSubmitReportCheck),
+                               submitReport);
 }
 
 static string Escape(const string& str) {
@@ -83,7 +76,7 @@ static string Escape(const string& str) {
   return ret;
 }
 
-static bool WriteStrings(ostream& out, const string& header,
+static bool WriteStrings(std::ostream& out, const string& header,
                          StringTable& strings, bool escape) {
   out << "[" << header << "]" << std::endl;
   for (const auto& iter : strings) {
@@ -102,7 +95,7 @@ static bool WriteStrings(ostream& out, const string& header,
 
 static bool WriteStringsToFile(const string& path, const string& header,
                                StringTable& strings, bool escape) {
-  ofstream* f = UIOpenWrite(path, ios::trunc);
+  std::ofstream* f = UIOpenWrite(path, ios::trunc);
   bool success = false;
   if (f->is_open()) {
     success = WriteStrings(*f, header, strings, escape);
@@ -122,14 +115,6 @@ void SaveSettings() {
   StringTable settings;
 
   ReadStringsFromFile(gSettingsPath + "/" + kIniFile, settings, true);
-  if (!gEmailFieldHint)
-    settings["Email"] = gtk_entry_get_text(GTK_ENTRY(gEmailEntry));
-  else
-    settings.erase("Email");
-
-  settings["EmailMe"] =
-      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gEmailMeCheck)) ? "1"
-                                                                     : "0";
   if (gIncludeURLCheck != 0)
     settings["IncludeURL"] =
         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gIncludeURLCheck)) ? "1"
@@ -155,8 +140,6 @@ void DisableGUIAndSendReport() {
   gtk_widget_set_sensitive(gViewReportButton, FALSE);
   gtk_widget_set_sensitive(gCommentText, FALSE);
   if (gIncludeURLCheck) gtk_widget_set_sensitive(gIncludeURLCheck, FALSE);
-  gtk_widget_set_sensitive(gEmailMeCheck, FALSE);
-  gtk_widget_set_sensitive(gEmailEntry, FALSE);
   gtk_widget_set_sensitive(gCloseButton, FALSE);
   if (gRestartButton) gtk_widget_set_sensitive(gRestartButton, FALSE);
   gtk_widget_show_all(gThrobber);
@@ -202,18 +185,12 @@ void UpdateSubmit() {
     gtk_widget_set_sensitive(gViewReportButton, TRUE);
     gtk_widget_set_sensitive(gCommentText, TRUE);
     if (gIncludeURLCheck) gtk_widget_set_sensitive(gIncludeURLCheck, TRUE);
-    gtk_widget_set_sensitive(gEmailMeCheck, TRUE);
-    gtk_widget_set_sensitive(
-        gEmailEntry,
-        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gEmailMeCheck)));
     gtk_label_set_text(GTK_LABEL(gProgressLabel),
                        gStrings[ST_REPORTPRESUBMIT].c_str());
   } else {
     gtk_widget_set_sensitive(gViewReportButton, FALSE);
     gtk_widget_set_sensitive(gCommentText, FALSE);
     if (gIncludeURLCheck) gtk_widget_set_sensitive(gIncludeURLCheck, FALSE);
-    gtk_widget_set_sensitive(gEmailMeCheck, FALSE);
-    gtk_widget_set_sensitive(gEmailEntry, FALSE);
     gtk_label_set_text(GTK_LABEL(gProgressLabel), "");
   }
 }
@@ -324,37 +301,6 @@ static gboolean CommentFocusChange(GtkWidget* widget, GdkEventFocus* event,
   return FALSE;
 }
 
-static void UpdateEmail() {
-  const char* email = gtk_entry_get_text(GTK_ENTRY(gEmailEntry));
-  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gEmailMeCheck))) {
-    gtk_widget_set_sensitive(gEmailEntry, TRUE);
-  } else {
-    email = "";
-    gtk_widget_set_sensitive(gEmailEntry, FALSE);
-  }
-  if (email[0] == '\0' || gEmailFieldHint) {
-    gQueryParameters.removeMember("Email");
-  } else {
-    gQueryParameters["Email"] = email;
-  }
-}
-
-static void EmailMeClicked(GtkButton* sender, gpointer userData) {
-  UpdateEmail();
-}
-
-static void EmailChanged(GtkEditable* editable, gpointer userData) {
-  UpdateEmail();
-}
-
-static gboolean EmailFocusChange(GtkWidget* widget, GdkEventFocus* event,
-                                 gpointer userData) {
-  UpdateHintText(widget, event->in, &gEmailFieldHint,
-                 gStrings[ST_EMAILGRAYTEXT].c_str());
-
-  return FALSE;
-}
-
 typedef struct _GnomeProgram GnomeProgram;
 typedef struct _GnomeModuleInfo GnomeModuleInfo;
 typedef GnomeProgram* (*_gnome_program_init_fn)(const char*, const char*,
@@ -372,8 +318,8 @@ void TryInitGnome() {
   _gnome_program_init_fn gnome_program_init =
       (_gnome_program_init_fn)(dlsym(gnomeLib, "gnome_program_init"));
   _libgnomeui_module_info_get_fn libgnomeui_module_info_get =
-      (_libgnomeui_module_info_get_fn)(
-          dlsym(gnomeuiLib, "libgnomeui_module_info_get"));
+      (_libgnomeui_module_info_get_fn)(dlsym(gnomeuiLib,
+                                             "libgnomeui_module_info_get"));
 
   if (gnome_program_init && libgnomeui_module_info_get) {
     gnome_program_init("crashreporter", "1.0", libgnomeui_module_info_get(),
@@ -523,26 +469,6 @@ bool UIShowCrashUI(const StringTable& files, const Json::Value& queryParameters,
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gIncludeURLCheck), TRUE);
   }
 
-  gEmailMeCheck =
-      gtk_check_button_new_with_label(gStrings[ST_CHECKEMAIL].c_str());
-  gtk_box_pack_start(GTK_BOX(innerVBox), gEmailMeCheck, FALSE, FALSE, 0);
-  g_signal_connect(gEmailMeCheck, "clicked", G_CALLBACK(EmailMeClicked), 0);
-
-  GtkWidget* emailIndentBox = gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(innerVBox), emailIndentBox, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(emailIndentBox), gtk_label_new(""), FALSE, FALSE,
-                     9);
-
-  gEmailEntryLabel = gtk_label_new(gStrings[ST_EMAILGRAYTEXT].c_str());
-  gEmailEntry = gtk_entry_new();
-  gtk_label_set_mnemonic_widget(GTK_LABEL(gEmailEntryLabel), gEmailEntry);
-  gtk_box_pack_start(GTK_BOX(emailIndentBox), gEmailEntry, TRUE, TRUE, 0);
-  g_signal_connect(gEmailEntry, "changed", G_CALLBACK(EmailChanged), 0);
-  g_signal_connect(gEmailEntry, "focus-in-event", G_CALLBACK(EmailFocusChange),
-                   0);
-  g_signal_connect(gEmailEntry, "focus-out-event", G_CALLBACK(EmailFocusChange),
-                   0);
-
   GtkWidget* progressBox = gtk_hbox_new(FALSE, 6);
   gtk_box_pack_start(GTK_BOX(vbox), progressBox, TRUE, TRUE, 0);
 
@@ -584,13 +510,10 @@ bool UIShowCrashUI(const StringTable& files, const Json::Value& queryParameters,
 
   LoadSettings();
 
-  UpdateEmail();
   UpdateSubmit();
 
   UpdateHintText(gCommentText, FALSE, &gCommentFieldHint,
                  gStrings[ST_COMMENTGRAYTEXT].c_str());
-  UpdateHintText(gEmailEntry, FALSE, &gEmailFieldHint,
-                 gStrings[ST_EMAILGRAYTEXT].c_str());
 
   gtk_widget_show_all(gWindow);
   // stick this here to avoid the show_all above...
