@@ -14,9 +14,9 @@
 #include <fcntl.h>
 #include <sstream>
 
+using std::ostringstream;
 using std::string;
 using std::vector;
-using std::ostringstream;
 
 using namespace CrashReporter;
 
@@ -37,6 +37,8 @@ static cpu_type_t pref_cpu_types[2] = {
     CPU_TYPE_X86_64,
 #elif defined(__ppc__)
     CPU_TYPE_POWERPC,
+#elif defined(__aarch64__)
+    CPU_TYPE_ARM64,
 #endif
     CPU_TYPE_ANY};
 
@@ -120,12 +122,10 @@ static bool RestartApplication() {
 
   [mSubmitReportButton setTitle:Str(ST_CHECKSUBMIT)];
   [mIncludeURLButton setTitle:Str(ST_CHECKURL)];
-  [mEmailMeButton setTitle:Str(ST_CHECKEMAIL)];
   [mViewReportOkButton setTitle:Str(ST_OK)];
 
   [mCommentText setPlaceholder:Str(ST_COMMENTGRAYTEXT)];
   if (gRTLlayout) [mCommentText toggleBaseWritingDirection:self];
-  [[mEmailText cell] setPlaceholderString:Str(ST_EMAILGRAYTEXT)];
 
   if (gQueryParameters.isMember("URL")) {
     // save the URL value in case the checkbox gets unchecked
@@ -169,6 +169,15 @@ static bool RestartApplication() {
   }
   [mSubmitReportButton setState:(submitChecked ? NSOnState : NSOffState)];
 
+  // load default state of include URL checkbox
+  BOOL includeChecked = YES;
+  if (nil != [userDefaults objectForKey:@"IncludeURL"]) {
+    includeChecked = [userDefaults boolForKey:@"IncludeURL"];
+  } else {
+    [userDefaults setBool:includeChecked forKey:@"IncludeURL"];
+  }
+  [mIncludeURLButton setState:(includeChecked ? NSOnState : NSOffState)];
+
   [self updateSubmit];
   [self updateURL];
   [self updateEmail];
@@ -189,12 +198,14 @@ static bool RestartApplication() {
 }
 
 - (void)showReportInfo {
-  NSDictionary* boldAttr =
-      [NSDictionary dictionaryWithObject:[NSFont boldSystemFontOfSize:[NSFont smallSystemFontSize]]
-                                  forKey:NSFontAttributeName];
-  NSDictionary* normalAttr =
-      [NSDictionary dictionaryWithObject:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]
-                                  forKey:NSFontAttributeName];
+  NSDictionary* boldAttr = @{
+    NSFontAttributeName : [NSFont boldSystemFontOfSize:[NSFont smallSystemFontSize]],
+    NSForegroundColorAttributeName : NSColor.textColor,
+  };
+  NSDictionary* normalAttr = @{
+    NSFontAttributeName : [NSFont systemFontOfSize:[NSFont smallSystemFontSize]],
+    NSForegroundColorAttributeName : NSColor.textColor,
+  };
 
   [mViewReportTextView setString:@""];
   for (Json::ValueConstIterator iter = gQueryParameters.begin(); iter != gQueryParameters.end();
@@ -276,14 +287,9 @@ static bool RestartApplication() {
 
 - (IBAction)includeURLClicked:(id)sender {
   [self updateURL];
-}
-
-- (IBAction)emailMeClicked:(id)sender {
-  [self updateEmail];
-}
-
-- (void)controlTextDidChange:(NSNotification*)note {
-  [self updateEmail];
+  NSUserDefaults* userDefaults = [NSUserDefaults standardUserDefaults];
+  [userDefaults setBool:([mIncludeURLButton state] == NSOnState) forKey:@"IncludeURL"];
+  [userDefaults synchronize];
 }
 
 - (void)textDidChange:(NSNotification*)aNotification {
@@ -360,20 +366,20 @@ static bool RestartApplication() {
     [mRestartButton setKeyEquivalent:@"\r"];
   }
 
-  NSButton* checkboxes[] = {mSubmitReportButton, mIncludeURLButton, mEmailMeButton};
+  NSButton* checkboxes[] = {mSubmitReportButton, mIncludeURLButton};
 
-  for (int i = 0; i < 3; i++) {
-    NSRect frame = [checkboxes[i] frame];
-    [checkboxes[i] sizeToFit];
+  for (auto checkbox : checkboxes) {
+    NSRect frame = [checkbox frame];
+    [checkbox sizeToFit];
     if (gRTLlayout) {
       // sizeToFit will keep the left side fixed, so realign
       float oldWidth = frame.size.width;
-      frame = [checkboxes[i] frame];
+      frame = [checkbox frame];
       frame.origin.x += oldWidth - frame.size.width;
-      [checkboxes[i] setFrame:frame];
+      [checkbox setFrame:frame];
     }
     // keep existing spacing on left side, + 20 px spare on right
-    float neededWidth = frame.origin.x + frame.size.width + 20;
+    float neededWidth = frame.origin.x + checkbox.intrinsicContentSize.width + 20;
     if (neededWidth > windowFrame.size.width) {
       windowFrame.size.width = neededWidth;
       [mWindow setFrame:windowFrame display:true animate:NO];
@@ -389,10 +395,10 @@ static bool RestartApplication() {
   // now pin all the controls (except quit/submit) in place,
   // if we lengthen the window after this, it's just to lengthen
   // the progress text, so nothing above that text should move.
-  NSView* views[] = {mSubmitReportButton, mViewReportButton, mCommentScrollView, mIncludeURLButton,
-                     mEmailMeButton,      mEmailText,        mProgressIndicator, mProgressText};
-  for (unsigned int i = 0; i < sizeof(views) / sizeof(views[0]); i++) {
-    [views[i] setAutoresizingMask:NSViewMinYMargin];
+  NSView* views[] = {mSubmitReportButton, mViewReportButton,  mCommentScrollView,
+                     mIncludeURLButton,   mProgressIndicator, mProgressText};
+  for (auto view : views) {
+    [view setAutoresizingMask:NSViewMinYMargin];
   }
 }
 
@@ -442,10 +448,8 @@ static bool RestartApplication() {
 - (void)enableControls:(BOOL)enabled {
   [mViewReportButton setEnabled:enabled];
   [mIncludeURLButton setEnabled:enabled];
-  [mEmailMeButton setEnabled:enabled];
   [mCommentText setEnabled:enabled];
   [mCommentScrollView setHasVerticalScroller:enabled];
-  [self updateEmail];
 }
 
 - (void)updateSubmit {
@@ -471,14 +475,11 @@ static bool RestartApplication() {
 }
 
 - (void)updateEmail {
-  if ([mEmailMeButton state] == NSOnState && [mSubmitReportButton state] == NSOnState) {
-    NSString* email = [mEmailText stringValue];
-    gQueryParameters["Email"] = [email UTF8String];
-    [mEmailText setEnabled:YES];
-  } else {
-    gQueryParameters.removeMember("Email");
-    [mEmailText setEnabled:NO];
-  }
+  // In order to remove the email fields, we have to edit the .nib files which
+  // we can't do with current xcode so we make them hidden; updating the
+  // crashreporter interface for mac is covered in bug #1696164
+  [mEmailMeButton setHidden:YES];
+  [mEmailText setHidden:YES];
 }
 
 - (void)sendReport {
@@ -625,7 +626,7 @@ static bool RestartApplication() {
   mPlaceHolderString = [[NSMutableAttributedString alloc] initWithString:placeholder
                                                               attributes:txtDict];
   if (gRTLlayout)
-    [mPlaceHolderString setAlignment:NSRightTextAlignment
+    [mPlaceHolderString setAlignment:NSTextAlignmentRight
                                range:NSMakeRange(0, [placeholder length])];
 }
 
@@ -724,13 +725,9 @@ bool UIGetIniPath(string& path) {
 }
 
 bool UIGetSettingsPath(const string& vendor, const string& product, string& settingsPath) {
-  FSRef foundRef;
-  OSErr err = FSFindFolder(kUserDomain, kApplicationSupportFolderType, kCreateFolder, &foundRef);
-  if (err != noErr) return false;
-
-  unsigned char path[PATH_MAX];
-  FSRefMakePath(&foundRef, path, sizeof(path));
-  NSString* destPath = [NSString stringWithUTF8String:reinterpret_cast<char*>(path)];
+  NSArray* paths =
+      NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
+  NSString* destPath = [paths firstObject];
 
   // Note that MacOS ignores the vendor when creating the profile hierarchy -
   // all application preferences directories live alongside one another in
