@@ -1,20 +1,17 @@
 /* eslint max-len: ["error", 80] */
 
-const { AddonTestUtils } = ChromeUtils.import(
-  "resource://testing-common/AddonTestUtils.jsm"
+const { AddonTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/AddonTestUtils.sys.mjs"
 );
 
 AddonTestUtils.initMochitest(this);
 
 let promptService;
 
-const SECTION_INDEXES = {
-  enabled: 0,
-  disabled: 1,
-};
-function getSection(doc, type) {
-  return doc.querySelector(`section[section="${SECTION_INDEXES[type]}"]`);
-}
+const SUPPORT_URL = Services.urlFormatter.formatURL(
+  Services.prefs.getStringPref("app.support.baseURL")
+);
+const REMOVE_SUMO_URL = SUPPORT_URL + "cant-remove-addon";
 
 function getTestCards(root) {
   return root.querySelectorAll('addon-card[addon-id$="@mochi.test"]');
@@ -25,7 +22,7 @@ function getCardByAddonId(root, id) {
 }
 
 function isEmpty(el) {
-  return el.children.length == 0;
+  return !el.children.length;
 }
 
 function waitForThemeChange(list) {
@@ -34,12 +31,11 @@ function waitForThemeChange(list) {
   return BrowserTestUtils.waitForEvent(list, "move", () => ++moveCount == 2);
 }
 
-add_task(async function enableHtmlViews() {
-  await SpecialPowers.pushPrefEnv({
-    set: [["extensions.htmlaboutaddons.enabled", true]],
-  });
+let mockProvider;
+
+add_setup(async function () {
+  mockProvider = new MockProvider(["extension", "sitepermission"]);
   promptService = mockPromptService();
-  Services.telemetry.clearEvents();
 });
 
 let extensionsCreated = 0;
@@ -49,7 +45,7 @@ function createExtensions(manifestExtras) {
     ExtensionTestUtils.loadExtension({
       manifest: {
         name: "Test extension",
-        applications: {
+        browser_specific_settings: {
           gecko: { id: `test-${extensionsCreated++}@mochi.test` },
         },
         icons: {
@@ -64,10 +60,11 @@ function createExtensions(manifestExtras) {
 
 add_task(async function testExtensionList() {
   let id = "test@mochi.test";
+  let headingId = "test_mochi_test-heading";
   let extension = ExtensionTestUtils.loadExtension({
     manifest: {
       name: "Test extension",
-      applications: { gecko: { id } },
+      browser_specific_settings: { gecko: { id } },
       icons: {
         32: "test-icon.png",
       },
@@ -86,30 +83,46 @@ add_task(async function testExtensionList() {
   let list = doc.querySelector("addon-list");
 
   // There shouldn't be any disabled extensions.
-  let disabledSection = getSection(doc, "disabled");
+  let disabledSection = getSection(doc, "extension-disabled-section");
   ok(isEmpty(disabledSection), "The disabled section is empty");
 
   // The loaded extension should be in the enabled list.
-  let enabledSection = getSection(doc, "enabled");
-  ok(!isEmpty(enabledSection), "The enabled section isn't empty");
+  let enabledSection = getSection(doc, "extension-enabled-section");
+  ok(
+    enabledSection && !isEmpty(enabledSection),
+    "The enabled section isn't empty"
+  );
   let card = getCardByAddonId(enabledSection, id);
   ok(card, "The card is in the enabled section");
 
   // Check the properties of the card.
   is(card.addonNameEl.textContent, "Test extension", "The name is set");
+  is(
+    card.querySelector("h3").id,
+    headingId,
+    "The add-on name has the correct id"
+  );
+  is(
+    card.querySelector(".card").getAttribute("aria-labelledby"),
+    headingId,
+    "The card is labelled by the heading"
+  );
   let icon = card.querySelector(".addon-icon");
   ok(icon.src.endsWith("/test-icon.png"), "The icon is set");
 
   // Disable the extension.
-  let disableButton = card.querySelector('[action="toggle-disabled"]');
+  let disableToggle = card.querySelector('[action="toggle-disabled"]');
+  ok(disableToggle.pressed, "The disable toggle is pressed");
   is(
-    doc.l10n.getAttributes(disableButton).id,
-    "disable-addon-button",
-    "The button has the disable label"
+    doc.l10n.getAttributes(disableToggle).id,
+    "extension-enable-addon-button-label",
+    "The toggle has the enable label"
   );
+  ok(disableToggle.getAttribute("aria-label"), "There's an aria-label");
+  ok(!disableToggle.hidden, "The toggle is visible");
 
   let disabled = BrowserTestUtils.waitForEvent(list, "move");
-  disableButton.click();
+  disableToggle.click();
   await disabled;
   is(
     card.parentNode,
@@ -117,12 +130,14 @@ add_task(async function testExtensionList() {
     "The card is now in the disabled section"
   );
 
-  // The disable button is now enable.
+  // The disable button is now enabled.
+  ok(!disableToggle.pressed, "The disable toggle is not pressed");
   is(
-    doc.l10n.getAttributes(disableButton).id,
-    "enable-addon-button",
-    "The button has the enable label"
+    doc.l10n.getAttributes(disableToggle).id,
+    "extension-enable-addon-button-label",
+    "The button has the same enable label"
   );
+  ok(disableToggle.getAttribute("aria-label"), "There's an aria-label");
 
   // Remove the add-on.
   let removeButton = card.querySelector('[action="remove"]');
@@ -131,6 +146,9 @@ add_task(async function testExtensionList() {
     "remove-addon-button",
     "The button has the remove label"
   );
+  // There is a support link when the add-on isn't removeable, verify we don't
+  // always include one.
+  ok(!removeButton.querySelector("a"), "There isn't a link in the item");
 
   // Remove but cancel.
   let cancelled = BrowserTestUtils.waitForEvent(card, "remove-cancelled");
@@ -161,7 +179,7 @@ add_task(async function testExtensionList() {
   const extension2 = ExtensionTestUtils.loadExtension({
     manifest: {
       name: "Test extension 2",
-      applications: { gecko: { id: "test-2@mochi.test" } },
+      browser_specific_settings: { gecko: { id: "test-2@mochi.test" } },
       icons: {
         32: "test-icon.png",
       },
@@ -215,7 +233,7 @@ add_task(async function testExtensionList() {
   const themeXpi = AddonTestUtils.createTempWebExtensionFile({
     manifest: {
       name: "My theme",
-      applications: { gecko: { id: "theme@mochi.test" } },
+      browser_specific_settings: { gecko: { id: "theme@mochi.test" } },
       theme: {},
     },
   });
@@ -229,7 +247,7 @@ add_task(async function testExtensionList() {
   const xpi = AddonTestUtils.createTempWebExtensionFile({
     manifest: {
       name: "Test extension 3",
-      applications: { gecko: { id: "test-3@mochi.test" } },
+      browser_specific_settings: { gecko: { id: "test-3@mochi.test" } },
       icons: {
         32: "test-icon.png",
       },
@@ -276,57 +294,13 @@ add_task(async function testExtensionList() {
     !(await AddonManager.getAddonByID(themeAddon.id)),
     "The theme addon is fully uninstalled"
   );
-
-  assertAboutAddonsTelemetryEvents([
-    ["addonsManager", "view", "aboutAddons", "list", { type: "extension" }],
-    [
-      "addonsManager",
-      "action",
-      "aboutAddons",
-      null,
-      { type: "extension", addonId: id, view: "list", action: "disable" },
-    ],
-    [
-      "addonsManager",
-      "action",
-      "aboutAddons",
-      "cancelled",
-      { type: "extension", addonId: id, view: "list", action: "uninstall" },
-    ],
-    [
-      "addonsManager",
-      "action",
-      "aboutAddons",
-      "accepted",
-      { type: "extension", addonId: id, view: "list", action: "uninstall" },
-    ],
-    [
-      "addonsManager",
-      "action",
-      "aboutAddons",
-      null,
-      { type: "extension", addonId: id, view: "list", action: "undo" },
-    ],
-    [
-      "addonsManager",
-      "action",
-      "aboutAddons",
-      null,
-      {
-        type: "extension",
-        addonId: "test-2@mochi.test",
-        view: "list",
-        action: "undo",
-      },
-    ],
-  ]);
 });
 
 add_task(async function testMouseSupport() {
   let extension = ExtensionTestUtils.loadExtension({
     manifest: {
       name: "Test extension",
-      applications: { gecko: { id: "test@mochi.test" } },
+      browser_specific_settings: { gecko: { id: "test@mochi.test" } },
     },
     useAddonManager: "temporary",
   });
@@ -338,14 +312,13 @@ add_task(async function testMouseSupport() {
   let [card] = getTestCards(doc);
   is(card.addon.id, "test@mochi.test", "The right card is found");
 
-  let menuButton = card.querySelector('[action="more-options"]');
   let panel = card.querySelector("panel-list");
 
   ok(!panel.open, "The panel is initially closed");
   await BrowserTestUtils.synthesizeMouseAtCenter(
-    menuButton,
+    "addon-card[addon-id$='@mochi.test'] button[action='more-options']",
     { type: "mousedown" },
-    gBrowser.selectedBrowser
+    win.docShell.browsingContext
   );
   ok(panel.open, "The panel is now open");
 
@@ -354,10 +327,11 @@ add_task(async function testMouseSupport() {
 });
 
 add_task(async function testKeyboardSupport() {
+  let id = "test@mochi.test";
   let extension = ExtensionTestUtils.loadExtension({
     manifest: {
       name: "Test extension",
-      applications: { gecko: { id: "test@mochi.test" } },
+      browser_specific_settings: { gecko: { id } },
     },
     useAddonManager: "temporary",
   });
@@ -373,8 +347,8 @@ add_task(async function testKeyboardSupport() {
 
   // Find the addon-list to listen for events.
   let list = doc.querySelector("addon-list");
-  let enabledSection = getSection(doc, "enabled");
-  let disabledSection = getSection(doc, "disabled");
+  let enabledSection = getSection(doc, "extension-enabled-section");
+  let disabledSection = getSection(doc, "extension-disabled-section");
 
   // Find the card.
   let [card] = getTestCards(list);
@@ -388,63 +362,60 @@ add_task(async function testKeyboardSupport() {
   // Test opening and closing the menu.
   let moreOptionsMenu = card.querySelector("panel-list");
   let expandButton = moreOptionsMenu.querySelector('[action="expand"]');
-  let toggleDisableButton = card.querySelector('[action="toggle-disabled"]');
+  let removeButton = card.querySelector('[action="remove"]');
   is(moreOptionsMenu.open, false, "The menu is closed");
   let shown = BrowserTestUtils.waitForEvent(moreOptionsMenu, "shown");
   space();
   await shown;
   is(moreOptionsMenu.open, true, "The menu is open");
-  isFocused(toggleDisableButton, "The disable button is now focused");
-  EventUtils.synthesizeKey("Escape", {});
-  is(moreOptionsMenu.open, false, "The menu is closed");
-  isFocused(moreOptionsButton, "The more options button is focused");
-
-  // Test tabbing out of the menu.
-  space();
-  shown = BrowserTestUtils.waitForEvent(moreOptionsMenu, "shown");
-  is(moreOptionsMenu.open, true, "The menu is open");
-  await shown;
+  isFocused(removeButton, "The remove button is now focused");
   tab({ shiftKey: true });
   is(moreOptionsMenu.open, true, "The menu stays open");
   isFocused(expandButton, "The focus has looped to the bottom");
   tab();
   is(moreOptionsMenu.open, true, "The menu stays open");
-  isFocused(toggleDisableButton, "The focus has looped to the top");
+  isFocused(removeButton, "The focus has looped to the top");
 
   let hidden = BrowserTestUtils.waitForEvent(moreOptionsMenu, "hidden");
   EventUtils.synthesizeKey("Escape", {});
   await hidden;
   isFocused(moreOptionsButton, "Escape closed the menu");
 
-  // Open the menu to test contents.
-  shown = BrowserTestUtils.waitForEvent(moreOptionsMenu, "shown");
-  space();
-  is(moreOptionsMenu.open, true, "The menu is open");
-  // Wait for the panel to be shown.
-  await shown;
-
   // Disable the add-on.
-  isFocused(toggleDisableButton, "The disable button is focused");
+  let disableButton = card.querySelector('[action="toggle-disabled"]');
+  tab({ shiftKey: true });
+  isFocused(disableButton, "The disable toggle is focused");
   is(card.parentNode, enabledSection, "The card is in the enabled section");
-  let disabled = BrowserTestUtils.waitForEvent(list, "move");
   space();
-  await disabled;
-  is(moreOptionsMenu.open, false, "The menu is closed");
+  // Wait for the add-on state to change.
+  let [disabledAddon] = await AddonTestUtils.promiseAddonEvent("onDisabled");
+  is(disabledAddon.id, id, "The right add-on was disabled");
+  is(
+    card.parentNode,
+    enabledSection,
+    "The card is still in the enabled section"
+  );
+  isFocused(disableButton, "The disable button is still focused");
+  let moved = BrowserTestUtils.waitForEvent(list, "move");
+  // Click outside the list to clear any focus.
+  EventUtils.synthesizeMouseAtCenter(
+    doc.querySelector(".header-name"),
+    {},
+    win
+  );
+  await moved;
   is(
     card.parentNode,
     disabledSection,
-    "The card is now in the disabled section"
+    "The card moved when keyboard focus left the list"
   );
 
-  // Open the menu again.
-  shown = BrowserTestUtils.waitForEvent(moreOptionsMenu, "shown");
-  isFocused(moreOptionsButton, "The more options button is focused");
-  space();
-  await shown;
-
   // Remove the add-on.
-  tab();
-  let removeButton = card.querySelector('[action="remove"]');
+  moreOptionsButton.focus();
+  shown = BrowserTestUtils.waitForEvent(moreOptionsMenu, "shown");
+  space();
+  is(moreOptionsMenu.open, true, "The menu is open");
+  await shown;
   isFocused(removeButton, "The remove button is focused");
   let removed = BrowserTestUtils.waitForEvent(list, "remove");
   space();
@@ -460,7 +431,7 @@ add_task(async function testOpenDetailFromNameKeyboard() {
   let extension = ExtensionTestUtils.loadExtension({
     manifest: {
       name: "Detail extension",
-      applications: { gecko: { id } },
+      browser_specific_settings: { gecko: { id } },
     },
     useAddonManager: "temporary",
   });
@@ -504,7 +475,7 @@ add_task(async function testExtensionReordering() {
   let list = doc.querySelector("addon-list");
 
   // Find the related cards, they should all have @mochi.test ids.
-  let enabledSection = getSection(doc, "enabled");
+  let enabledSection = getSection(doc, "extension-enabled-section");
   let cards = getTestCards(enabledSection);
 
   is(cards.length, 3, "Each extension has an addon-card");
@@ -517,7 +488,7 @@ add_task(async function testExtensionReordering() {
   );
 
   // Disable the second extension.
-  let disabledSection = getSection(doc, "disabled");
+  let disabledSection = getSection(doc, "extension-disabled-section");
   ok(isEmpty(disabledSection), "The disabled section is initially empty");
 
   // Disable the add-ons in a different order.
@@ -582,7 +553,7 @@ add_task(async function testExtensionReordering() {
 add_task(async function testThemeList() {
   let theme = ExtensionTestUtils.loadExtension({
     manifest: {
-      applications: { gecko: { id: "theme@mochi.test" } },
+      browser_specific_settings: { gecko: { id: "theme@mochi.test" } },
       name: "My theme",
       theme: {},
     },
@@ -607,8 +578,8 @@ add_task(async function testThemeList() {
   let [card] = cards;
   is(card.addon.name, "My theme", "The card is for the test theme");
 
-  let enabledSection = getSection(doc, "enabled");
-  let disabledSection = getSection(doc, "disabled");
+  let enabledSection = getSection(doc, "theme-enabled-section");
+  let disabledSection = getSection(doc, "theme-disabled-section");
 
   await TestUtils.waitForCondition(
     () => enabledSection.querySelectorAll("addon-card").length == 1
@@ -625,13 +596,17 @@ add_task(async function testThemeList() {
     "There is one enabled theme"
   );
 
-  let themesChanged = waitForThemeChange(list);
-  card.querySelector('[action="toggle-disabled"]').click();
-  await themesChanged;
+  let toggleThemeEnabled = async () => {
+    let themesChanged = waitForThemeChange(list);
+    card.querySelector('[action="toggle-disabled"]').click();
+    await themesChanged;
 
-  await TestUtils.waitForCondition(
-    () => enabledSection.querySelectorAll("addon-card").length == 1
-  );
+    await TestUtils.waitForCondition(
+      () => enabledSection.querySelectorAll("addon-card").length == 1
+    );
+  };
+
+  await toggleThemeEnabled();
 
   is(
     card.parentNode,
@@ -644,6 +619,31 @@ add_task(async function testThemeList() {
     "There is one enabled theme"
   );
 
+  // Re-enable the theme.
+  await toggleThemeEnabled();
+  is(card.parentNode, enabledSection, "Card is back in the Enabled section");
+
+  // Remove theme and verify that the default theme is re-enabled.
+  let removed = BrowserTestUtils.waitForEvent(list, "remove");
+  // Confirm removal.
+  promptService._response = 0;
+  card.querySelector('[action="remove"]').click();
+  await removed;
+  is(card.parentNode, null, "Card has been removed from the view");
+  await TestUtils.waitForCondition(
+    () => enabledSection.querySelectorAll("addon-card").length == 1
+  );
+
+  let defaultTheme = getCardByAddonId(doc, "default-theme@mozilla.org");
+  is(defaultTheme.parentNode, enabledSection, "The default theme is reenabled");
+
+  await testUndoPendingUninstall(list, card.addon);
+  await TestUtils.waitForCondition(
+    () => enabledSection.querySelectorAll("addon-card").length == 1
+  );
+  is(defaultTheme.parentNode, disabledSection, "The default theme is disabled");
+  ok(getCardByAddonId(enabledSection, theme.id), "Theme should be reenabled");
+
   await theme.unload();
   await closeView(win);
 });
@@ -654,8 +654,8 @@ add_task(async function testBuiltInThemeButtons() {
 
   // Find the addon-list to listen for events.
   let list = doc.querySelector("addon-list");
-  let enabledSection = getSection(doc, "enabled");
-  let disabledSection = getSection(doc, "disabled");
+  let enabledSection = getSection(doc, "theme-enabled-section");
+  let disabledSection = getSection(doc, "theme-disabled-section");
 
   let defaultTheme = getCardByAddonId(doc, "default-theme@mozilla.org");
   let darkTheme = getCardByAddonId(doc, "firefox-compact-dark@mozilla.org");
@@ -711,6 +711,59 @@ add_task(async function testBuiltInThemeButtons() {
   await closeView(win);
 });
 
+add_task(async function testSideloadRemoveButton() {
+  const id = "sideload@mochi.test";
+  mockProvider.createAddons([
+    {
+      id,
+      name: "Sideloaded",
+      permissions: 0,
+    },
+  ]);
+
+  let win = await loadInitialView("extension");
+  let doc = win.document;
+
+  let card = getCardByAddonId(doc, id);
+
+  let moreOptionsPanel = card.querySelector("panel-list");
+  let moreOptionsButton = card.querySelector('[action="more-options"]');
+  let panelOpened = BrowserTestUtils.waitForEvent(moreOptionsPanel, "shown");
+  EventUtils.synthesizeMouseAtCenter(moreOptionsButton, {}, win);
+  await panelOpened;
+
+  // Verify the remove button is visible with a SUMO link.
+  let removeButton = card.querySelector('[action="remove"]');
+  ok(removeButton.disabled, "Remove is disabled");
+  ok(!removeButton.hidden, "Remove is visible");
+
+  // Remove but cancel.
+  let prevented = BrowserTestUtils.waitForEvent(card, "remove-disabled");
+  removeButton.click();
+  await prevented;
+
+  // reopen the panel
+  panelOpened = BrowserTestUtils.waitForEvent(moreOptionsPanel, "shown");
+  EventUtils.synthesizeMouseAtCenter(moreOptionsButton, {}, win);
+  await panelOpened;
+
+  let sumoLink = removeButton.querySelector("a");
+  ok(sumoLink, "There's a link");
+  is(
+    doc.l10n.getAttributes(removeButton).id,
+    "remove-addon-disabled-button",
+    "The can't remove text is shown"
+  );
+  sumoLink.focus();
+  is(doc.activeElement, sumoLink, "The link can be focused");
+
+  let newTabOpened = BrowserTestUtils.waitForNewTab(gBrowser, REMOVE_SUMO_URL);
+  sumoLink.click();
+  BrowserTestUtils.removeTab(await newTabOpened);
+
+  await closeView(win);
+});
+
 add_task(async function testOnlyTypeIsShown() {
   let win = await loadInitialView("theme");
   let doc = win.document;
@@ -721,7 +774,7 @@ add_task(async function testOnlyTypeIsShown() {
   let extension = ExtensionTestUtils.loadExtension({
     manifest: {
       name: "Test extension",
-      applications: { gecko: { id: "test@mochi.test" } },
+      browser_specific_settings: { gecko: { id: "test@mochi.test" } },
     },
     useAddonManager: "temporary",
   });
@@ -742,14 +795,14 @@ add_task(async function testOnlyTypeIsShown() {
 });
 
 add_task(async function testPluginIcons() {
-  const pluginIconUrl = "chrome://global/skin/plugins/pluginGeneric.svg";
+  const pluginIconUrl = "chrome://global/skin/icons/plugin.svg";
 
   let win = await loadInitialView("plugin");
   let doc = win.document;
 
   // Check that the icons are set to the plugin icon.
   let icons = doc.querySelectorAll(".card-heading-icon");
-  ok(icons.length > 0, "There are some plugins listed");
+  ok(!!icons.length, "There are some plugins listed");
 
   for (let icon of icons) {
     is(icon.src, pluginIconUrl, "Plugins use the plugin icon");
@@ -766,7 +819,7 @@ add_task(async function testExtensionGenericIcon() {
   let extension = ExtensionTestUtils.loadExtension({
     manifest: {
       name: "Test extension",
-      applications: { gecko: { id } },
+      browser_specific_settings: { gecko: { id } },
     },
     useAddonManager: "temporary",
   });
@@ -782,3 +835,204 @@ add_task(async function testExtensionGenericIcon() {
   await extension.unload();
   await closeView(win);
 });
+
+add_task(async function testSectionHeadingKeys() {
+  mockProvider.createAddons([
+    {
+      id: "test-theme",
+      name: "Test Theme",
+      type: "theme",
+    },
+    {
+      id: "test-extension-disabled",
+      name: "Test Disabled Extension",
+      type: "extension",
+      userDisabled: true,
+    },
+    {
+      id: "test-plugin-disabled",
+      name: "Test Disabled Plugin",
+      type: "plugin",
+      userDisabled: true,
+    },
+    {
+      id: "test-locale",
+      name: "Test Enabled Locale",
+      type: "locale",
+    },
+    {
+      id: "test-locale-disabled",
+      name: "Test Disabled Locale",
+      type: "locale",
+      userDisabled: true,
+    },
+    {
+      id: "test-dictionary",
+      name: "Test Enabled Dictionary",
+      type: "dictionary",
+    },
+    {
+      id: "test-dictionary-disabled",
+      name: "Test Disabled Dictionary",
+      type: "dictionary",
+      userDisabled: true,
+    },
+    {
+      id: "test-sitepermission",
+      name: "Test Enabled Site Permission",
+      type: "sitepermission",
+    },
+    {
+      id: "test-sitepermission-disabled",
+      name: "Test Disabled Site Permission",
+      type: "sitepermission",
+      userDisabled: true,
+    },
+  ]);
+
+  for (let type of [
+    "extension",
+    "theme",
+    "plugin",
+    "locale",
+    "dictionary",
+    "sitepermission",
+  ]) {
+    info(`loading view for addon type ${type}`);
+    let win = await loadInitialView(type);
+    let doc = win.document;
+
+    for (let status of ["enabled", "disabled"]) {
+      let section = getSection(doc, `${type}-${status}-section`);
+      let el = section?.querySelector(".list-section-heading");
+      isnot(el, null, `Should have ${status} heading for ${type} section`);
+      is(
+        el && doc.l10n.getAttributes(el).id,
+        win.getL10nIdMapping(`${type}-${status}-heading`),
+        `Should have correct ${status} heading for ${type} section`
+      );
+    }
+
+    await closeView(win);
+  }
+});
+
+add_task(async function testDisabledDimming() {
+  const id = "disabled@mochi.test";
+  let extension = ExtensionTestUtils.loadExtension({
+    manifest: {
+      name: "Disable me",
+      browser_specific_settings: { gecko: { id } },
+    },
+    useAddonManager: "temporary",
+  });
+  await extension.startup();
+
+  let addon = await AddonManager.getAddonByID(id);
+
+  let win = await loadInitialView("extension");
+  let doc = win.document;
+  let pageHeader = doc.querySelector("addon-page-header");
+
+  // Ensure there's no focus on the list.
+  EventUtils.synthesizeMouseAtCenter(pageHeader, {}, win);
+
+  const checkOpacity = (card, expected, msg) => {
+    let { opacity } = card.ownerGlobal.getComputedStyle(card.firstElementChild);
+    let normalize = val => Math.floor(val * 10);
+    is(normalize(opacity), normalize(expected), msg);
+  };
+  const waitForTransition = card =>
+    BrowserTestUtils.waitForEvent(
+      card.firstElementChild,
+      "transitionend",
+      e => e.propertyName === "opacity" && e.target.classList.contains("card")
+    );
+
+  let card = getCardByAddonId(doc, id);
+  checkOpacity(card, "1", "The opacity is 1 when enabled");
+
+  // Disable the add-on, check again.
+  let list = doc.querySelector("addon-list");
+  let moved = BrowserTestUtils.waitForEvent(list, "move");
+  await addon.disable();
+  await moved;
+
+  let disabledSection = getSection(doc, "extension-disabled-section");
+  is(card.parentNode, disabledSection, "The card is in the disabled section");
+  checkOpacity(card, "0.6", "The opacity is dimmed when disabled");
+
+  // Click on the menu button, this should un-dim the card.
+  let transitionEnded = waitForTransition(card);
+  let moreOptionsButton = card.querySelector(".more-options-button");
+  EventUtils.synthesizeMouseAtCenter(moreOptionsButton, {}, win);
+  await transitionEnded;
+  checkOpacity(card, "1", "The opacity is 1 when the menu is open");
+
+  // Close the menu, opacity should return.
+  transitionEnded = waitForTransition(card);
+  EventUtils.synthesizeMouseAtCenter(pageHeader, {}, win);
+  await transitionEnded;
+  checkOpacity(card, "0.6", "The card is dimmed again");
+
+  await closeView(win);
+  await extension.unload();
+});
+
+add_task(async function testEmptyMessage() {
+  let tests = [
+    {
+      type: "extension",
+      message: "Get extensions and themes on ",
+    },
+    {
+      type: "theme",
+      message: "Get extensions and themes on ",
+    },
+    {
+      type: "plugin",
+      message: "Get extensions and themes on ",
+    },
+    {
+      type: "locale",
+      message: "Get language packs on ",
+    },
+    {
+      type: "dictionary",
+      message: "Get dictionaries on ",
+    },
+  ];
+
+  for (let test of tests) {
+    let win = await loadInitialView(test.type);
+    let doc = win.document;
+    let enabledSection = getSection(doc, `${test.type}-enabled-section`);
+    let disabledSection = getSection(doc, `${test.type}-disabled-section`);
+    const message = doc.querySelector("#empty-addons-message");
+
+    // Test if the correct locale has been applied.
+    ok(
+      message.textContent.startsWith(test.message),
+      `View ${test.type} has correct empty list message`
+    );
+
+    // With at least one enabled/disabled add-on (see testSectionHeadingKeys),
+    // the message is hidden.
+    is_element_hidden(message, "Empty addons message hidden");
+
+    // The test runner (Mochitest) relies on add-ons that should not be removed.
+    // Simulate the scenario of zero add-ons by clearing all rendered sections.
+    while (enabledSection.firstChild) {
+      enabledSection.firstChild.remove();
+    }
+
+    while (disabledSection.firstChild) {
+      disabledSection.firstChild.remove();
+    }
+
+    // Message should now be displayed
+    is_element_visible(message, "Empty addons message visible");
+
+    await closeView(win);
+  }
+});
