@@ -12,8 +12,6 @@ const BASE_URL = "http://example.com";
 const PREF_GETADDONS_CACHE_ENABLED = "extensions.getAddons.cache.enabled";
 const PREF_GETADDONS_CACHE_TYPES = "extensions.getAddons.cache.types";
 const GETADDONS_RESULTS = BASE_URL + "/data/test_AddonRepository_cache.json";
-const COMPAT_RESULTS =
-  BASE_URL + "/data/test_AddonRepository_cache_compat.json";
 const EMPTY_RESULT = BASE_URL + "/data/test_AddonRepository_empty.json";
 const FAILED_RESULT = BASE_URL + "/data/test_AddonRepository_fail.json";
 
@@ -39,7 +37,7 @@ const ADDONS = [
         page: "options.html",
       },
 
-      applications: {
+      browser_specific_settings: {
         gecko: { id: "test_AddonRepository_1@tests.mozilla.org" },
       },
     },
@@ -49,7 +47,7 @@ const ADDONS = [
       name: "XPI Add-on 2",
       version: "1.2",
       theme: {},
-      applications: {
+      browser_specific_settings: {
         gecko: { id: "test_AddonRepository_2@tests.mozilla.org" },
       },
     },
@@ -62,7 +60,7 @@ const ADDONS = [
         32: "icon.png",
       },
       theme: {},
-      applications: {
+      browser_specific_settings: {
         gecko: { id: "test_AddonRepository_3@tests.mozilla.org" },
       },
     },
@@ -72,7 +70,9 @@ const ADDONS = [
   },
 ];
 
-const ADDON_IDS = ADDONS.map(addon => addon.manifest.applications.gecko.id);
+const ADDON_IDS = ADDONS.map(
+  addon => addon.manifest.browser_specific_settings.gecko.id
+);
 const ADDON_FILES = ADDONS.map(addon =>
   AddonTestUtils.createTempWebExtensionFile(addon)
 );
@@ -137,7 +137,7 @@ const REPOSITORY_ADDONS = [
     ],
     description: "Repo Add-on 1 - Description\nSecond line",
     fullDescription: "Repo Add-on 1 - Full Description & some extra",
-    icons: { "32": BASE_URL + "/repo/1/icon.png" },
+    icons: { 32: BASE_URL + "/repo/1/icon.png" },
     homepageURL: BASE_URL + "/repo/1/homepage.html",
     supportURL: BASE_URL + "/repo/1/support.html",
     averageRating: 1,
@@ -163,7 +163,7 @@ const REPOSITORY_ADDONS = [
     ],
     description: "Repo Add-on 2 - Description",
     fullDescription: "Repo Add-on 2 - Full Description",
-    icons: { "32": BASE_URL + "/repo/2/icon.png" },
+    icons: { 32: BASE_URL + "/repo/2/icon.png" },
     screenshots: [
       {
         url: BASE_URL + "/repo/2/firstFull.png",
@@ -189,7 +189,7 @@ const REPOSITORY_ADDONS = [
     type: "theme",
     name: "Repo Add-on 3",
     version: "2.3",
-    icons: { "32": BASE_URL + "/repo/3/icon.png" },
+    icons: { 32: BASE_URL + "/repo/3/icon.png" },
     screenshots: [
       {
         url: BASE_URL + "/repo/3/firstFull.png",
@@ -219,7 +219,7 @@ const WITHOUT_CACHE = [
     authors: [{ name: "XPI Add-on 1 - Author" }],
     description: "XPI Add-on 1 - Description",
     get icons() {
-      return { "32": get_subfile_uri(ADDON_IDS[0], "icon.png") };
+      return { 32: get_subfile_uri(ADDON_IDS[0], "icon.png") };
     },
     homepageURL: `${BASE_URL}/xpi/1/homepage.html`,
     get optionsURL() {
@@ -241,7 +241,7 @@ const WITHOUT_CACHE = [
     name: "XPI Add-on 3",
     version: "1.3",
     get icons() {
-      return { "32": get_subfile_uri(ADDON_IDS[2], "icon.png") };
+      return { 32: get_subfile_uri(ADDON_IDS[2], "icon.png") };
     },
     screenshots: [
       {
@@ -274,7 +274,7 @@ const WITH_CACHE = [
     description: "XPI Add-on 1 - Description",
     fullDescription: "Repo Add-on 1 - Full Description & some extra",
     get icons() {
-      return { "32": get_subfile_uri(ADDON_IDS[0], "icon.png") };
+      return { 32: get_subfile_uri(ADDON_IDS[0], "icon.png") };
     },
     homepageURL: BASE_URL + "/xpi/1/homepage.html",
     supportURL: BASE_URL + "/repo/1/support.html",
@@ -304,7 +304,7 @@ const WITH_CACHE = [
     ],
     description: "Repo Add-on 2 - Description",
     fullDescription: "Repo Add-on 2 - Full Description",
-    icons: { "32": BASE_URL + "/repo/2/icon.png" },
+    icons: { 32: BASE_URL + "/repo/2/icon.png" },
     screenshots: [
       {
         url: BASE_URL + "/repo/2/firstFull.png",
@@ -334,7 +334,7 @@ const WITH_CACHE = [
       return get_subfile_uri(ADDON_IDS[2], "icon.png");
     },
     get icons() {
-      return { "32": get_subfile_uri(ADDON_IDS[2], "icon.png") };
+      return { 32: get_subfile_uri(ADDON_IDS[2], "icon.png") };
     },
     screenshots: [
       {
@@ -372,7 +372,7 @@ const WITH_EXTENSION_CACHE = [
     description: "XPI Add-on 1 - Description",
     fullDescription: "Repo Add-on 1 - Full Description & some extra",
     get icons() {
-      return { "32": get_subfile_uri(ADDON_IDS[0], "icon.png") };
+      return { 32: get_subfile_uri(ADDON_IDS[0], "icon.png") };
     },
     homepageURL: BASE_URL + "/xpi/1/homepage.html",
     supportURL: BASE_URL + "/repo/1/support.html",
@@ -402,7 +402,7 @@ const WITH_EXTENSION_CACHE = [
       return get_subfile_uri(ADDON_IDS[2], "icon.png");
     },
     get icons() {
-      return { "32": get_subfile_uri(ADDON_IDS[2], "icon.png") };
+      return { 32: get_subfile_uri(ADDON_IDS[2], "icon.png") };
     },
     screenshots: [
       {
@@ -436,7 +436,7 @@ function check_results(aActualAddons, aExpectedAddons, aFromRepository) {
 
   // Separately test updateDate (it should only be equal to the
   // REPOSITORY values if it is from the repository)
-  aActualAddons.forEach(function(aActualAddon) {
+  aActualAddons.forEach(function (aActualAddon) {
     if (aActualAddon.updateDate) {
       let time = aActualAddon.updateDate.getTime();
       Assert.equal(time === 1000 * REPOSITORY_UPDATEDATE, aFromRepository);
@@ -470,17 +470,18 @@ function check_cache(aExpectedToFind, aExpectedImmediately) {
         let expected = aExpectedToFind[i] ? REPOSITORY_ADDONS[i] : null;
         // can't Promise-wrap this because we're also testing whether the callback is
         // sync or async
-        AddonRepository.getCachedAddonByID(REPOSITORY_ADDONS[i].id, function(
-          aAddon
-        ) {
-          Assert.equal(immediatelyFound, aExpectedImmediately);
-          if (expected == null) {
-            Assert.equal(aAddon, null);
-          } else {
-            check_results([aAddon], [expected], true);
+        AddonRepository.getCachedAddonByID(
+          REPOSITORY_ADDONS[i].id,
+          function (aAddon) {
+            Assert.equal(immediatelyFound, aExpectedImmediately);
+            if (expected == null) {
+              Assert.equal(aAddon, null);
+            } else {
+              check_results([aAddon], [expected], true);
+            }
+            resolve();
           }
-          resolve();
-        });
+        );
         immediatelyFound = false;
       })
     );
@@ -540,7 +541,6 @@ add_task(async function run_test_3() {
   Assert.ok(gDBFile.exists());
   Services.prefs.setBoolPref(PREF_GETADDONS_CACHE_ENABLED, true);
   Services.prefs.setCharPref(PREF_GETADDONS_BYIDS, FAILED_RESULT);
-  Services.prefs.setCharPref(PREF_COMPAT_OVERRIDES, FAILED_RESULT);
 
   await AddonRepository.backgroundUpdateCheck();
   await check_initialized_cache([false, false, false]);
@@ -549,7 +549,6 @@ add_task(async function run_test_3() {
 // Tests repopulateCache when search returns no results
 add_task(async function run_test_4() {
   Services.prefs.setCharPref(PREF_GETADDONS_BYIDS, EMPTY_RESULT);
-  Services.prefs.setCharPref(PREF_COMPAT_OVERRIDES, FAILED_RESULT);
 
   await AddonRepository.backgroundUpdateCheck();
   await check_initialized_cache([false, false, false]);
@@ -558,7 +557,6 @@ add_task(async function run_test_4() {
 // Tests repopulateCache when search returns results
 add_task(async function run_test_5() {
   Services.prefs.setCharPref(PREF_GETADDONS_BYIDS, GETADDONS_RESULTS);
-  Services.prefs.setCharPref(PREF_COMPAT_OVERRIDES, COMPAT_RESULTS);
 
   await AddonRepository.backgroundUpdateCheck();
   await check_initialized_cache([true, true, true]);
@@ -662,7 +660,7 @@ add_task(async function run_test_13() {
   Assert.ok(gDBFile.exists());
   Services.prefs.setCharPref(PREF_GETADDONS_BYIDS, EMPTY_RESULT);
 
-  await AddonManagerInternal.backgroundUpdateCheck();
+  await AddonManagerPrivate.backgroundUpdateCheck();
   // Database should have been deleted
   Assert.ok(!gDBFile.exists());
 
@@ -675,7 +673,7 @@ add_task(async function run_test_13() {
 add_task(async function run_test_14() {
   Services.prefs.setBoolPref(PREF_GETADDONS_CACHE_ENABLED, true);
 
-  await AddonManagerInternal.backgroundUpdateCheck();
+  await AddonManagerPrivate.backgroundUpdateCheck();
   await AddonRepository.flush();
   Assert.ok(gDBFile.exists());
 
@@ -688,7 +686,7 @@ add_task(async function run_test_14() {
 add_task(async function run_test_15() {
   Services.prefs.setCharPref(PREF_GETADDONS_BYIDS, GETADDONS_RESULTS);
 
-  await AddonManagerInternal.backgroundUpdateCheck();
+  await AddonManagerPrivate.backgroundUpdateCheck();
   let aAddons = await promiseAddonsByIDs(ADDON_IDS);
   check_results(aAddons, WITH_CACHE);
 });
@@ -710,7 +708,7 @@ add_task(async function run_test_17() {
     "foo,bar,extension,baz"
   );
 
-  await AddonManagerInternal.backgroundUpdateCheck();
+  await AddonManagerPrivate.backgroundUpdateCheck();
   let aAddons = await promiseAddonsByIDs(ADDON_IDS);
   check_results(aAddons, WITH_EXTENSION_CACHE);
 });
