@@ -2,6 +2,9 @@
 // Test whether an install succeeds when authentication is required
 // This verifies bug 312473
 function test() {
+  // This test depends on InstallTrigger.install availability.
+  setInstallTriggerPrefs();
+
   // Turn off the authentication dialog blocking for this test.
   Services.prefs.setBoolPref(
     "network.auth.non-web-content-triggered-resources-http-auth-allow",
@@ -16,8 +19,11 @@ function test() {
 
   Services.prefs.setIntPref("network.auth.subresource-http-auth-allow", 2);
 
-  var pm = Services.perms;
-  pm.add(makeURI("http://example.com/"), "install", pm.ALLOW_ACTION);
+  PermissionTestUtils.add(
+    "http://example.com/",
+    "install",
+    Services.perms.ALLOW_ACTION
+  );
 
   var triggers = encodeURIComponent(
     JSON.stringify({
@@ -26,7 +32,7 @@ function test() {
     })
   );
   gBrowser.selectedTab = BrowserTestUtils.addTab(gBrowser);
-  BrowserTestUtils.loadURI(
+  BrowserTestUtils.loadURIString(
     gBrowser,
     TESTROOT + "installtrigger.html?" + triggers
   );
@@ -41,7 +47,7 @@ function download_failed(install) {
 }
 
 function install_ended(install, addon) {
-  install.cancel();
+  return addon.uninstall();
 }
 
 function finish_test(count) {
@@ -51,7 +57,7 @@ function finish_test(count) {
   );
   authMgr.clearAll();
 
-  Services.perms.remove(makeURI("http://example.com"), "install");
+  PermissionTestUtils.remove("http://example.com", "install");
 
   Services.prefs.clearUserPref(
     "network.auth.non-web-content-triggered-resources-http-auth-allow"
