@@ -2,16 +2,23 @@
 // Test whether setting a new property in InstallTrigger then persists to other
 // page loads
 add_task(async function test() {
+  await SpecialPowers.pushPrefEnv({
+    set: [
+      ["extensions.InstallTrigger.enabled", true],
+      ["extensions.InstallTriggerImpl.enabled", true],
+    ],
+  });
+
   await BrowserTestUtils.withNewTab(
     { gBrowser, url: TESTROOT + "enabled.html" },
-    async function(browser) {
-      await ContentTask.spawn(browser, null, () => {
-        content.wrappedJSObject.InstallTrigger.enabled.k = function() {};
+    async function (browser) {
+      await SpecialPowers.spawn(browser, [], () => {
+        content.wrappedJSObject.InstallTrigger.enabled.k = function () {};
       });
 
-      BrowserTestUtils.loadURI(browser, TESTROOT2 + "enabled.html");
+      BrowserTestUtils.loadURIString(browser, TESTROOT2 + "enabled.html");
       await BrowserTestUtils.browserLoaded(browser);
-      await ContentTask.spawn(browser, null, () => {
+      await SpecialPowers.spawn(browser, [], () => {
         is(
           content.wrappedJSObject.InstallTrigger.enabled.k,
           undefined,
